/**
 * Apache License
 * Version 2.0, January 2004
 * http://www.apache.org/licenses/
 *
 * TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *
 * 1. Definitions.
 *
 * "License" shall mean the terms and conditions for use, reproduction,
 * and distribution as defined by Sections 1 through 9 of this document.
 *
 * "Licensor" shall mean the copyright owner or entity authorized by
 * the copyright owner that is granting the License.
 *
 * "Legal Entity" shall mean the union of the acting entity and all
 * other entities that control, are controlled by, or are under common
 * control with that entity. For the purposes of this definition,
 * "control" means (i) the power, direct or indirect, to cause the
 * direction or management of such entity, whether by contract or
 * otherwise, or (ii) ownership of fifty percent (50%) or more of the
 * outstanding shares, or (iii) beneficial ownership of such entity.
 *
 * "You" (or "Your") shall mean an individual or Legal Entity
 * exercising permissions granted by this License.
 *
 * "Source" form shall mean the preferred form for making modifications,
 * including but not limited to software source code, documentation
 * source, and configuration files.
 *
 * "Object" form shall mean any form resulting from mechanical
 * transformation or translation of a Source form, including but
 * not limited to compiled object code, generated documentation,
 * and conversions to other media types.
 *
 * "Work" shall mean the work of authorship, whether in Source or
 * Object form, made available under the License, as indicated by a
 * copyright notice that is included in or attached to the work
 * (an example is provided in the Appendix below).
 *
 * "Derivative Works" shall mean any work, whether in Source or Object
 * form, that is based on (or derived from) the Work and for which the
 * editorial revisions, annotations, elaborations, or other modifications
 * represent, as a whole, an original work of authorship. For the purposes
 * of this License, Derivative Works shall not include works that remain
 * separable from, or merely link (or bind by name) to the interfaces of,
 * the Work and Derivative Works thereof.
 *
 * "Contribution" shall mean any work of authorship, including
 * the original version of the Work and any modifications or additions
 * to that Work or Derivative Works thereof, that is intentionally
 * submitted to Licensor for inclusion in the Work by the copyright owner
 * or by an individual or Legal Entity authorized to submit on behalf of
 * the copyright owner. For the purposes of this definition, "submitted"
 * means any form of electronic, verbal, or written communication sent
 * to the Licensor or its representatives, including but not limited to
 * communication on electronic mailing lists, source code control systems,
 * and issue tracking systems that are managed by, or on behalf of, the
 * Licensor for the purpose of discussing and improving the Work, but
 * excluding communication that is conspicuously marked or otherwise
 * designated in writing by the copyright owner as "Not a Contribution."
 *
 * "Contributor" shall mean Licensor and any individual or Legal Entity
 * on behalf of whom a Contribution has been received by Licensor and
 * subsequently incorporated within the Work.
 *
 * 2. Grant of Copyright License. Subject to the terms and conditions of
 * this License, each Contributor hereby grants to You a perpetual,
 * worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 * copyright license to reproduce, prepare Derivative Works of,
 * publicly display, publicly perform, sublicense, and distribute the
 * Work and such Derivative Works in Source or Object form.
 *
 * 3. Grant of Patent License. Subject to the terms and conditions of
 * this License, each Contributor hereby grants to You a perpetual,
 * worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 * (except as stated in this section) patent license to make, have made,
 * use, offer to sell, sell, import, and otherwise transfer the Work,
 * where such license applies only to those patent claims licensable
 * by such Contributor that are necessarily infringed by their
 * Contribution(s) alone or by combination of their Contribution(s)
 * with the Work to which such Contribution(s) was submitted. If You
 * institute patent litigation against any entity (including a
 * cross-claim or counterclaim in a lawsuit) alleging that the Work
 * or a Contribution incorporated within the Work constitutes direct
 * or contributory patent infringement, then any patent licenses
 * granted to You under this License for that Work shall terminate
 * as of the date such litigation is filed.
 *
 * 4. Redistribution. You may reproduce and distribute copies of the
 * Work or Derivative Works thereof in any medium, with or without
 * modifications, and in Source or Object form, provided that You
 * meet the following conditions:
 *
 * (a) You must give any other recipients of the Work or
 * Derivative Works a copy of this License; and
 *
 * (b) You must cause any modified files to carry prominent notices
 * stating that You changed the files; and
 *
 * (c) You must retain, in the Source form of any Derivative Works
 * that You distribute, all copyright, patent, trademark, and
 * attribution notices from the Source form of the Work,
 * excluding those notices that do not pertain to any part of
 * the Derivative Works; and
 *
 * (d) If the Work includes a "NOTICE" text file as part of its
 * distribution, then any Derivative Works that You distribute must
 * include a readable copy of the attribution notices contained
 * within such NOTICE file, excluding those notices that do not
 * pertain to any part of the Derivative Works, in at least one
 * of the following places: within a NOTICE text file distributed
 * as part of the Derivative Works; within the Source form or
 * documentation, if provided along with the Derivative Works; or,
 * within a display generated by the Derivative Works, if and
 * wherever such third-party notices normally appear. The contents
 * of the NOTICE file are for informational purposes only and
 * do not modify the License. You may add Your own attribution
 * notices within Derivative Works that You distribute, alongside
 * or as an addendum to the NOTICE text from the Work, provided
 * that such additional attribution notices cannot be construed
 * as modifying the License.
 *
 * You may add Your own copyright statement to Your modifications and
 * may provide additional or different license terms and conditions
 * for use, reproduction, or distribution of Your modifications, or
 * for any such Derivative Works as a whole, provided Your use,
 * reproduction, and distribution of the Work otherwise complies with
 * the conditions stated in this License.
 *
 * 5. Submission of Contributions. Unless You explicitly state otherwise,
 * any Contribution intentionally submitted for inclusion in the Work
 * by You to the Licensor shall be under the terms and conditions of
 * this License, without any additional terms or conditions.
 * Notwithstanding the above, nothing herein shall supersede or modify
 * the terms of any separate license agreement you may have executed
 * with Licensor regarding such Contributions.
 *
 * 6. Trademarks. This License does not grant permission to use the trade
 * names, trademarks, service marks, or product names of the Licensor,
 * except as required for reasonable and customary use in describing the
 * origin of the Work and reproducing the content of the NOTICE file.
 *
 * 7. Disclaimer of Warranty. Unless required by applicable law or
 * agreed to in writing, Licensor provides the Work (and each
 * Contributor provides its Contributions) on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied, including, without limitation, any warranties or conditions
 * of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 * PARTICULAR PURPOSE. You are solely responsible for determining the
 * appropriateness of using or redistributing the Work and assume any
 * risks associated with Your exercise of permissions under this License.
 *
 * 8. Limitation of Liability. In no event and under no legal theory,
 * whether in tort (including negligence), contract, or otherwise,
 * unless required by applicable law (such as deliberate and grossly
 * negligent acts) or agreed to in writing, shall any Contributor be
 * liable to You for damages, including any direct, indirect, special,
 * incidental, or consequential damages of any character arising as a
 * result of this License or out of the use or inability to use the
 * Work (including but not limited to damages for loss of goodwill,
 * work stoppage, computer failure or malfunction, or any and all
 * other commercial damages or losses), even if such Contributor
 * has been advised of the possibility of such damages.
 *
 * 9. Accepting Warranty or Additional Liability. While redistributing
 * the Work or Derivative Works thereof, You may choose to offer,
 * and charge a fee for, acceptance of support, warranty, indemnity,
 * or other liability obligations and/or rights consistent with this
 * License. However, in accepting such obligations, You may act only
 * on Your own behalf and on Your sole responsibility, not on behalf
 * of any other Contributor, and only if You agree to indemnify,
 * defend, and hold each Contributor harmless for any liability
 * incurred by, or claims asserted against, such Contributor by reason
 * of your accepting any such warranty or additional liability.
 *
 * END OF TERMS AND CONDITIONS
 *
 * APPENDIX: How to apply the Apache License to your work.
 *
 * To apply the Apache License to your work, attach the following
 * boilerplate notice, with the fields enclosed by brackets "[]"
 * replaced with your own identifying information. (Don't include
 * the brackets!)  The text should be enclosed in the appropriate
 * comment syntax for the file format. We also recommend that a
 * file or class name and description of purpose be included on the
 * same "printed page" as the copyright notice for easier
 * identification within third-party archives.
 *
 * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

!function(e,t){"object"==typeof exports&&"undefined"!=typeof module?module.exports=t():"function"==typeof define&&define.amd?define(t):(e="undefined"!=typeof globalThis?globalThis:e||self).JSpider=t()}(this,(function(){"use strict";function e(e){for(var t=arguments.length,r=new Array(t>1?t-1:0),n=1;n<t;n++)r[n-1]=arguments[n];throw new Error("number"==typeof e?"[MobX] minified error nr: "+e+(r.length?" "+r.map(String).join(","):"")+". Find the full error at: https://github.com/mobxjs/mobx/blob/main/packages/mobx/src/errors.ts":"[MobX] "+e)}var t={};function r(){return"undefined"!=typeof globalThis?globalThis:"undefined"!=typeof window?window:"undefined"!=typeof global?global:"undefined"!=typeof self?self:t}var n=Object.assign,i=Object.getOwnPropertyDescriptor,o=Object.defineProperty,a=Object.prototype,s=[];Object.freeze(s);var u={};Object.freeze(u);var c="undefined"!=typeof Proxy,l=Object.toString();function f(){c||e("Proxy not available")}function p(e){var t=!1;return function(){if(!t)return t=!0,e.apply(this,arguments)}}var h=function(){};function b(e){return"function"==typeof e}function d(e){switch(typeof e){case"string":case"symbol":case"number":return!0}return!1}function v(e){return null!==e&&"object"==typeof e}function y(e){var t;if(!v(e))return!1;var r=Object.getPrototypeOf(e);return null==r||(null==(t=r.constructor)?void 0:t.toString())===l}function g(e){var t=null==e?void 0:e.constructor;return!!t&&("GeneratorFunction"===t.name||"GeneratorFunction"===t.displayName)}function m(e,t,r){o(e,t,{enumerable:!1,writable:!0,configurable:!0,value:r})}function _(e,t,r){o(e,t,{enumerable:!1,writable:!1,configurable:!0,value:r})}function w(e,t){var r="isMobX"+e;return t.prototype[r]=!0,function(e){return v(e)&&!0===e[r]}}function O(e){return e instanceof Map}function j(e){return e instanceof Set}var P=void 0!==Object.getOwnPropertySymbols;var S="undefined"!=typeof Reflect&&Reflect.ownKeys?Reflect.ownKeys:P?function(e){return Object.getOwnPropertyNames(e).concat(Object.getOwnPropertySymbols(e))}:Object.getOwnPropertyNames;function A(e){return null===e?null:"object"==typeof e?""+e:e}function x(e,t){return a.hasOwnProperty.call(e,t)}var E=Object.getOwnPropertyDescriptors||function(e){var t={};return S(e).forEach((function(r){t[r]=i(e,r)})),t};function T(e,t){for(var r=0;r<t.length;r++){var n=t[r];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function I(e,t,r){return t&&T(e.prototype,t),r&&T(e,r),e}function C(){return(C=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var r=arguments[t];for(var n in r)Object.prototype.hasOwnProperty.call(r,n)&&(e[n]=r[n])}return e}).apply(this,arguments)}function k(e,t){e.prototype=Object.create(t.prototype),e.prototype.constructor=e,e.__proto__=t}function D(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function N(e,t){(null==t||t>e.length)&&(t=e.length);for(var r=0,n=new Array(t);r<t;r++)n[r]=e[r];return n}function V(e,t){var r;if("undefined"==typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(r=function(e,t){if(e){if("string"==typeof e)return N(e,t);var r=Object.prototype.toString.call(e).slice(8,-1);return"Object"===r&&e.constructor&&(r=e.constructor.name),"Map"===r||"Set"===r?Array.from(e):"Arguments"===r||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(r)?N(e,t):void 0}}(e))||t&&e&&"number"==typeof e.length){r&&(e=r);var n=0;return function(){return n>=e.length?{done:!0}:{done:!1,value:e[n++]}}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}return(r=e[Symbol.iterator]()).next.bind(r)}var R=Symbol("mobx-stored-annotations");function U(e){return Object.assign((function(t,r){M(t,r,e)}),e)}function M(e,t,r){x(e,R)||m(e,R,C({},e[R])),function(e){return e.annotationType_===K}(r)||(e[R][t]=r)}var z=Symbol("mobx administration"),F=function(){function e(e){void 0===e&&(e="Atom"),this.name_=void 0,this.isPendingUnobservation_=!1,this.isBeingObserved_=!1,this.observers_=new Set,this.diffValue_=0,this.lastAccessedBy_=0,this.lowestObserverState_=ze.NOT_TRACKING_,this.onBOL=void 0,this.onBUOL=void 0,this.name_=e}var t=e.prototype;return t.onBO=function(){this.onBOL&&this.onBOL.forEach((function(e){return e()}))},t.onBUO=function(){this.onBUOL&&this.onBUOL.forEach((function(e){return e()}))},t.reportObserved=function(){return ut(this)},t.reportChanged=function(){at(),ct(this),st()},t.toString=function(){return this.name_},e}(),L=w("Atom",F);function $(e,t,r){void 0===t&&(t=h),void 0===r&&(r=h);var n,i=new F(e);return t!==h&&Ct(Tt,i,t,n),r!==h&&It(i,r),i}var H={identity:function(e,t){return e===t},structural:function(e,t){return Wr(e,t)},default:function(e,t){return Object.is(e,t)},shallow:function(e,t){return Wr(e,t,1)}};function B(e,t,r){return Lt(e)?e:Array.isArray(e)?Oe.array(e,{name:r}):y(e)?Oe.object(e,void 0,{name:r}):O(e)?Oe.map(e,{name:r}):j(e)?Oe.set(e,{name:r}):"function"!=typeof e||St(e)||Mt(e)?e:g(e)?Rt(e):jt(r,e)}function G(e){return e}var K="override";function q(e,t){return{annotationType_:e,options_:t,make_:W,extend_:Y}}function W(e,t,r,n){var i;if(null==(i=this.options_)?void 0:i.bound)return null===this.extend_(e,t,r,!1)?0:1;if(n===e.target_)return null===this.extend_(e,t,r,!1)?0:2;if(St(r.value))return 1;var a=Q(e,this,t,r,!1);return o(n,t,a),2}function Y(e,t,r,n){var i=Q(e,this,t,r);return e.defineProperty_(t,i,n)}function Q(e,t,r,n,i){var o,a,s,u,c,l;void 0===i&&(i=rt.safeDescriptors),l=n,t.annotationType_,l.value;var f,p=n.value;(null==(o=t.options_)?void 0:o.bound)&&(p=p.bind(null!=(f=e.proxy_)?f:e.target_));return{value:ke(null!=(a=null==(s=t.options_)?void 0:s.name)?a:r.toString(),p,null!=(u=null==(c=t.options_)?void 0:c.autoAction)&&u),configurable:!i||e.isPlainObject_,enumerable:!1,writable:!i}}function X(e,t){return{annotationType_:e,options_:t,make_:Z,extend_:J}}function Z(e,t,r,n){var i;if(n===e.target_)return null===this.extend_(e,t,r,!1)?0:2;if((null==(i=this.options_)?void 0:i.bound)&&!Mt(e.target_[t])&&null===this.extend_(e,t,r,!1))return 0;if(Mt(r.value))return 1;var a=ee(e,this,t,r,!1,!1);return o(n,t,a),2}function J(e,t,r,n){var i,o=ee(e,this,t,r,null==(i=this.options_)?void 0:i.bound);return e.defineProperty_(t,o,n)}function ee(e,t,r,n,i,o){var a;void 0===o&&(o=rt.safeDescriptors),a=n,t.annotationType_,a.value;var s,u=n.value;i&&(u=u.bind(null!=(s=e.proxy_)?s:e.target_));return{value:Rt(u),configurable:!o||e.isPlainObject_,enumerable:!1,writable:!o}}function te(e,t){return{annotationType_:e,options_:t,make_:re,extend_:ne}}function re(e,t,r){return null===this.extend_(e,t,r,!1)?0:1}function ne(e,t,r,n){var i;return i=r,this.annotationType_,i.get,e.defineComputedProperty_(t,C({},this.options_,{get:r.get,set:r.set}),n)}function ie(e,t){return{annotationType_:e,options_:t,make_:oe,extend_:ae}}function oe(e,t,r){return null===this.extend_(e,t,r,!1)?0:1}function ae(e,t,r,n){var i,o;return this.annotationType_,e.defineObservableProperty_(t,r.value,null!=(i=null==(o=this.options_)?void 0:o.enhancer)?i:B,n)}var se=ue();function ue(e){return{annotationType_:"true",options_:e,make_:ce,extend_:le}}function ce(e,t,r,n){var i,a,s,u;if(r.get)return Ae.make_(e,t,r,n);if(r.set){var c=ke(t.toString(),r.set);return n===e.target_?null===e.defineProperty_(t,{configurable:!rt.safeDescriptors||e.isPlainObject_,set:c})?0:2:(o(n,t,{configurable:!0,set:c}),2)}if(n!==e.target_&&"function"==typeof r.value)return g(r.value)?((null==(u=this.options_)?void 0:u.autoBind)?Rt.bound:Rt).make_(e,t,r,n):((null==(s=this.options_)?void 0:s.autoBind)?jt.bound:jt).make_(e,t,r,n);var l,f=!1===(null==(i=this.options_)?void 0:i.deep)?Oe.ref:Oe;"function"==typeof r.value&&(null==(a=this.options_)?void 0:a.autoBind)&&(r.value=r.value.bind(null!=(l=e.proxy_)?l:e.target_));return f.make_(e,t,r,n)}function le(e,t,r,n){var i,o,a;if(r.get)return Ae.extend_(e,t,r,n);if(r.set)return e.defineProperty_(t,{configurable:!rt.safeDescriptors||e.isPlainObject_,set:ke(t.toString(),r.set)},n);"function"==typeof r.value&&(null==(i=this.options_)?void 0:i.autoBind)&&(r.value=r.value.bind(null!=(a=e.proxy_)?a:e.target_));return(!1===(null==(o=this.options_)?void 0:o.deep)?Oe.ref:Oe).extend_(e,t,r,n)}var fe={deep:!0,name:void 0,defaultDecorator:void 0,proxy:!0};function pe(e){return e||fe}Object.freeze(fe);var he=ie("observable"),be=ie("observable.ref",{enhancer:G}),de=ie("observable.shallow",{enhancer:function(e,t,r){return null==e||Nr(e)||yr(e)||Pr(e)||xr(e)?e:Array.isArray(e)?Oe.array(e,{name:r,deep:!1}):y(e)?Oe.object(e,void 0,{name:r,deep:!1}):O(e)?Oe.map(e,{name:r,deep:!1}):j(e)?Oe.set(e,{name:r,deep:!1}):void 0}}),ve=ie("observable.struct",{enhancer:function(e,t){return Wr(e,t)?t:e}}),ye=U(he);function ge(e){return!0===e.deep?B:!1===e.deep?G:(t=e.defaultDecorator)&&null!=(r=null==(n=t.options_)?void 0:n.enhancer)?r:B;var t,r,n}function me(e,t,r){if(!d(t))return Lt(e)?e:y(e)?Oe.object(e,t,r):Array.isArray(e)?Oe.array(e,t):O(e)?Oe.map(e,t):j(e)?Oe.set(e,t):"object"==typeof e&&null!==e?e:Oe.box(e,t);M(e,t,he)}Object.assign(me,ye);var _e,we,Oe=n(me,{box:function(e,t){var r=pe(t);return new Ue(e,ge(r),r.name,!0,r.equals)},array:function(e,t){var r=pe(t);return(!1===rt.useProxies||!1===r.proxy?Hr:ur)(e,ge(r),r.name)},map:function(e,t){var r=pe(t);return new jr(e,ge(r),r.name)},set:function(e,t){var r=pe(t);return new Ar(e,ge(r),r.name)},object:function(e,t,r){return function(e,t,r,n){var i=E(t),o=Cr(e,n)[z];at();try{S(i).forEach((function(e){o.extend_(e,i[e],!r||(!(e in r)||r[e]))}))}finally{st()}return e}(!1===rt.useProxies||!1===(null==r?void 0:r.proxy)?Cr({},r):function(e,t){var r,n;return f(),e=Cr(e,t),null!=(n=(r=e[z]).proxy_)?n:r.proxy_=new Proxy(e,Qt)}({},r),e,t)},ref:U(be),shallow:U(de),deep:ye,struct:U(ve)}),je="computed",Pe=te(je),Se=te("computed.struct",{equals:H.structural}),Ae=function(e,t){if(d(t))return M(e,t,Pe);if(y(e))return U(te(je,e));var r=y(t)?t:{};return r.get=e,r.name||(r.name=e.name||""),new Le(r)};Object.assign(Ae,Pe),Ae.struct=U(Se);var xe,Ee=0,Te=1,Ie=null!=(_e=null==(we=i((function(){}),"name"))?void 0:we.configurable)&&_e,Ce={value:"action",configurable:!0,writable:!1,enumerable:!1};function ke(e,t,r,n){function i(){return De(e,r,t,n||this,arguments)}return void 0===r&&(r=!1),i.isMobxAction=!0,Ie&&(Ce.value=e,Object.defineProperty(i,"name",Ce)),i}function De(t,r,n,i,o){var a=function(e,t,r,n){var i=!1,o=0,a=rt.trackingDerivation,s=!t||!a;at();var u=rt.allowStateChanges;s&&(Ye(),u=Ne(!0));var c=Xe(!0),l={runAsAction_:s,prevDerivation_:a,prevAllowStateChanges_:u,prevAllowStateReads_:c,notifySpy_:i,startTime_:o,actionId_:Te++,parentActionId_:Ee};return Ee=l.actionId_,l}(0,r);try{return n.apply(i,o)}catch(e){throw a.error_=e,e}finally{!function(t){Ee!==t.actionId_&&e(30);Ee=t.parentActionId_,void 0!==t.error_&&(rt.suppressReactionErrors=!0);Ve(t.prevAllowStateChanges_),Ze(t.prevAllowStateReads_),st(),t.runAsAction_&&Qe(t.prevDerivation_);rt.suppressReactionErrors=!1}(a)}}function Ne(e){var t=rt.allowStateChanges;return rt.allowStateChanges=e,t}function Ve(e){rt.allowStateChanges=e}xe=Symbol.toPrimitive;var Re,Ue=function(e){function t(t,r,n,i,o){var a;return void 0===n&&(n="ObservableValue"),void 0===o&&(o=H.default),(a=e.call(this,n)||this).enhancer=void 0,a.name_=void 0,a.equals=void 0,a.hasUnreportedChange_=!1,a.interceptors_=void 0,a.changeListeners_=void 0,a.value_=void 0,a.dehancer=void 0,a.enhancer=r,a.name_=n,a.equals=o,a.value_=r(t,void 0,n),a}k(t,e);var r=t.prototype;return r.dehanceValue=function(e){return void 0!==this.dehancer?this.dehancer(e):e},r.set=function(e){this.value_,(e=this.prepareNewValue_(e))!==rt.UNCHANGED&&this.setNewValue_(e)},r.prepareNewValue_=function(e){if(Xt(this)){var t=Jt(this,{object:this,type:or,newValue:e});if(!t)return rt.UNCHANGED;e=t.newValue}return e=this.enhancer(e,this.value_,this.name_),this.equals(this.value_,e)?rt.UNCHANGED:e},r.setNewValue_=function(e){var t=this.value_;this.value_=e,this.reportChanged(),er(this)&&rr(this,{type:or,object:this,newValue:e,oldValue:t})},r.get=function(){return this.reportObserved(),this.dehanceValue(this.value_)},r.intercept_=function(e){return Zt(this,e)},r.observe_=function(e,t){return t&&e({observableKind:"value",debugObjectName:this.name_,object:this,type:or,newValue:this.value_,oldValue:void 0}),tr(this,e)},r.raw=function(){return this.value_},r.toJSON=function(){return this.get()},r.toString=function(){return this.name_+"["+this.value_+"]"},r.valueOf=function(){return A(this.get())},r[xe]=function(){return this.valueOf()},t}(F),Me=w("ObservableValue",Ue);Re=Symbol.toPrimitive;var ze,Fe,Le=function(){function t(t){this.dependenciesState_=ze.NOT_TRACKING_,this.observing_=[],this.newObserving_=null,this.isBeingObserved_=!1,this.isPendingUnobservation_=!1,this.observers_=new Set,this.diffValue_=0,this.runId_=0,this.lastAccessedBy_=0,this.lowestObserverState_=ze.UP_TO_DATE_,this.unboundDepsCount_=0,this.value_=new He(null),this.name_=void 0,this.triggeredBy_=void 0,this.isComputing_=!1,this.isRunningSetter_=!1,this.derivation=void 0,this.setter_=void 0,this.isTracing_=Fe.NONE,this.scope_=void 0,this.equals_=void 0,this.requiresReaction_=void 0,this.keepAlive_=void 0,this.onBOL=void 0,this.onBUOL=void 0,t.get||e(31),this.derivation=t.get,this.name_=t.name||"ComputedValue",t.set&&(this.setter_=ke("ComputedValue-setter",t.set)),this.equals_=t.equals||(t.compareStructural||t.struct?H.structural:H.default),this.scope_=t.context,this.requiresReaction_=!!t.requiresReaction,this.keepAlive_=!!t.keepAlive}var r=t.prototype;return r.onBecomeStale_=function(){!function(e){if(e.lowestObserverState_!==ze.UP_TO_DATE_)return;e.lowestObserverState_=ze.POSSIBLY_STALE_,e.observers_.forEach((function(e){e.dependenciesState_===ze.UP_TO_DATE_&&(e.dependenciesState_=ze.POSSIBLY_STALE_,e.onBecomeStale_())}))}(this)},r.onBO=function(){this.onBOL&&this.onBOL.forEach((function(e){return e()}))},r.onBUO=function(){this.onBUOL&&this.onBUOL.forEach((function(e){return e()}))},r.get=function(){if(this.isComputing_&&e(32,this.name_,this.derivation),0!==rt.inBatch||0!==this.observers_.size||this.keepAlive_){if(ut(this),Ge(this)){var t=rt.trackingContext;this.keepAlive_&&!t&&(rt.trackingContext=this),this.trackAndCompute()&&function(e){if(e.lowestObserverState_===ze.STALE_)return;e.lowestObserverState_=ze.STALE_,e.observers_.forEach((function(t){t.dependenciesState_===ze.POSSIBLY_STALE_?t.dependenciesState_=ze.STALE_:t.dependenciesState_===ze.UP_TO_DATE_&&(e.lowestObserverState_=ze.UP_TO_DATE_)}))}(this),rt.trackingContext=t}}else Ge(this)&&(this.warnAboutUntrackedRead_(),at(),this.value_=this.computeValue_(!1),st());var r=this.value_;if(Be(r))throw r.cause;return r},r.set=function(t){if(this.setter_){this.isRunningSetter_&&e(33,this.name_),this.isRunningSetter_=!0;try{this.setter_.call(this.scope_,t)}finally{this.isRunningSetter_=!1}}else e(34,this.name_)},r.trackAndCompute=function(){var e=this.value_,t=this.dependenciesState_===ze.NOT_TRACKING_,r=this.computeValue_(!0),n=t||Be(e)||Be(r)||!this.equals_(e,r);return n&&(this.value_=r),n},r.computeValue_=function(e){this.isComputing_=!0;var t,r=Ne(!1);if(e)t=Ke(this,this.derivation,this.scope_);else if(!0===rt.disableErrorBoundaries)t=this.derivation.call(this.scope_);else try{t=this.derivation.call(this.scope_)}catch(e){t=new He(e)}return Ve(r),this.isComputing_=!1,t},r.suspend_=function(){this.keepAlive_||(qe(this),this.value_=void 0)},r.observe_=function(e,t){var r=this,n=!0,i=void 0;return function(e,t){var r,n;void 0===t&&(t=u);var i,o=null!=(r=null==(n=t)?void 0:n.name)?r:"Autorun";if(t.scheduler||t.delay){var a=xt(t),s=!1;i=new lt(o,(function(){s||(s=!0,a((function(){s=!1,i.isDisposed_||i.track(c)})))}),t.onError,t.requiresObservable)}else i=new lt(o,(function(){this.track(c)}),t.onError,t.requiresObservable);function c(){e(i)}return i.schedule_(),i.getDisposer_()}((function(){var o=r.get();if(!n||t){var a=Ye();e({observableKind:"computed",debugObjectName:r.name_,type:or,object:r,newValue:o,oldValue:i}),Qe(a)}n=!1,i=o}))},r.warnAboutUntrackedRead_=function(){},r.toString=function(){return this.name_+"["+this.derivation.toString()+"]"},r.valueOf=function(){return A(this.get())},r[Re]=function(){return this.valueOf()},t}(),$e=w("ComputedValue",Le);!function(e){e[e.NOT_TRACKING_=-1]="NOT_TRACKING_",e[e.UP_TO_DATE_=0]="UP_TO_DATE_",e[e.POSSIBLY_STALE_=1]="POSSIBLY_STALE_",e[e.STALE_=2]="STALE_"}(ze||(ze={})),function(e){e[e.NONE=0]="NONE",e[e.LOG=1]="LOG",e[e.BREAK=2]="BREAK"}(Fe||(Fe={}));var He=function(e){this.cause=void 0,this.cause=e};function Be(e){return e instanceof He}function Ge(e){switch(e.dependenciesState_){case ze.UP_TO_DATE_:return!1;case ze.NOT_TRACKING_:case ze.STALE_:return!0;case ze.POSSIBLY_STALE_:for(var t=Xe(!0),r=Ye(),n=e.observing_,i=n.length,o=0;o<i;o++){var a=n[o];if($e(a)){if(rt.disableErrorBoundaries)a.get();else try{a.get()}catch(e){return Qe(r),Ze(t),!0}if(e.dependenciesState_===ze.STALE_)return Qe(r),Ze(t),!0}}return Je(e),Qe(r),Ze(t),!1}}function Ke(e,t,r){var n=Xe(!0);Je(e),e.newObserving_=new Array(e.observing_.length+100),e.unboundDepsCount_=0,e.runId_=++rt.runId;var i,o=rt.trackingDerivation;if(rt.trackingDerivation=e,rt.inBatch++,!0===rt.disableErrorBoundaries)i=t.call(r);else try{i=t.call(r)}catch(e){i=new He(e)}return rt.inBatch--,rt.trackingDerivation=o,function(e){for(var t=e.observing_,r=e.observing_=e.newObserving_,n=ze.UP_TO_DATE_,i=0,o=e.unboundDepsCount_,a=0;a<o;a++){var s=r[a];0===s.diffValue_&&(s.diffValue_=1,i!==a&&(r[i]=s),i++),s.dependenciesState_>n&&(n=s.dependenciesState_)}r.length=i,e.newObserving_=null,o=t.length;for(;o--;){var u=t[o];0===u.diffValue_&&it(u,e),u.diffValue_=0}for(;i--;){var c=r[i];1===c.diffValue_&&(c.diffValue_=0,nt(c,e))}n!==ze.UP_TO_DATE_&&(e.dependenciesState_=n,e.onBecomeStale_())}(e),Ze(n),i}function qe(e){var t=e.observing_;e.observing_=[];for(var r=t.length;r--;)it(t[r],e);e.dependenciesState_=ze.NOT_TRACKING_}function We(e){var t=Ye();try{return e()}finally{Qe(t)}}function Ye(){var e=rt.trackingDerivation;return rt.trackingDerivation=null,e}function Qe(e){rt.trackingDerivation=e}function Xe(e){var t=rt.allowStateReads;return rt.allowStateReads=e,t}function Ze(e){rt.allowStateReads=e}function Je(e){if(e.dependenciesState_!==ze.UP_TO_DATE_){e.dependenciesState_=ze.UP_TO_DATE_;for(var t=e.observing_,r=t.length;r--;)t[r].lowestObserverState_=ze.UP_TO_DATE_}}var et=function(){this.version=6,this.UNCHANGED={},this.trackingDerivation=null,this.trackingContext=null,this.runId=0,this.mobxGuid=0,this.inBatch=0,this.pendingUnobservations=[],this.pendingReactions=[],this.isRunningReactions=!1,this.allowStateChanges=!1,this.allowStateReads=!0,this.enforceActions=!0,this.spyListeners=[],this.globalReactionErrorHandlers=[],this.computedRequiresReaction=!1,this.reactionRequiresObservable=!1,this.observableRequiresReaction=!1,this.disableErrorBoundaries=!1,this.suppressReactionErrors=!1,this.useProxies=!0,this.verifyProxies=!1,this.safeDescriptors=!0},tt=!0,rt=function(){var t=r();return t.__mobxInstanceCount>0&&!t.__mobxGlobals&&(tt=!1),t.__mobxGlobals&&t.__mobxGlobals.version!==(new et).version&&(tt=!1),tt?t.__mobxGlobals?(t.__mobxInstanceCount+=1,t.__mobxGlobals.UNCHANGED||(t.__mobxGlobals.UNCHANGED={}),t.__mobxGlobals):(t.__mobxInstanceCount=1,t.__mobxGlobals=new et):(setTimeout((function(){e(35)}),1),new et)}();function nt(e,t){e.observers_.add(t),e.lowestObserverState_>t.dependenciesState_&&(e.lowestObserverState_=t.dependenciesState_)}function it(e,t){e.observers_.delete(t),0===e.observers_.size&&ot(e)}function ot(e){!1===e.isPendingUnobservation_&&(e.isPendingUnobservation_=!0,rt.pendingUnobservations.push(e))}function at(){rt.inBatch++}function st(){if(0==--rt.inBatch){ft();for(var e=rt.pendingUnobservations,t=0;t<e.length;t++){var r=e[t];r.isPendingUnobservation_=!1,0===r.observers_.size&&(r.isBeingObserved_&&(r.isBeingObserved_=!1,r.onBUO()),r instanceof Le&&r.suspend_())}rt.pendingUnobservations=[]}}function ut(e){var t=rt.trackingDerivation;return null!==t?(t.runId_!==e.lastAccessedBy_&&(e.lastAccessedBy_=t.runId_,t.newObserving_[t.unboundDepsCount_++]=e,!e.isBeingObserved_&&rt.trackingContext&&(e.isBeingObserved_=!0,e.onBO())),!0):(0===e.observers_.size&&rt.inBatch>0&&ot(e),!1)}function ct(e){e.lowestObserverState_!==ze.STALE_&&(e.lowestObserverState_=ze.STALE_,e.observers_.forEach((function(e){e.dependenciesState_===ze.UP_TO_DATE_&&e.onBecomeStale_(),e.dependenciesState_=ze.STALE_})))}var lt=function(){function t(e,t,r,n){void 0===e&&(e="Reaction"),void 0===n&&(n=!1),this.name_=void 0,this.onInvalidate_=void 0,this.errorHandler_=void 0,this.requiresObservable_=void 0,this.observing_=[],this.newObserving_=[],this.dependenciesState_=ze.NOT_TRACKING_,this.diffValue_=0,this.runId_=0,this.unboundDepsCount_=0,this.isDisposed_=!1,this.isScheduled_=!1,this.isTrackPending_=!1,this.isRunning_=!1,this.isTracing_=Fe.NONE,this.name_=e,this.onInvalidate_=t,this.errorHandler_=r,this.requiresObservable_=n}var r=t.prototype;return r.onBecomeStale_=function(){this.schedule_()},r.schedule_=function(){this.isScheduled_||(this.isScheduled_=!0,rt.pendingReactions.push(this),ft())},r.isScheduled=function(){return this.isScheduled_},r.runReaction_=function(){if(!this.isDisposed_){at(),this.isScheduled_=!1;var e=rt.trackingContext;if(rt.trackingContext=this,Ge(this)){this.isTrackPending_=!0;try{this.onInvalidate_()}catch(e){this.reportExceptionInDerivation_(e)}}rt.trackingContext=e,st()}},r.track=function(e){if(!this.isDisposed_){at(),this.isRunning_=!0;var t=rt.trackingContext;rt.trackingContext=this;var r=Ke(this,e,void 0);rt.trackingContext=t,this.isRunning_=!1,this.isTrackPending_=!1,this.isDisposed_&&qe(this),Be(r)&&this.reportExceptionInDerivation_(r.cause),st()}},r.reportExceptionInDerivation_=function(e){var t=this;if(this.errorHandler_)this.errorHandler_(e,this);else{if(rt.disableErrorBoundaries)throw e;var r="[mobx] uncaught error in '"+this+"'";rt.suppressReactionErrors||console.error(r,e),rt.globalReactionErrorHandlers.forEach((function(r){return r(e,t)}))}},r.dispose=function(){this.isDisposed_||(this.isDisposed_=!0,this.isRunning_||(at(),qe(this),st()))},r.getDisposer_=function(){var e=this.dispose.bind(this);return e[z]=this,e},r.toString=function(){return"Reaction["+this.name_+"]"},r.trace=function(t){void 0===t&&(t=!1),function(){e("trace() is not available in production builds");for(var t=!1,r=arguments.length,n=new Array(r),i=0;i<r;i++)n[i]=arguments[i];"boolean"==typeof n[n.length-1]&&(t=n.pop());var o=qt(n);if(!o)return e("'trace(break?)' can only be used inside a tracked computed value or a Reaction. Consider passing in the computed value or reaction explicitly");o.isTracing_===Fe.NONE&&console.log("[mobx.trace] '"+o.name_+"' tracing enabled");o.isTracing_=t?Fe.BREAK:Fe.LOG}(this,t)},t}();function ft(){rt.inBatch>0||rt.isRunningReactions||function(e){e()}(pt)}function pt(){rt.isRunningReactions=!0;for(var e=rt.pendingReactions,t=0;e.length>0;){100==++t&&(console.error("[mobx] cycle in reaction: "+e[0]),e.splice(0));for(var r=e.splice(0),n=0,i=r.length;n<i;n++)r[n].runReaction_()}rt.isRunningReactions=!1}var ht=w("Reaction",lt);var bt="action",dt="autoAction",vt="<unnamed action>",yt=q(bt),gt=q("action.bound",{bound:!0}),mt=q(dt,{autoAction:!0}),_t=q("autoAction.bound",{autoAction:!0,bound:!0});function wt(e){return function(t,r){return b(t)?ke(t.name||vt,t,e):b(r)?ke(t,r,e):d(r)?M(t,r,e?mt:yt):d(t)?U(q(e?dt:bt,{name:t,autoAction:e})):void 0}}var Ot=wt(!1);Object.assign(Ot,yt);var jt=wt(!0);function Pt(e){return De(e.name,!1,e,this,void 0)}function St(e){return b(e)&&!0===e.isMobxAction}Object.assign(jt,mt),Ot.bound=U(gt),jt.bound=U(_t);var At=function(e){return e()};function xt(e){return e.scheduler?e.scheduler:e.delay?function(t){return setTimeout(t,e.delay)}:At}function Et(e,t,r){var n;void 0===r&&(r=u);var i,o,a,s=null!=(n=r.name)?n:"Reaction",c=Ot(s,r.onError?(i=r.onError,o=t,function(){try{return o.apply(this,arguments)}catch(e){i.call(this,e)}}):t),l=!r.scheduler&&!r.delay,f=xt(r),p=!0,h=!1,b=void 0,d=r.compareStructural?H.structural:r.equals||H.default,v=new lt(s,(function(){p||l?y():h||(h=!0,f(y))}),r.onError,r.requiresObservable);function y(){if(h=!1,!v.isDisposed_){var t=!1;v.track((function(){var r=function(e,t){var r=Ne(e);try{return t()}finally{Ve(r)}}(!1,(function(){return e(v)}));t=p||!d(a,r),b=a,a=r})),(p&&r.fireImmediately||!p&&t)&&c(a,b,v),p=!1}}return v.schedule_(),v.getDisposer_()}var Tt="onBO";function It(e,t,r){return Ct("onBUO",e,t,r)}function Ct(e,t,r,n){var i="function"==typeof n?Br(t,r):Br(t),o=b(n)?n:r,a=e+"L";return i[a]?i[a].add(o):i[a]=new Set([o]),function(){var e=i[a];e&&(e.delete(o),0===e.size&&delete i[a])}}var kt=0;function Dt(){this.message="FLOW_CANCELLED"}Dt.prototype=Object.create(Error.prototype);var Nt=X("flow"),Vt=X("flow.bound",{bound:!0}),Rt=Object.assign((function(e,t){if(d(t))return M(e,t,Nt);var r=e,n=r.name||"<unnamed flow>",i=function(){var e,t=this,i=arguments,o=++kt,a=Ot(n+" - runid: "+o+" - init",r).apply(t,i),s=void 0,u=new Promise((function(t,r){var i=0;function u(e){var t;s=void 0;try{t=Ot(n+" - runid: "+o+" - yield "+i++,a.next).call(a,e)}catch(e){return r(e)}l(t)}function c(e){var t;s=void 0;try{t=Ot(n+" - runid: "+o+" - yield "+i++,a.throw).call(a,e)}catch(e){return r(e)}l(t)}function l(e){if(!b(null==e?void 0:e.then))return e.done?t(e.value):(s=Promise.resolve(e.value)).then(u,c);e.then(l,r)}e=r,u(void 0)}));return u.cancel=Ot(n+" - runid: "+o+" - cancel",(function(){try{s&&Ut(s);var t=a.return(void 0),r=Promise.resolve(t.value);r.then(h,h),Ut(r),e(new Dt)}catch(t){e(t)}})),u};return i.isMobXFlow=!0,i}),Nt);function Ut(e){b(e.cancel)&&e.cancel()}function Mt(e){return!0===(null==e?void 0:e.isMobXFlow)}function zt(e,t,r){var n;return Pr(e)||yr(e)||Me(e)?n=Gr(e):Nr(e)&&(n=Gr(e,t)),n.dehancer="function"==typeof t?t:r,function(){n.dehancer=void 0}}function Ft(e,t,r){return b(r)?function(e,t,r){return Gr(e,t).intercept_(r)}(e,t,r):function(e,t){return Gr(e).intercept_(t)}(e,t)}function Lt(e){return function(e,t){return!!e&&(void 0!==t?!!Nr(e)&&e[z].values_.has(t):Nr(e)||!!e[z]||L(e)||ht(e)||$e(e))}(e)}function $t(t){return Nr(t)?t[z].keys_():Pr(t)||xr(t)?Array.from(t.keys()):yr(t)?t.map((function(e,t){return t})):void e(5)}function Ht(t){return Nr(t)?$t(t).map((function(e){return t[e]})):Pr(t)?$t(t).map((function(e){return t.get(e)})):xr(t)?Array.from(t.values()):yr(t)?t.slice():void e(6)}function Bt(t,r,n){if(2!==arguments.length||xr(t))Nr(t)?t[z].set_(r,n):Pr(t)?t.set(r,n):xr(t)?t.add(r):yr(t)?("number"!=typeof r&&(r=parseInt(r,10)),r<0&&e("Invalid index: '"+r+"'"),at(),r>=t.length&&(t.length=r+1),t[r]=n,st()):e(8);else{at();var i=r;try{for(var o in i)Bt(t,o,i[o])}finally{st()}}}function Gt(t,r,n){if(Nr(t))return t[z].defineProperty_(r,n);e(39)}function Kt(e,t,r,n){return b(r)?function(e,t,r,n){return Gr(e,t).observe_(r,n)}(e,t,r,n):function(e,t,r){return Gr(e).observe_(t,r)}(e,t,r)}function qt(e){switch(e.length){case 0:return rt.trackingDerivation;case 1:return Br(e[0]);case 2:return Br(e[0],e[1])}}function Wt(e,t){void 0===t&&(t=void 0),at();try{return e.apply(t)}finally{st()}}function Yt(e){return e[z]}Rt.bound=U(Vt);var Qt={has:function(e,t){return Yt(e).has_(t)},get:function(e,t){return Yt(e).get_(t)},set:function(e,t,r){var n;return!!d(t)&&(null==(n=Yt(e).set_(t,r,!0))||n)},deleteProperty:function(e,t){var r;return!!d(t)&&(null==(r=Yt(e).delete_(t,!0))||r)},defineProperty:function(e,t,r){var n;return null==(n=Yt(e).defineProperty_(t,r))||n},ownKeys:function(e){return Yt(e).ownKeys_()},preventExtensions:function(t){e(13)}};function Xt(e){return void 0!==e.interceptors_&&e.interceptors_.length>0}function Zt(e,t){var r=e.interceptors_||(e.interceptors_=[]);return r.push(t),p((function(){var e=r.indexOf(t);-1!==e&&r.splice(e,1)}))}function Jt(t,r){var n=Ye();try{for(var i=[].concat(t.interceptors_||[]),o=0,a=i.length;o<a&&((r=i[o](r))&&!r.type&&e(14),r);o++);return r}finally{Qe(n)}}function er(e){return void 0!==e.changeListeners_&&e.changeListeners_.length>0}function tr(e,t){var r=e.changeListeners_||(e.changeListeners_=[]);return r.push(t),p((function(){var e=r.indexOf(t);-1!==e&&r.splice(e,1)}))}function rr(e,t){var r=Ye(),n=e.changeListeners_;if(n){for(var i=0,o=(n=n.slice()).length;i<o;i++)n[i](t);Qe(r)}}function nr(e,t,r){var n=Cr(e,r)[z];at();try{null!=t||(t=function(e){return x(e,R)||m(e,R,C({},e[R])),e[R]}(e)),S(t).forEach((function(e){return n.make_(e,t[e])}))}finally{st()}return e}var ir="splice",or="update",ar={get:function(e,t){var r=e[z];return t===z?r:"length"===t?r.getArrayLength_():"string"!=typeof t||isNaN(t)?x(cr,t)?cr[t]:e[t]:r.get_(parseInt(t))},set:function(e,t,r){var n=e[z];return"length"===t&&n.setArrayLength_(r),"symbol"==typeof t||isNaN(t)?e[t]=r:n.set_(parseInt(t),r),!0},preventExtensions:function(){e(15)}},sr=function(){function t(e,t,r,n){void 0===e&&(e="ObservableArray"),this.owned_=void 0,this.legacyMode_=void 0,this.atom_=void 0,this.values_=[],this.interceptors_=void 0,this.changeListeners_=void 0,this.enhancer_=void 0,this.dehancer=void 0,this.proxy_=void 0,this.lastKnownLength_=0,this.owned_=r,this.legacyMode_=n,this.atom_=new F(e),this.enhancer_=function(e,r){return t(e,r,"ObservableArray[..]")}}var r=t.prototype;return r.dehanceValue_=function(e){return void 0!==this.dehancer?this.dehancer(e):e},r.dehanceValues_=function(e){return void 0!==this.dehancer&&e.length>0?e.map(this.dehancer):e},r.intercept_=function(e){return Zt(this,e)},r.observe_=function(e,t){return void 0===t&&(t=!1),t&&e({observableKind:"array",object:this.proxy_,debugObjectName:this.atom_.name_,type:"splice",index:0,added:this.values_.slice(),addedCount:this.values_.length,removed:[],removedCount:0}),tr(this,e)},r.getArrayLength_=function(){return this.atom_.reportObserved(),this.values_.length},r.setArrayLength_=function(t){("number"!=typeof t||t<0)&&e("Out of range: "+t);var r=this.values_.length;if(t!==r)if(t>r){for(var n=new Array(t-r),i=0;i<t-r;i++)n[i]=void 0;this.spliceWithArray_(r,0,n)}else this.spliceWithArray_(t,r-t)},r.updateArrayLength_=function(t,r){t!==this.lastKnownLength_&&e(16),this.lastKnownLength_+=r,this.legacyMode_&&r>0&&$r(t+r+1)},r.spliceWithArray_=function(e,t,r){var n=this;this.atom_;var i=this.values_.length;if(void 0===e?e=0:e>i?e=i:e<0&&(e=Math.max(0,i+e)),t=1===arguments.length?i-e:null==t?0:Math.max(0,Math.min(t,i-e)),void 0===r&&(r=s),Xt(this)){var o=Jt(this,{object:this.proxy_,type:ir,index:e,removedCount:t,added:r});if(!o)return s;t=o.removedCount,r=o.added}if(r=0===r.length?r:r.map((function(e){return n.enhancer_(e,void 0)})),this.legacyMode_){var a=r.length-t;this.updateArrayLength_(i,a)}var u=this.spliceItemsIntoValues_(e,t,r);return 0===t&&0===r.length||this.notifyArraySplice_(e,r,u),this.dehanceValues_(u)},r.spliceItemsIntoValues_=function(e,t,r){var n;if(r.length<1e4)return(n=this.values_).splice.apply(n,[e,t].concat(r));var i=this.values_.slice(e,e+t),o=this.values_.slice(e+t);this.values_.length=e+r.length-t;for(var a=0;a<r.length;a++)this.values_[e+a]=r[a];for(var s=0;s<o.length;s++)this.values_[e+r.length+s]=o[s];return i},r.notifyArrayChildUpdate_=function(e,t,r){var n=!this.owned_&&!1,i=er(this),o=i||n?{observableKind:"array",object:this.proxy_,type:or,debugObjectName:this.atom_.name_,index:e,newValue:t,oldValue:r}:null;this.atom_.reportChanged(),i&&rr(this,o)},r.notifyArraySplice_=function(e,t,r){var n=!this.owned_&&!1,i=er(this),o=i||n?{observableKind:"array",object:this.proxy_,debugObjectName:this.atom_.name_,type:ir,index:e,removed:r,added:t,removedCount:r.length,addedCount:t.length}:null;this.atom_.reportChanged(),i&&rr(this,o)},r.get_=function(e){if(e<this.values_.length)return this.atom_.reportObserved(),this.dehanceValue_(this.values_[e]);console.warn("[mobx.array] Attempt to read an array index ("+e+") that is out of bounds ("+this.values_.length+"). Please check length first. Out of bound indices will not be tracked by MobX")},r.set_=function(t,r){var n=this.values_;if(t<n.length){this.atom_;var i=n[t];if(Xt(this)){var o=Jt(this,{type:or,object:this.proxy_,index:t,newValue:r});if(!o)return;r=o.newValue}(r=this.enhancer_(r,i))!==i&&(n[t]=r,this.notifyArrayChildUpdate_(t,r,i))}else t===n.length?this.spliceWithArray_(t,0,[r]):e(17,t,n.length)},t}();function ur(e,t,r,n){void 0===r&&(r="ObservableArray"),void 0===n&&(n=!1),f();var i=new sr(r,t,n,!1);_(i.values_,z,i);var o=new Proxy(i.values_,ar);if(i.proxy_=o,e&&e.length){var a=Ne(!0);i.spliceWithArray_(0,0,e),Ve(a)}return o}var cr={clear:function(){return this.splice(0)},replace:function(e){var t=this[z];return t.spliceWithArray_(0,t.values_.length,e)},toJSON:function(){return this.slice()},splice:function(e,t){for(var r=arguments.length,n=new Array(r>2?r-2:0),i=2;i<r;i++)n[i-2]=arguments[i];var o=this[z];switch(arguments.length){case 0:return[];case 1:return o.spliceWithArray_(e);case 2:return o.spliceWithArray_(e,t)}return o.spliceWithArray_(e,t,n)},spliceWithArray:function(e,t,r){return this[z].spliceWithArray_(e,t,r)},push:function(){for(var e=this[z],t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return e.spliceWithArray_(e.values_.length,0,r),e.values_.length},pop:function(){return this.splice(Math.max(this[z].values_.length-1,0),1)[0]},shift:function(){return this.splice(0,1)[0]},unshift:function(){for(var e=this[z],t=arguments.length,r=new Array(t),n=0;n<t;n++)r[n]=arguments[n];return e.spliceWithArray_(0,0,r),e.values_.length},reverse:function(){return rt.trackingDerivation&&e(37,"reverse"),this.replace(this.slice().reverse()),this},sort:function(){rt.trackingDerivation&&e(37,"sort");var t=this.slice();return t.sort.apply(t,arguments),this.replace(t),this},remove:function(e){var t=this[z],r=t.dehanceValues_(t.values_).indexOf(e);return r>-1&&(this.splice(r,1),!0)}};function lr(e,t){"function"==typeof Array.prototype[e]&&(cr[e]=t(e))}function fr(e){return function(){var t=this[z];t.atom_.reportObserved();var r=t.dehanceValues_(t.values_);return r[e].apply(r,arguments)}}function pr(e){return function(t,r){var n=this,i=this[z];return i.atom_.reportObserved(),i.dehanceValues_(i.values_)[e]((function(e,i){return t.call(r,e,i,n)}))}}function hr(e){return function(){var t=this,r=this[z];r.atom_.reportObserved();var n=r.dehanceValues_(r.values_),i=arguments[0];return arguments[0]=function(e,r,n){return i(e,r,n,t)},n[e].apply(n,arguments)}}lr("concat",fr),lr("flat",fr),lr("includes",fr),lr("indexOf",fr),lr("join",fr),lr("lastIndexOf",fr),lr("slice",fr),lr("toString",fr),lr("toLocaleString",fr),lr("every",pr),lr("filter",pr),lr("find",pr),lr("findIndex",pr),lr("flatMap",pr),lr("forEach",pr),lr("map",pr),lr("some",pr),lr("reduce",hr),lr("reduceRight",hr);var br,dr,vr=w("ObservableArrayAdministration",sr);function yr(e){return v(e)&&vr(e[z])}var gr={},mr="add",_r="delete";br=Symbol.iterator,dr=Symbol.toStringTag;var wr,Or,jr=function(){function t(t,r,n){void 0===r&&(r=B),void 0===n&&(n="ObservableMap"),this.enhancer_=void 0,this.name_=void 0,this[z]=gr,this.data_=void 0,this.hasMap_=void 0,this.keysAtom_=void 0,this.interceptors_=void 0,this.changeListeners_=void 0,this.dehancer=void 0,this.enhancer_=r,this.name_=n,b(Map)||e(18),this.keysAtom_=$("ObservableMap.keys()"),this.data_=new Map,this.hasMap_=new Map,this.merge(t)}var r=t.prototype;return r.has_=function(e){return this.data_.has(e)},r.has=function(e){var t=this;if(!rt.trackingDerivation)return this.has_(e);var r=this.hasMap_.get(e);if(!r){var n=r=new Ue(this.has_(e),G,"ObservableMap.key?",!1);this.hasMap_.set(e,n),It(n,(function(){return t.hasMap_.delete(e)}))}return r.get()},r.set=function(e,t){var r=this.has_(e);if(Xt(this)){var n=Jt(this,{type:r?or:mr,object:this,newValue:t,name:e});if(!n)return this;t=n.newValue}return r?this.updateValue_(e,t):this.addValue_(e,t),this},r.delete=function(e){var t=this;if((this.keysAtom_,Xt(this))&&!Jt(this,{type:_r,object:this,name:e}))return!1;if(this.has_(e)){var r=er(this),n=r?{observableKind:"map",debugObjectName:this.name_,type:_r,object:this,oldValue:this.data_.get(e).value_,name:e}:null;return Wt((function(){t.keysAtom_.reportChanged(),t.updateHasMapEntry_(e,!1),t.data_.get(e).setNewValue_(void 0),t.data_.delete(e)})),r&&rr(this,n),!0}return!1},r.updateHasMapEntry_=function(e,t){var r=this.hasMap_.get(e);r&&r.setNewValue_(t)},r.updateValue_=function(e,t){var r=this.data_.get(e);if((t=r.prepareNewValue_(t))!==rt.UNCHANGED){var n=er(this),i=n?{observableKind:"map",debugObjectName:this.name_,type:or,object:this,oldValue:r.value_,name:e,newValue:t}:null;r.setNewValue_(t),n&&rr(this,i)}},r.addValue_=function(e,t){var r=this;this.keysAtom_,Wt((function(){var n=new Ue(t,r.enhancer_,"ObservableMap.key",!1);r.data_.set(e,n),t=n.value_,r.updateHasMapEntry_(e,!0),r.keysAtom_.reportChanged()}));var n=er(this),i=n?{observableKind:"map",debugObjectName:this.name_,type:mr,object:this,name:e,newValue:t}:null;n&&rr(this,i)},r.get=function(e){return this.has(e)?this.dehanceValue_(this.data_.get(e).get()):this.dehanceValue_(void 0)},r.dehanceValue_=function(e){return void 0!==this.dehancer?this.dehancer(e):e},r.keys=function(){return this.keysAtom_.reportObserved(),this.data_.keys()},r.values=function(){var e=this,t=this.keys();return Xr({next:function(){var r=t.next(),n=r.done,i=r.value;return{done:n,value:n?void 0:e.get(i)}}})},r.entries=function(){var e=this,t=this.keys();return Xr({next:function(){var r=t.next(),n=r.done,i=r.value;return{done:n,value:n?void 0:[i,e.get(i)]}}})},r[br]=function(){return this.entries()},r.forEach=function(e,t){for(var r,n=V(this);!(r=n()).done;){var i=r.value,o=i[0],a=i[1];e.call(t,a,o,this)}},r.merge=function(t){var r=this;return Pr(t)&&(t=new Map(t)),Wt((function(){y(t)?function(e){var t=Object.keys(e);if(!P)return t;var r=Object.getOwnPropertySymbols(e);return r.length?[].concat(t,r.filter((function(t){return a.propertyIsEnumerable.call(e,t)}))):t}(t).forEach((function(e){return r.set(e,t[e])})):Array.isArray(t)?t.forEach((function(e){var t=e[0],n=e[1];return r.set(t,n)})):O(t)?(t.constructor!==Map&&e(19,t),t.forEach((function(e,t){return r.set(t,e)}))):null!=t&&e(20,t)})),this},r.clear=function(){var e=this;Wt((function(){We((function(){for(var t,r=V(e.keys());!(t=r()).done;){var n=t.value;e.delete(n)}}))}))},r.replace=function(t){var r=this;return Wt((function(){for(var n,i=function(t){if(O(t)||Pr(t))return t;if(Array.isArray(t))return new Map(t);if(y(t)){var r=new Map;for(var n in t)r.set(n,t[n]);return r}return e(21,t)}(t),o=new Map,a=!1,s=V(r.data_.keys());!(n=s()).done;){var u=n.value;if(!i.has(u))if(r.delete(u))a=!0;else{var c=r.data_.get(u);o.set(u,c)}}for(var l,f=V(i.entries());!(l=f()).done;){var p=l.value,h=p[0],b=p[1],d=r.data_.has(h);if(r.set(h,b),r.data_.has(h)){var v=r.data_.get(h);o.set(h,v),d||(a=!0)}}if(!a)if(r.data_.size!==o.size)r.keysAtom_.reportChanged();else for(var g=r.data_.keys(),m=o.keys(),_=g.next(),w=m.next();!_.done;){if(_.value!==w.value){r.keysAtom_.reportChanged();break}_=g.next(),w=m.next()}r.data_=o})),this},r.toString=function(){return"[object ObservableMap]"},r.toJSON=function(){return Array.from(this)},r.observe_=function(e,t){return tr(this,e)},r.intercept_=function(e){return Zt(this,e)},I(t,[{key:"size",get:function(){return this.keysAtom_.reportObserved(),this.data_.size}},{key:dr,get:function(){return"Map"}}]),t}(),Pr=w("ObservableMap",jr);var Sr={};wr=Symbol.iterator,Or=Symbol.toStringTag;var Ar=function(){function t(t,r,n){void 0===r&&(r=B),void 0===n&&(n="ObservableSet"),this.name_=void 0,this[z]=Sr,this.data_=new Set,this.atom_=void 0,this.changeListeners_=void 0,this.interceptors_=void 0,this.dehancer=void 0,this.enhancer_=void 0,this.name_=n,b(Set)||e(22),this.atom_=$(this.name_),this.enhancer_=function(e,t){return r(e,t,n)},t&&this.replace(t)}var r=t.prototype;return r.dehanceValue_=function(e){return void 0!==this.dehancer?this.dehancer(e):e},r.clear=function(){var e=this;Wt((function(){We((function(){for(var t,r=V(e.data_.values());!(t=r()).done;){var n=t.value;e.delete(n)}}))}))},r.forEach=function(e,t){for(var r,n=V(this);!(r=n()).done;){var i=r.value;e.call(t,i,i,this)}},r.add=function(e){var t=this;if((this.atom_,Xt(this))&&!Jt(this,{type:mr,object:this,newValue:e}))return this;if(!this.has(e)){Wt((function(){t.data_.add(t.enhancer_(e,void 0)),t.atom_.reportChanged()}));var r=er(this),n=r?{observableKind:"set",debugObjectName:this.name_,type:mr,object:this,newValue:e}:null;r&&rr(this,n)}return this},r.delete=function(e){var t=this;if(Xt(this)&&!Jt(this,{type:_r,object:this,oldValue:e}))return!1;if(this.has(e)){var r=er(this),n=r?{observableKind:"set",debugObjectName:this.name_,type:_r,object:this,oldValue:e}:null;return Wt((function(){t.atom_.reportChanged(),t.data_.delete(e)})),r&&rr(this,n),!0}return!1},r.has=function(e){return this.atom_.reportObserved(),this.data_.has(this.dehanceValue_(e))},r.entries=function(){var e=0,t=Array.from(this.keys()),r=Array.from(this.values());return Xr({next:function(){var n=e;return e+=1,n<r.length?{value:[t[n],r[n]],done:!1}:{done:!0}}})},r.keys=function(){return this.values()},r.values=function(){this.atom_.reportObserved();var e=this,t=0,r=Array.from(this.data_.values());return Xr({next:function(){return t<r.length?{value:e.dehanceValue_(r[t++]),done:!1}:{done:!0}}})},r.replace=function(t){var r=this;return xr(t)&&(t=new Set(t)),Wt((function(){Array.isArray(t)||j(t)?(r.clear(),t.forEach((function(e){return r.add(e)}))):null!=t&&e("Cannot initialize set from "+t)})),this},r.observe_=function(e,t){return tr(this,e)},r.intercept_=function(e){return Zt(this,e)},r.toJSON=function(){return Array.from(this)},r.toString=function(){return"[object ObservableSet]"},r[wr]=function(){return this.values()},I(t,[{key:"size",get:function(){return this.atom_.reportObserved(),this.data_.size}},{key:Or,get:function(){return"Set"}}]),t}(),xr=w("ObservableSet",Ar),Er=Object.create(null),Tr="remove",Ir=function(){function t(e,t,r,n){void 0===t&&(t=new Map),void 0===n&&(n=se),this.target_=void 0,this.values_=void 0,this.name_=void 0,this.defaultAnnotation_=void 0,this.keysAtom_=void 0,this.changeListeners_=void 0,this.interceptors_=void 0,this.proxy_=void 0,this.isPlainObject_=void 0,this.appliedAnnotations_=void 0,this.pendingKeys_=void 0,this.target_=e,this.values_=t,this.name_=r,this.defaultAnnotation_=n,this.keysAtom_=new F("ObservableObject.keys"),this.isPlainObject_=y(this.target_)}var r=t.prototype;return r.getObservablePropValue_=function(e){return this.values_.get(e).get()},r.setObservablePropValue_=function(e,t){var r=this.values_.get(e);if(r instanceof Le)return r.set(t),!0;if(Xt(this)){var n=Jt(this,{type:or,object:this.proxy_||this.target_,name:e,newValue:t});if(!n)return null;t=n.newValue}if((t=r.prepareNewValue_(t))!==rt.UNCHANGED){var i=er(this),o=i?{type:or,observableKind:"object",debugObjectName:this.name_,object:this.proxy_||this.target_,oldValue:r.value_,name:e,newValue:t}:null;r.setNewValue_(t),i&&rr(this,o)}return!0},r.get_=function(e){return rt.trackingDerivation&&!x(this.target_,e)&&this.has_(e),this.target_[e]},r.set_=function(e,t,r){return void 0===r&&(r=!1),x(this.target_,e)?this.values_.has(e)?this.setObservablePropValue_(e,t):r?Reflect.set(this.target_,e,t):(this.target_[e]=t,!0):this.extend_(e,{value:t,enumerable:!0,writable:!0,configurable:!0},this.defaultAnnotation_,r)},r.has_=function(e){if(!rt.trackingDerivation)return e in this.target_;this.pendingKeys_||(this.pendingKeys_=new Map);var t=this.pendingKeys_.get(e);return t||(t=new Ue(e in this.target_,G,"ObservableObject.key?",!1),this.pendingKeys_.set(e,t)),t.get()},r.make_=function(t,r){if(!0===r&&(r=this.defaultAnnotation_),!1!==r){if(!(t in this.target_)){var n;if(null==(n=this.target_[R])?void 0:n[t])return;e(1,r.annotationType_,this.name_+"."+t.toString())}for(var o=this.target_;o&&o!==a;){var s=i(o,t);if(s){var u=r.make_(this,t,s,o);if(0===u)return;if(1===u)break}o=Object.getPrototypeOf(o)}Vr(this,r,t)}},r.extend_=function(e,t,r,n){if(void 0===n&&(n=!1),!0===r&&(r=this.defaultAnnotation_),!1===r)return this.defineProperty_(e,t,n);var i=r.extend_(this,e,t,n);return i&&Vr(this,r,e),i},r.defineProperty_=function(e,t,r){void 0===r&&(r=!1);try{at();var n=this.delete_(e);if(!n)return n;if(Xt(this)){var i=Jt(this,{object:this.proxy_||this.target_,name:e,type:mr,newValue:t.value});if(!i)return null;var a=i.newValue;t.value!==a&&(t=C({},t,{value:a}))}if(r){if(!Reflect.defineProperty(this.target_,e,t))return!1}else o(this.target_,e,t);this.notifyPropertyAddition_(e,t.value)}finally{st()}return!0},r.defineObservableProperty_=function(e,t,r,n){void 0===n&&(n=!1);try{at();var i=this.delete_(e);if(!i)return i;if(Xt(this)){var a=Jt(this,{object:this.proxy_||this.target_,name:e,type:mr,newValue:t});if(!a)return null;t=a.newValue}var s=Dr(e),u={configurable:!rt.safeDescriptors||this.isPlainObject_,enumerable:!0,get:s.get,set:s.set};if(n){if(!Reflect.defineProperty(this.target_,e,u))return!1}else o(this.target_,e,u);var c=new Ue(t,r,"ObservableObject.key",!1);this.values_.set(e,c),this.notifyPropertyAddition_(e,c.value_)}finally{st()}return!0},r.defineComputedProperty_=function(e,t,r){void 0===r&&(r=!1);try{at();var n=this.delete_(e);if(!n)return n;if(Xt(this))if(!Jt(this,{object:this.proxy_||this.target_,name:e,type:mr,newValue:void 0}))return null;t.name||(t.name="ObservableObject.key"),t.context=this.proxy_||this.target_;var i=Dr(e),a={configurable:!rt.safeDescriptors||this.isPlainObject_,enumerable:!1,get:i.get,set:i.set};if(r){if(!Reflect.defineProperty(this.target_,e,a))return!1}else o(this.target_,e,a);this.values_.set(e,new Le(t)),this.notifyPropertyAddition_(e,void 0)}finally{st()}return!0},r.delete_=function(e,t){if(void 0===t&&(t=!1),!x(this.target_,e))return!0;if(Xt(this)&&!Jt(this,{object:this.proxy_||this.target_,name:e,type:Tr}))return null;try{var r,n;at();var o,a=er(this),s=this.values_.get(e),u=void 0;if(!s&&a)u=null==(o=i(this.target_,e))?void 0:o.value;if(t){if(!Reflect.deleteProperty(this.target_,e))return!1}else delete this.target_[e];if(s&&(this.values_.delete(e),s instanceof Ue&&(u=s.value_),ct(s)),this.keysAtom_.reportChanged(),null==(r=this.pendingKeys_)||null==(n=r.get(e))||n.set(e in this.target_),a){var c={type:Tr,observableKind:"object",object:this.proxy_||this.target_,debugObjectName:this.name_,oldValue:u,name:e};0,a&&rr(this,c)}}finally{st()}return!0},r.observe_=function(e,t){return tr(this,e)},r.intercept_=function(e){return Zt(this,e)},r.notifyPropertyAddition_=function(e,t){var r,n,i=er(this);if(i){var o=i?{type:mr,observableKind:"object",debugObjectName:this.name_,object:this.proxy_||this.target_,name:e,newValue:t}:null;i&&rr(this,o)}null==(r=this.pendingKeys_)||null==(n=r.get(e))||n.set(!0),this.keysAtom_.reportChanged()},r.ownKeys_=function(){return this.keysAtom_.reportObserved(),S(this.target_)},r.keys_=function(){return this.keysAtom_.reportObserved(),Object.keys(this.target_)},t}();function Cr(e,t){var r;if(x(e,z))return e;var n=null!=(r=null==t?void 0:t.name)?r:"ObservableObject",i=new Ir(e,new Map,String(n),function(e){var t;return e?null!=(t=e.defaultDecorator)?t:ue(e):void 0}(t));return m(e,z,i),e}var kr=w("ObservableObjectAdministration",Ir);function Dr(e){return Er[e]||(Er[e]={get:function(){return this[z].getObservablePropValue_(e)},set:function(t){return this[z].setObservablePropValue_(e,t)}})}function Nr(e){return!!v(e)&&kr(e[z])}function Vr(e,t,r){var n;null==(n=e.target_[R])||delete n[r]}var Rr,Ur,Mr=0,zr=function(){};Rr=zr,Ur=Array.prototype,Object.setPrototypeOf?Object.setPrototypeOf(Rr.prototype,Ur):void 0!==Rr.prototype.__proto__?Rr.prototype.__proto__=Ur:Rr.prototype=Ur;var Fr=function(e){function t(t,r,n,i){var o;void 0===n&&(n="ObservableArray"),void 0===i&&(i=!1),o=e.call(this)||this;var a=new sr(n,r,i,!0);if(a.proxy_=D(o),_(D(o),z,a),t&&t.length){var s=Ne(!0);o.spliceWithArray(0,0,t),Ve(s)}return o}k(t,e);var r=t.prototype;return r.concat=function(){this[z].atom_.reportObserved();for(var e=arguments.length,t=new Array(e),r=0;r<e;r++)t[r]=arguments[r];return Array.prototype.concat.apply(this.slice(),t.map((function(e){return yr(e)?e.slice():e})))},r[Symbol.iterator]=function(){var e=this,t=0;return Xr({next:function(){return t<e.length?{value:e[t++],done:!1}:{done:!0,value:void 0}}})},I(t,[{key:"length",get:function(){return this[z].getArrayLength_()},set:function(e){this[z].setArrayLength_(e)}},{key:Symbol.toStringTag,get:function(){return"Array"}}]),t}(zr);function Lr(e){o(Fr.prototype,""+e,function(e){return{enumerable:!1,configurable:!0,get:function(){return this[z].get_(e)},set:function(t){this[z].set_(e,t)}}}(e))}function $r(e){if(e>Mr){for(var t=Mr;t<e+100;t++)Lr(t);Mr=e}}function Hr(e,t,r){return new Fr(e,t,r)}function Br(t,r){if("object"==typeof t&&null!==t){if(yr(t))return void 0!==r&&e(23),t[z].atom_;if(xr(t))return t[z];if(Pr(t)){if(void 0===r)return t.keysAtom_;var n=t.data_.get(r)||t.hasMap_.get(r);return n||e(25,r,Kr(t)),n}if(Nr(t)){if(!r)return e(26);var i=t[z].values_.get(r);return i||e(27,r,Kr(t)),i}if(L(t)||$e(t)||ht(t))return t}else if(b(t)&&ht(t[z]))return t[z];e(28)}function Gr(t,r){return t||e(29),void 0!==r?Gr(Br(t,r)):L(t)||$e(t)||ht(t)||Pr(t)||xr(t)?t:t[z]?t[z]:void e(24,t)}function Kr(e,t){var r;if(void 0!==t)r=Br(e,t);else{if(St(e))return e.name;r=Nr(e)||Pr(e)||xr(e)?Gr(e):Br(e)}return r.name_}Object.entries(cr).forEach((function(e){var t=e[0],r=e[1];"concat"!==t&&m(Fr.prototype,t,r)})),$r(1e3);var qr=a.toString;function Wr(e,t,r){return void 0===r&&(r=-1),Yr(e,t,r)}function Yr(e,t,r,n,i){if(e===t)return 0!==e||1/e==1/t;if(null==e||null==t)return!1;if(e!=e)return t!=t;var o=typeof e;if(!b(o)&&"object"!==o&&"object"!=typeof t)return!1;var a=qr.call(e);if(a!==qr.call(t))return!1;switch(a){case"[object RegExp]":case"[object String]":return""+e==""+t;case"[object Number]":return+e!=+e?+t!=+t:0==+e?1/+e==1/t:+e==+t;case"[object Date]":case"[object Boolean]":return+e==+t;case"[object Symbol]":return"undefined"!=typeof Symbol&&Symbol.valueOf.call(e)===Symbol.valueOf.call(t);case"[object Map]":case"[object Set]":r>=0&&r++}e=Qr(e),t=Qr(t);var s="[object Array]"===a;if(!s){if("object"!=typeof e||"object"!=typeof t)return!1;var u=e.constructor,c=t.constructor;if(u!==c&&!(b(u)&&u instanceof u&&b(c)&&c instanceof c)&&"constructor"in e&&"constructor"in t)return!1}if(0===r)return!1;r<0&&(r=-1),i=i||[];for(var l=(n=n||[]).length;l--;)if(n[l]===e)return i[l]===t;if(n.push(e),i.push(t),s){if((l=e.length)!==t.length)return!1;for(;l--;)if(!Yr(e[l],t[l],r-1,n,i))return!1}else{var f,p=Object.keys(e);if(l=p.length,Object.keys(t).length!==l)return!1;for(;l--;)if(!x(t,f=p[l])||!Yr(e[f],t[f],r-1,n,i))return!1}return n.pop(),i.pop(),!0}function Qr(e){return yr(e)?e.slice():O(e)||Pr(e)||j(e)||xr(e)?Array.from(e.entries()):e}function Xr(e){return e[Symbol.iterator]=Zr,e}function Zr(){return this}["Symbol","Map","Set","Symbol"].forEach((function(t){void 0===r()[t]&&e("MobX requires global '"+t+"' to be available or polyfilled")})),"object"==typeof __MOBX_DEVTOOLS_GLOBAL_HOOK__&&__MOBX_DEVTOOLS_GLOBAL_HOOK__.injectMobx({spy:function(e){return console.warn("[mobx.spy] Is a no-op in production builds"),function(){}},extras:{getDebugName:Kr},$mobx:z});var Jr;!function(e){e.afterCreate="afterCreate",e.afterAttach="afterAttach",e.afterCreationFinalization="afterCreationFinalization",e.beforeDetach="beforeDetach",e.beforeDestroy="beforeDestroy"}(Jr||(Jr={}));
/*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */
var en=function(e,t){return(en=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)};function tn(e,t){function r(){this.constructor=e}en(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}var rn=function(){return(rn=Object.assign||function(e){for(var t,r=1,n=arguments.length;r<n;r++)for(var i in t=arguments[r])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)};function nn(e){var t="function"==typeof Symbol&&Symbol.iterator,r=t&&e[t],n=0;if(r)return r.call(e);if(e&&"number"==typeof e.length)return{next:function(){return e&&n>=e.length&&(e=void 0),{value:e&&e[n++],done:!e}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")}function on(e,t){var r="function"==typeof Symbol&&e[Symbol.iterator];if(!r)return e;var n,i,o=r.call(e),a=[];try{for(;(void 0===t||t-- >0)&&!(n=o.next()).done;)a.push(n.value)}catch(e){i={error:e}}finally{try{n&&!n.done&&(r=o.return)&&r.call(o)}finally{if(i)throw i.error}}return a}function an(){for(var e=[],t=0;t<arguments.length;t++)e=e.concat(on(arguments[t]));return e}function sn(e){return Bn(e).type}function un(e,t){Bn(e).applyPatches(ni(t))}function cn(e,t){void 0===t&&(t=!0);var r=Bn(e);return t?r.snapshot:r.type.getSnapshot(r,!1)}function ln(e){return Bn(e).root.storedValue}var fn=function(){function e(e,t,r,n){Object.defineProperty(this,"type",{enumerable:!0,configurable:!0,writable:!0,value:e}),Object.defineProperty(this,"environment",{enumerable:!0,configurable:!0,writable:!0,value:n}),Object.defineProperty(this,"_escapedSubpath",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"_subpath",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"_subpathUponDeath",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"_pathUponDeath",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"storedValue",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"aliveAtom",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"_state",{enumerable:!0,configurable:!0,writable:!0,value:Mn.INITIALIZING}),Object.defineProperty(this,"_hookSubscribers",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"_parent",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"pathAtom",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),this.environment=n,this.baseSetParent(t,r)}return Object.defineProperty(e.prototype,"subpath",{get:function(){return this._subpath},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"subpathUponDeath",{get:function(){return this._subpathUponDeath},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"pathUponDeath",{get:function(){return this._pathUponDeath},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"value",{get:function(){return this.type.getValue(this)},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"state",{get:function(){return this._state},set:function(e){var t=this.isAlive;this._state=e;var r=this.isAlive;this.aliveAtom&&t!==r&&this.aliveAtom.reportChanged()},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"fireInternalHook",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this._hookSubscribers&&this._hookSubscribers.emit(e,this,e)}}),Object.defineProperty(e.prototype,"registerHook",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return this._hookSubscribers||(this._hookSubscribers=new ci),this._hookSubscribers.register(e,t)}}),Object.defineProperty(e.prototype,"parent",{get:function(){return this._parent},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"baseSetParent",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){this._parent=e,this._subpath=t,this._escapedSubpath=void 0,this.pathAtom&&this.pathAtom.reportChanged()}}),Object.defineProperty(e.prototype,"path",{get:function(){return this.getEscapedPath(!0)},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"getEscapedPath",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e&&(this.pathAtom||(this.pathAtom=$("path")),this.pathAtom.reportObserved()),this.parent?(void 0===this._escapedSubpath&&(this._escapedSubpath=this._subpath?bi(this._subpath):""),this.parent.getEscapedPath(e)+"/"+this._escapedSubpath):""}}),Object.defineProperty(e.prototype,"isRoot",{get:function(){return null===this.parent},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"isAlive",{get:function(){return this.state!==Mn.DEAD},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"isDetaching",{get:function(){return this.state===Mn.DETACHING},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"observableIsAlive",{get:function(){return this.aliveAtom||(this.aliveAtom=$("alive")),this.aliveAtom.reportObserved(),this.isAlive},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"baseFinalizeCreation",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(this.state===Mn.CREATED){if(this.parent){if(this.parent.state!==Mn.FINALIZED)return;this.fireHook(Jr.afterAttach)}this.state=Mn.FINALIZED,e&&e()}}}),Object.defineProperty(e.prototype,"baseFinalizeDeath",{enumerable:!1,configurable:!0,writable:!0,value:function(){this._hookSubscribers&&this._hookSubscribers.clearAll(),this._subpathUponDeath=this._subpath,this._pathUponDeath=this.getEscapedPath(!1),this.baseSetParent(null,""),this.state=Mn.DEAD}}),Object.defineProperty(e.prototype,"baseAboutToDie",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.fireHook(Jr.beforeDestroy)}}),e}(),pn=function(e){function t(t,r,n,i,o){var a=e.call(this,t,r,n,i)||this;try{a.storedValue=t.createNewInstance(o)}catch(e){throw a.state=Mn.DEAD,e}return a.state=Mn.CREATED,a.finalizeCreation(),a}return tn(t,e),Object.defineProperty(t.prototype,"root",{get:function(){if(!this.parent)throw Jn("This scalar node is not part of a tree");return this.parent.root},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"setParent",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.parent!==e,n=this.subpath!==t;(r||n)&&(this.environment=void 0,this.baseSetParent(this.parent,t))}}),Object.defineProperty(t.prototype,"snapshot",{get:function(){return this.getSnapshot()},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.type.getSnapshot(this)}}),Object.defineProperty(t.prototype,"toString",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=(this.isAlive?this.path:this.pathUponDeath)||"<root>";return this.type.name+"@"+e+(this.isAlive?"":" [dead]")}}),Object.defineProperty(t.prototype,"die",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.isAlive&&this.state!==Mn.DETACHING&&(this.aboutToDie(),this.finalizeDeath())}}),Object.defineProperty(t.prototype,"finalizeCreation",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.baseFinalizeCreation()}}),Object.defineProperty(t.prototype,"aboutToDie",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.baseAboutToDie()}}),Object.defineProperty(t.prototype,"finalizeDeath",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.baseFinalizeDeath()}}),Object.defineProperty(t.prototype,"fireHook",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.fireInternalHook(e)}}),t}(fn);pn.prototype.die=Ot(pn.prototype.die);var hn,bn,dn=1,vn={onError:function(e){throw e}},yn=function(e){function t(t,r,n,i,o){var a=e.call(this,t,r,n,i)||this;if(Object.defineProperty(a,"nodeId",{enumerable:!0,configurable:!0,writable:!0,value:++dn}),Object.defineProperty(a,"identifierAttribute",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"identifier",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"unnormalizedIdentifier",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"identifierCache",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"isProtectionEnabled",{enumerable:!0,configurable:!0,writable:!0,value:!0}),Object.defineProperty(a,"middlewares",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_applyPatches",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_applySnapshot",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_autoUnbox",{enumerable:!0,configurable:!0,writable:!0,value:!0}),Object.defineProperty(a,"_isRunningAction",{enumerable:!0,configurable:!0,writable:!0,value:!1}),Object.defineProperty(a,"_hasSnapshotReaction",{enumerable:!0,configurable:!0,writable:!0,value:!1}),Object.defineProperty(a,"_observableInstanceState",{enumerable:!0,configurable:!0,writable:!0,value:0}),Object.defineProperty(a,"_childNodes",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_initialSnapshot",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_cachedInitialSnapshot",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_cachedInitialSnapshotCreated",{enumerable:!0,configurable:!0,writable:!0,value:!1}),Object.defineProperty(a,"_snapshotComputed",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_snapshotUponDeath",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(a,"_internalEvents",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),a._snapshotComputed=Ae((function(){return a.getSnapshot()})),a.unbox=a.unbox.bind(a),a._initialSnapshot=o,a.identifierAttribute=t.identifierAttribute,r||(a.identifierCache=new Fn),a._childNodes=t.initializeChildNodes(a,a._initialSnapshot),a.identifier=null,a.unnormalizedIdentifier=null,a.identifierAttribute&&a._initialSnapshot){var s=a._initialSnapshot[a.identifierAttribute];if(void 0===s){var u=a._childNodes[a.identifierAttribute];u&&(s=u.value)}if("string"!=typeof s&&"number"!=typeof s)throw Jn("Instance identifier '"+a.identifierAttribute+"' for type '"+a.type.name+"' must be a string or a number");a.identifier=ho(s),a.unnormalizedIdentifier=s}return r?r.root.identifierCache.addNodeToCache(a):a.identifierCache.addNodeToCache(a),a}return tn(t,e),Object.defineProperty(t.prototype,"applyPatches",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.createObservableInstanceIfNeeded(),this._applyPatches(e)}}),Object.defineProperty(t.prototype,"applySnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.createObservableInstanceIfNeeded(),this._applySnapshot(e)}}),Object.defineProperty(t.prototype,"createObservableInstanceIfNeeded",{enumerable:!1,configurable:!0,writable:!0,value:function(){0===this._observableInstanceState&&this.createObservableInstance()}}),Object.defineProperty(t.prototype,"createObservableInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e,t;this._observableInstanceState=1;for(var r=[],n=this.parent;n&&0===n._observableInstanceState;)r.unshift(n),n=n.parent;try{for(var i=nn(r),o=i.next();!o.done;o=i.next()){o.value.createObservableInstanceIfNeeded()}}catch(t){e={error:t}}finally{try{o&&!o.done&&(t=i.return)&&t.call(i)}finally{if(e)throw e.error}}var a=this.type;try{this.storedValue=a.createNewInstance(this._childNodes),this.preboot(),this._isRunningAction=!0,a.finalizeNewInstance(this,this.storedValue)}catch(e){throw this.state=Mn.DEAD,e}finally{this._isRunningAction=!1}this._observableInstanceState=2,this._snapshotComputed.trackAndCompute(),this.isRoot&&this._addSnapshotReaction(),this._childNodes=Xn,this.state=Mn.CREATED,this.fireHook(Jr.afterCreate),this.finalizeCreation()}}),Object.defineProperty(t.prototype,"root",{get:function(){var e=this.parent;return e?e.root:this},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"clearParent",{enumerable:!1,configurable:!0,writable:!0,value:function(){if(this.parent){this.fireHook(Jr.beforeDetach);var e=this.state;this.state=Mn.DETACHING;var t=this.root,r=t.environment,n=t.identifierCache.splitCache(this);try{this.parent.removeChild(this.subpath),this.baseSetParent(null,""),this.environment=r,this.identifierCache=n}finally{this.state=e}}}}),Object.defineProperty(t.prototype,"setParent",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=e!==this.parent,n=t!==this.subpath;(r||n)&&(r?(this.environment=void 0,e.root.identifierCache.mergeCache(this),this.baseSetParent(e,t),this.fireHook(Jr.afterAttach)):n&&this.baseSetParent(this.parent,t))}}),Object.defineProperty(t.prototype,"fireHook",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;this.fireInternalHook(e);var r=this.storedValue&&"object"==typeof this.storedValue&&this.storedValue[e];"function"==typeof r&&(Pt?Pt((function(){r.apply(t.storedValue)})):r.apply(this.storedValue))}}),Object.defineProperty(t.prototype,"snapshot",{get:function(){return this._snapshotComputed.get()},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.isAlive?2===this._observableInstanceState?this._getActualSnapshot():this._getCachedInitialSnapshot():this._snapshotUponDeath}}),Object.defineProperty(t.prototype,"_getActualSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.type.getSnapshot(this)}}),Object.defineProperty(t.prototype,"_getCachedInitialSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){if(!this._cachedInitialSnapshotCreated){var e=this.type,t=this._childNodes,r=this._initialSnapshot;this._cachedInitialSnapshot=e.processInitialSnapshot(t,r),this._cachedInitialSnapshotCreated=!0}return this._cachedInitialSnapshot}}),Object.defineProperty(t.prototype,"isRunningAction",{enumerable:!1,configurable:!0,writable:!0,value:function(){return!!this._isRunningAction||!this.isRoot&&this.parent.isRunningAction()}}),Object.defineProperty(t.prototype,"assertAlive",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t;if(!this.isAlive){var r=this._getAssertAliveError(e);t=r,console.warn(new Error("[mobx-state-tree] "+t))}}}),Object.defineProperty(t.prototype,"_getAssertAliveError",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.getEscapedPath(!1)||this.pathUponDeath||"",r=e.subpath&&bi(e.subpath)||"",n=e.actionContext||Pn;n&&"action"!==n.type&&n.parentActionEvent&&(n=n.parentActionEvent);var i="";n&&null!=n.name&&(i=(n&&n.context&&Bn(n.context).path||t)+"."+n.name+"()");return"You are trying to read or write to an object that is no longer part of a state tree. (Object type: '"+this.type.name+"', Path upon death: '"+t+"', Subpath: '"+r+"', Action: '"+i+"'). Either detach nodes first, or don't use objects after removing / replacing them in the tree."}}),Object.defineProperty(t.prototype,"getChildNode",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.assertAlive({subpath:e}),this._autoUnbox=!1;try{return 2===this._observableInstanceState?this.type.getChildNode(this,e):this._childNodes[e]}finally{this._autoUnbox=!0}}}),Object.defineProperty(t.prototype,"getChildren",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.assertAlive(Xn),this._autoUnbox=!1;try{return 2===this._observableInstanceState?this.type.getChildren(this):Wn(this._childNodes)}finally{this._autoUnbox=!0}}}),Object.defineProperty(t.prototype,"getChildType",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this.type.getChildType(e)}}),Object.defineProperty(t.prototype,"isProtected",{get:function(){return this.root.isProtectionEnabled},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"assertWritable",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(this.assertAlive(e),!this.isRunningAction()&&this.isProtected)throw Jn("Cannot modify '"+this+"', the object is protected and can only be modified by using an action.")}}),Object.defineProperty(t.prototype,"removeChild",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.type.removeChild(this,e)}}),Object.defineProperty(t.prototype,"unbox",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e?(this.assertAlive({subpath:e.subpath||e.subpathUponDeath}),this._autoUnbox?e.value:e):e}}),Object.defineProperty(t.prototype,"toString",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=(this.isAlive?this.path:this.pathUponDeath)||"<root>",t=this.identifier?"(id: "+this.identifier+")":"";return this.type.name+"@"+e+t+(this.isAlive?"":" [dead]")}}),Object.defineProperty(t.prototype,"finalizeCreation",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this;this.baseFinalizeCreation((function(){var t,r;try{for(var n=nn(e.getChildren()),i=n.next();!i.done;i=n.next()){i.value.finalizeCreation()}}catch(e){t={error:e}}finally{try{i&&!i.done&&(r=n.return)&&r.call(n)}finally{if(t)throw t.error}}e.fireInternalHook(Jr.afterCreationFinalization)}))}}),Object.defineProperty(t.prototype,"detach",{enumerable:!1,configurable:!0,writable:!0,value:function(){if(!this.isAlive)throw Jn("Error while detaching, node is not alive.");this.clearParent()}}),Object.defineProperty(t.prototype,"preboot",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this;this._applyPatches=Tn(this.storedValue,"@APPLY_PATCHES",(function(t){t.forEach((function(t){if(t.path){var r=function(e){var t=e.split("/").map(di);if(!(""===e||"."===e||".."===e||fi(e,"/")||fi(e,"./")||fi(e,"../")))throw Jn("a json path must be either rooted, empty or relative, but got '"+e+"'");""===t[0]&&t.shift();return t}(t.path);qn(e,r.slice(0,-1)).applyPatchLocally(r[r.length-1],t)}else e.type.applySnapshot(e,t.value)}))})),this._applySnapshot=Tn(this.storedValue,"@APPLY_SNAPSHOT",(function(t){if(t!==e.snapshot)return e.type.applySnapshot(e,t)})),si(this.storedValue,"$treenode",this),si(this.storedValue,"toJSON",Kn)}}),Object.defineProperty(t.prototype,"die",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.isAlive&&this.state!==Mn.DETACHING&&(this.aboutToDie(),this.finalizeDeath())}}),Object.defineProperty(t.prototype,"aboutToDie",{enumerable:!1,configurable:!0,writable:!0,value:function(){0!==this._observableInstanceState&&(this.getChildren().forEach((function(e){e.aboutToDie()})),this.baseAboutToDie(),this._internalEventsEmit("dispose"),this._internalEventsClear("dispose"))}}),Object.defineProperty(t.prototype,"finalizeDeath",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.getChildren().forEach((function(e){e.finalizeDeath()})),this.root.identifierCache.notifyDied(this);var e=this.snapshot;this._snapshotUponDeath=e,this._internalEventsClearAll(),this.baseFinalizeDeath()}}),Object.defineProperty(t.prototype,"onSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._addSnapshotReaction(),this._internalEventsRegister("snapshot",e)}}),Object.defineProperty(t.prototype,"emitSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this._internalEventsEmit("snapshot",e)}}),Object.defineProperty(t.prototype,"onPatch",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._internalEventsRegister("patch",e)}}),Object.defineProperty(t.prototype,"emitPatch",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(this._internalEventsHasSubscribers("patch")){var r=on(function(e){if(!("oldValue"in e))throw Jn("Patches without `oldValue` field cannot be inversed");return[pi(e),hi(e)]}(function(e){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];for(var n=0;n<t.length;n++){var i=t[n];for(var o in i)e[o]=i[o]}return e}({},e,{path:t.path.substr(this.path.length)+"/"+e.path})),2),n=r[0],i=r[1];this._internalEventsEmit("patch",n,i)}this.parent&&this.parent.emitPatch(e,t)}}),Object.defineProperty(t.prototype,"hasDisposer",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._internalEventsHas("dispose",e)}}),Object.defineProperty(t.prototype,"addDisposer",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(this.hasDisposer(e))throw Jn("cannot add a disposer when it is already registered for execution");this._internalEventsRegister("dispose",e,!0)}}),Object.defineProperty(t.prototype,"removeDisposer",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(!this._internalEventsHas("dispose",e))throw Jn("cannot remove a disposer which was never registered for execution");this._internalEventsUnregister("dispose",e)}}),Object.defineProperty(t.prototype,"removeMiddleware",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(this.middlewares){var t=this.middlewares.indexOf(e);t>=0&&this.middlewares.splice(t,1)}}}),Object.defineProperty(t.prototype,"addMiddleWare",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this;void 0===t&&(t=!0);var n={handler:e,includeHooks:t};return this.middlewares?this.middlewares.push(n):this.middlewares=[n],function(){r.removeMiddleware(n)}}}),Object.defineProperty(t.prototype,"applyPatchLocally",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){this.assertWritable({subpath:e}),this.createObservableInstanceIfNeeded(),this.type.applyPatchLocally(this,e,t)}}),Object.defineProperty(t.prototype,"_addSnapshotReaction",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this;if(!this._hasSnapshotReaction){var t=Et((function(){return e.snapshot}),(function(t){return e.emitSnapshot(t)}),vn);this.addDisposer(t),this._hasSnapshotReaction=!0}}}),Object.defineProperty(t.prototype,"_internalEventsHasSubscribers",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return!!this._internalEvents&&this._internalEvents.hasSubscribers(e)}}),Object.defineProperty(t.prototype,"_internalEventsRegister",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r){return void 0===r&&(r=!1),this._internalEvents||(this._internalEvents=new ci),this._internalEvents.register(e,t,r)}}),Object.defineProperty(t.prototype,"_internalEventsHas",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return!!this._internalEvents&&this._internalEvents.has(e,t)}}),Object.defineProperty(t.prototype,"_internalEventsUnregister",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){this._internalEvents&&this._internalEvents.unregister(e,t)}}),Object.defineProperty(t.prototype,"_internalEventsEmit",{enumerable:!1,configurable:!0,writable:!0,value:function(e){for(var t,r=[],n=1;n<arguments.length;n++)r[n-1]=arguments[n];this._internalEvents&&(t=this._internalEvents).emit.apply(t,an([e],r))}}),Object.defineProperty(t.prototype,"_internalEventsClear",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this._internalEvents&&this._internalEvents.clear(e)}}),Object.defineProperty(t.prototype,"_internalEventsClearAll",{enumerable:!1,configurable:!0,writable:!0,value:function(){this._internalEvents&&this._internalEvents.clearAll()}}),t}(fn);yn.prototype.createObservableInstance=Ot(yn.prototype.createObservableInstance),yn.prototype.detach=Ot(yn.prototype.detach),yn.prototype.die=Ot(yn.prototype.die),function(e){e[e.String=1]="String",e[e.Number=2]="Number",e[e.Boolean=4]="Boolean",e[e.Date=8]="Date",e[e.Literal=16]="Literal",e[e.Array=32]="Array",e[e.Map=64]="Map",e[e.Object=128]="Object",e[e.Frozen=256]="Frozen",e[e.Optional=512]="Optional",e[e.Reference=1024]="Reference",e[e.Identifier=2048]="Identifier",e[e.Late=4096]="Late",e[e.Refinement=8192]="Refinement",e[e.Union=16384]="Union",e[e.Null=32768]="Null",e[e.Undefined=65536]="Undefined",e[e.Integer=131072]="Integer",e[e.Custom=262144]="Custom",e[e.SnapshotProcessor=524288]="SnapshotProcessor"}(bn||(bn={}));var gn="cannotDetermine",mn=Symbol("$type"),_n=function(){function e(e){Object.defineProperty(this,hn,{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"C",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"S",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"T",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"N",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"isType",{enumerable:!0,configurable:!0,writable:!0,value:!0}),Object.defineProperty(this,"name",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),this.name=e}return Object.defineProperty(e.prototype,"create",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return Un(this,e),this.instantiate(null,"",t,e).value}}),Object.defineProperty(e.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){throw Jn("unimplemented method")}}),Object.defineProperty(e.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e===this}}),Object.defineProperty(e.prototype,"validate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(Gn(e)){var r=sn(e);return this.isAssignableFrom(r)?Nn():Vn(t,e)}return this.isValidSnapshot(e,t)}}),Object.defineProperty(e.prototype,"is",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return 0===this.validate(e,[{path:"",type:this}]).length}}),Object.defineProperty(e.prototype,"Type",{get:function(){throw Jn("Factory.Type should not be actually called. It is just a Type signature that can be used at compile time with Typescript, by using `typeof type.Type`")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"TypeWithoutSTN",{get:function(){throw Jn("Factory.TypeWithoutSTN should not be actually called. It is just a Type signature that can be used at compile time with Typescript, by using `typeof type.TypeWithoutSTN`")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"SnapshotType",{get:function(){throw Jn("Factory.SnapshotType should not be actually called. It is just a Type signature that can be used at compile time with Typescript, by using `typeof type.SnapshotType`")},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"CreationType",{get:function(){throw Jn("Factory.CreationType should not be actually called. It is just a Type signature that can be used at compile time with Typescript, by using `typeof type.CreationType`")},enumerable:!1,configurable:!0}),e}();hn=mn,_n.prototype.create=Ot(_n.prototype.create);var wn=function(e){function t(t){var r=e.call(this,t)||this;return Object.defineProperty(r,"identifierAttribute",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),r}return tn(t,e),Object.defineProperty(t.prototype,"create",{enumerable:!1,configurable:!0,writable:!0,value:function(t,r){return void 0===t&&(t=this.getDefaultSnapshot()),e.prototype.create.call(this,t,r)}}),Object.defineProperty(t.prototype,"getValue",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.createObservableInstanceIfNeeded(),e.storedValue}}),Object.defineProperty(t.prototype,"tryToReconcileNode",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return!e.isDetaching&&(e.snapshot===t||(!(!Hn(t)||Bn(t)!==e)||!(e.type!==this||!oi(t)||Hn(t)||e.identifierAttribute&&e.identifier!==ho(t[e.identifierAttribute]))&&(e.applySnapshot(t),!0)))}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){if(this.tryToReconcileNode(e,t))return e.setParent(r,n),e;if(e.die(),Hn(t)&&this.isAssignableFrom(sn(t))){var i=Bn(t);return i.setParent(r,n),i}return this.instantiate(r,n,void 0,t)}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){return null}}),t}(_n);wn.prototype.create=Ot(wn.prototype.create);var On=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return tn(t,e),Object.defineProperty(t.prototype,"createNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e}}),Object.defineProperty(t.prototype,"getValue",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.storedValue}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.storedValue}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){if(!e.isDetaching&&e.type===this&&e.storedValue===t)return e;var i=this.instantiate(r,n,void 0,t);return e.die(),i}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){return null}}),t}(_n);function jn(e){return"object"==typeof e&&e&&!0===e.isType}!function(){function e(e,t){Object.defineProperty(this,"hooks",{enumerable:!0,configurable:!0,writable:!0,value:e}),Object.defineProperty(this,"call",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(this,"flowsPending",{enumerable:!0,configurable:!0,writable:!0,value:0}),Object.defineProperty(this,"running",{enumerable:!0,configurable:!0,writable:!0,value:!0}),e&&e.onStart(t)}Object.defineProperty(e.prototype,"finish",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.running&&(this.running=!1,this.hooks&&this.hooks.onFinish(this.call,e))}}),Object.defineProperty(e.prototype,"incFlowsPending",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.flowsPending++}}),Object.defineProperty(e.prototype,"decFlowsPending",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.flowsPending--}}),Object.defineProperty(e.prototype,"hasFlowsPending",{get:function(){return this.flowsPending>0},enumerable:!1,configurable:!0})}();var Pn,Sn=1;function An(){return Sn++}function xn(e,t){var r=Bn(e.context);"action"===e.type&&r.assertAlive({actionContext:e});var n=r._isRunningAction;r._isRunningAction=!0;var i=Pn;Pn=e;try{return function(e,t,r){var n=new In(e,r);if(n.isEmpty)return Ot(r).apply(null,t.args);var i=null;function o(e){var t=n.getNextMiddleware(),a=t&&t.handler;if(!a)return Ot(r).apply(null,e.args);if(!t.includeHooks&&Jr[e.name])return o(e);function s(e,t){i=o(e),t&&(i=t(i))}function u(e){i=e}return a(e,s,u),i}return o(t)}(r,e,t)}finally{Pn=i,r._isRunningAction=n}}function En(e){if(e)return"action"===e.type?e:e.parentActionEvent}function Tn(e,t,r){var n=function(){var n=An(),i=Pn,o=En(i);return xn({type:"action",name:t,id:n,args:li(arguments),context:e,tree:ln(e),rootId:i?i.rootId:n,parentId:i?i.id:0,allParentIds:i?an(i.allParentIds,[i.id]):[],parentEvent:i,parentActionEvent:o},r)};return n._isMSTAction=!0,n}var In=function(){function e(e,t){Object.defineProperty(this,"arrayIndex",{enumerable:!0,configurable:!0,writable:!0,value:0}),Object.defineProperty(this,"inArrayIndex",{enumerable:!0,configurable:!0,writable:!0,value:0}),Object.defineProperty(this,"middlewares",{enumerable:!0,configurable:!0,writable:!0,value:[]}),t.$mst_middleware&&this.middlewares.push(t.$mst_middleware);for(var r=e;r;)r.middlewares&&this.middlewares.push(r.middlewares),r=r.parent}return Object.defineProperty(e.prototype,"isEmpty",{get:function(){return this.middlewares.length<=0},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"getNextMiddleware",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this.middlewares[this.arrayIndex];if(e){var t=e[this.inArrayIndex++];return t||(this.arrayIndex++,this.inArrayIndex=0,this.getNextMiddleware())}}}),e}();function Cn(e){return"function"==typeof e?"<function"+(e.name?" "+e.name:"")+">":Hn(e)?"<"+e+">":"`"+function(e){try{return JSON.stringify(e)}catch(e){return"<Unserializable: "+e+">"}}(e)+"`"}function kn(e){var t=e.value,r=e.context[e.context.length-1].type,n=e.context.map((function(e){return e.path})).filter((function(e){return e.length>0})).join("/"),i=n.length>0?'at path "/'+n+'" ':"",o=Hn(t)?"value of type "+Bn(t).type.name+":":ai(t)?"value":"snapshot",a=r&&Hn(t)&&r.is(Bn(t).snapshot);return""+i+o+" "+Cn(t)+" is not assignable "+(r?"to type: `"+r.name+"`":"")+(e.message?" ("+e.message+")":"")+(r?function(e){return jn(e)&&(e.flags&(bn.String|bn.Number|bn.Integer|bn.Boolean|bn.Date))>0}(r)||ai(t)?".":", expected an instance of `"+r.name+"` or a snapshot like `"+r.describe()+"` instead."+(a?" (Note that a snapshot of the provided value is compatible with the targeted type)":""):".")}function Dn(e,t,r){return e.concat([{path:t,type:r}])}function Nn(){return Qn}function Vn(e,t,r){return[{context:e,value:t,message:r}]}function Rn(e){return e.reduce((function(e,t){return e.concat(t)}),[])}function Un(e,t){"undefined"!=typeof process&&process.env&&"true"===process.env.ENABLE_TYPE_CHECK&&function(e,t){var r=e.validate(t,[{path:"",type:e}]);if(r.length>0)throw Jn(function(e,t,r){if(0===r.length)return;return"Error while converting "+(n=Cn(t),(n.length<280?n:n.substring(0,272)+"......"+n.substring(n.length-8))+" to `")+e.name+"`:\n\n    "+r.map(kn).join("\n    ");var n}(e,t,r))}(e,t)}var Mn,zn=0,Fn=function(){function t(){Object.defineProperty(this,"cacheId",{enumerable:!0,configurable:!0,writable:!0,value:zn++}),Object.defineProperty(this,"cache",{enumerable:!0,configurable:!0,writable:!0,value:Oe.map()}),Object.defineProperty(this,"lastCacheModificationPerId",{enumerable:!0,configurable:!0,writable:!0,value:Oe.map()})}return Object.defineProperty(t.prototype,"updateLastCacheModificationPerId",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.lastCacheModificationPerId.get(e);this.lastCacheModificationPerId.set(e,void 0===t?1:t+1)}}),Object.defineProperty(t.prototype,"getLastCacheModificationPerId",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.lastCacheModificationPerId.get(e)||0;return this.cacheId+"-"+t}}),Object.defineProperty(t.prototype,"addNodeToCache",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(void 0===t&&(t=!0),e.identifierAttribute){var r=e.identifier;this.cache.has(r)||this.cache.set(r,Oe.array([],Zn));var n=this.cache.get(r);if(-1!==n.indexOf(e))throw Jn("Already registered");n.push(e),t&&this.updateLastCacheModificationPerId(r)}}}),Object.defineProperty(t.prototype,"mergeCache",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;Ht(e.identifierCache.cache).forEach((function(e){return e.forEach((function(e){t.addNodeToCache(e)}))}))}}),Object.defineProperty(t.prototype,"notifyDied",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(e.identifierAttribute){var t=e.identifier,r=this.cache.get(t);r&&(r.remove(e),r.length||this.cache.delete(t),this.updateLastCacheModificationPerId(e.identifier))}}}),Object.defineProperty(t.prototype,"splitCache",{enumerable:!1,configurable:!0,writable:!0,value:function(r){var n,i=this,o=new t,a=r.path;return(n=this.cache,Nr(n)?$t(n).map((function(e){return[e,n[e]]})):Pr(n)?$t(n).map((function(e){return[e,n.get(e)]})):xr(n)?Array.from(n.entries()):yr(n)?n.map((function(e,t){return[t,e]})):void e(7)).forEach((function(e){for(var t=on(e,2),r=t[0],n=t[1],s=!1,u=n.length-1;u>=0;u--)0===n[u].path.indexOf(a)&&(o.addNodeToCache(n[u],!1),n.splice(u,1),s=!0);s&&i.updateLastCacheModificationPerId(r)})),o}}),Object.defineProperty(t.prototype,"has",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.cache.get(t);return!!r&&r.some((function(t){return e.isAssignableFrom(t.type)}))}}),Object.defineProperty(t.prototype,"resolve",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.cache.get(t);if(!r)return null;var n=r.filter((function(t){return e.isAssignableFrom(t.type)}));switch(n.length){case 0:return null;case 1:return n[0];default:throw Jn("Cannot resolve a reference to type '"+e.name+"' with id: '"+t+"' unambigously, there are multiple candidates: "+n.map((function(e){return e.path})).join(", "))}}}),t}();function Ln(e,t,r,n,i){var o=Gn(i);if(o){if(o.parent)throw Jn("Cannot add an object to a state tree if it is already part of the same or another state tree. Tried to assign an object to '"+(t?t.path:"")+"/"+r+"', but it lives already at '"+o.path+"'");return t&&o.setParent(t,r),o}return new yn(e,t,r,n,i)}function $n(e,t,r,n,i){return new pn(e,t,r,n,i)}function Hn(e){return!(!e||!e.$treenode)}function Bn(e){if(!Hn(e))throw Jn("Value "+e+" is no MST Node");return e.$treenode}function Gn(e){return e&&e.$treenode||null}function Kn(){return Bn(this).snapshot}function qn(e,t,r){void 0===r&&(r=!0);for(var n=e,i=0;i<t.length;i++){var o=t[i];if(".."===o){if(n=n.parent)continue}else{if("."===o)continue;if(n){if(n instanceof pn)try{var a=n.value;Hn(a)&&(n=Bn(a))}catch(e){if(!r)return;throw e}if(n instanceof yn)if(n.getChildType(o)&&(n=n.getChildNode(o)))continue}}if(r)throw Jn("Could not resolve '"+o+"' in path '"+(vi(t.slice(0,i))||"/")+"' while resolving '"+vi(t)+"'");return}return n}function Wn(e){if(!e)return Qn;var t=Object.keys(e);if(!t.length)return Qn;var r=new Array(t.length);return t.forEach((function(t,n){r[n]=e[t]})),r}!function(e){e[e.INITIALIZING=0]="INITIALIZING",e[e.CREATED=1]="CREATED",e[e.FINALIZED=2]="FINALIZED",e[e.DETACHING=3]="DETACHING",e[e.DEAD=4]="DEAD"}(Mn||(Mn={}));var Yn=Object.toString(),Qn=Object.freeze([]),Xn=Object.freeze({}),Zn=rt.useProxies?{deep:!1}:{deep:!1,proxy:!1};function Jn(e){return void 0===e&&(e="Illegal state"),new Error("[mobx-state-tree] "+e)}function ei(e){return e}Object.freeze(Zn);var ti=Number.isInteger||function(e){return"number"==typeof e&&isFinite(e)&&Math.floor(e)===e};function ri(e){return Array.isArray(e)||yr(e)}function ni(e){return e?ri(e)?e:[e]:Qn}function ii(e){var t;if(null===e||"object"!=typeof e)return!1;var r=Object.getPrototypeOf(e);return null==r||(null===(t=r.constructor)||void 0===t?void 0:t.toString())===Yn}function oi(e){return!(null===e||"object"!=typeof e||e instanceof Date||e instanceof RegExp)}function ai(e,t){return void 0===t&&(t=!0),null==e||!!("string"==typeof e||"number"==typeof e||"boolean"==typeof e||t&&e instanceof Date)}function si(e,t,r){!function(e,t,r){Nr(e)?Gt(e,t,r):Object.defineProperty(e,t,r)}(e,t,{enumerable:!1,writable:!1,configurable:!0,value:r})}var ui=function(){function e(){Object.defineProperty(this,"handlers",{enumerable:!0,configurable:!0,writable:!0,value:[]})}return Object.defineProperty(e.prototype,"hasSubscribers",{get:function(){return this.handlers.length>0},enumerable:!1,configurable:!0}),Object.defineProperty(e.prototype,"register",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this;return void 0===t&&(t=!1),t?this.handlers.unshift(e):this.handlers.push(e),function(){r.unregister(e)}}}),Object.defineProperty(e.prototype,"has",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this.handlers.indexOf(e)>=0}}),Object.defineProperty(e.prototype,"unregister",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.handlers.indexOf(e);t>=0&&this.handlers.splice(t,1)}}),Object.defineProperty(e.prototype,"clear",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.handlers.length=0}}),Object.defineProperty(e.prototype,"emit",{enumerable:!1,configurable:!0,writable:!0,value:function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var r=this.handlers.slice();r.forEach((function(t){return t.apply(void 0,an(e))}))}}),e}(),ci=function(){function e(){Object.defineProperty(this,"eventHandlers",{enumerable:!0,configurable:!0,writable:!0,value:void 0})}return Object.defineProperty(e.prototype,"hasSubscribers",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.eventHandlers&&this.eventHandlers[e];return!!t&&t.hasSubscribers}}),Object.defineProperty(e.prototype,"register",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r){void 0===r&&(r=!1),this.eventHandlers||(this.eventHandlers={});var n=this.eventHandlers[e];return n||(n=this.eventHandlers[e]=new ui),n.register(t,r)}}),Object.defineProperty(e.prototype,"has",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.eventHandlers&&this.eventHandlers[e];return!!r&&r.has(t)}}),Object.defineProperty(e.prototype,"unregister",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.eventHandlers&&this.eventHandlers[e];r&&r.unregister(t)}}),Object.defineProperty(e.prototype,"clear",{enumerable:!1,configurable:!0,writable:!0,value:function(e){this.eventHandlers&&delete this.eventHandlers[e]}}),Object.defineProperty(e.prototype,"clearAll",{enumerable:!1,configurable:!0,writable:!0,value:function(){this.eventHandlers=void 0}}),Object.defineProperty(e.prototype,"emit",{enumerable:!1,configurable:!0,writable:!0,value:function(e){for(var t,r=[],n=1;n<arguments.length;n++)r[n-1]=arguments[n];var i=this.eventHandlers&&this.eventHandlers[e];i&&(t=i).emit.apply(t,an(r))}}),e}();function li(e){for(var t=new Array(e.length),r=0;r<e.length;r++)t[r]=e[r];return t}function fi(e,t){return 0===e.indexOf(t)}function pi(e){switch(e.op){case"add":return{op:"add",path:e.path,value:e.value};case"remove":return{op:"remove",path:e.path};case"replace":return{op:"replace",path:e.path,value:e.value}}}function hi(e){switch(e.op){case"add":return{op:"remove",path:e.path};case"remove":return{op:"add",path:e.path,value:e.oldValue};case"replace":return{op:"replace",path:e.path,value:e.oldValue}}}function bi(e){return!0==("number"==typeof e)?""+e:-1===e.indexOf("/")&&-1===e.indexOf("~")?e:e.replace(/~/g,"~0").replace(/\//g,"~1")}function di(e){return e.replace(/~1/g,"/").replace(/~0/g,"~")}function vi(e){if(0===e.length)return"";var t=function(e){return e.map(bi).join("/")};return"."===e[0]||".."===e[0]?t(e):"/"+t(e)}var yi=function(e){function t(t,r,n){var i=e.call(this,n||t.name)||this;return Object.defineProperty(i,"_subtype",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(i,"_processors",{enumerable:!0,configurable:!0,writable:!0,value:r}),i}return tn(t,e),Object.defineProperty(t.prototype,"flags",{get:function(){return this._subtype.flags|bn.SnapshotProcessor},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return"snapshotProcessor("+this._subtype.describe()+")"}}),Object.defineProperty(t.prototype,"preProcessSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._processors.preProcessor?this._processors.preProcessor.call(null,e):e}}),Object.defineProperty(t.prototype,"postProcessSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._processors.postProcessor?this._processors.postProcessor.call(null,e):e}}),Object.defineProperty(t.prototype,"_fixNode",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;!function(e,t){for(var r,n,i=[],o=2;o<arguments.length;o++)i[o-2]=arguments[o];try{for(var a=nn(i),s=a.next();!s.done;s=a.next()){var u=s.value;e[u]=t[u].bind(t)}}catch(e){r={error:e}}finally{try{s&&!s.done&&(n=a.return)&&n.call(a)}finally{if(r)throw r.error}}}(e.type,this,"create");var r=e.getSnapshot;e.getSnapshot=function(){return t.postProcessSnapshot(r.call(e))}}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=Hn(n)?n:this.preProcessSnapshot(n),o=this._subtype.instantiate(e,t,r,i);return this._fixNode(o),o}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=this._subtype.reconcile(e,Hn(t)?t:this.preProcessSnapshot(t),r,n);return i!==e&&this._fixNode(i),i}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){void 0===t&&(t=!0);var r=this._subtype.getSnapshot(e);return t?this.postProcessSnapshot(r):r}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.preProcessSnapshot(e);return this._subtype.validate(r,t)}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subtype}}),Object.defineProperty(t.prototype,"is",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=jn(e)?this._subtype:Hn(e)?cn(e,!1):this.preProcessSnapshot(e);return 0===this._subtype.validate(t,[{path:"",type:this._subtype}]).length}}),Object.defineProperty(t.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._subtype.isAssignableFrom(e)}}),t}(_n);var gi,mi="Map.put can only be used to store complex values that have an identifier type attribute";function _i(e,t){var r,n,i=e.getSubTypes();if(i===gn)return!1;if(i){var o=ni(i);try{for(var a=nn(o),s=a.next();!s.done;s=a.next()){if(!_i(s.value,t))return!1}}catch(e){r={error:e}}finally{try{s&&!s.done&&(n=a.return)&&n.call(a)}finally{if(r)throw r.error}}}return e instanceof ki&&t.push(e),!0}!function(e){e[e.UNKNOWN=0]="UNKNOWN",e[e.YES=1]="YES",e[e.NO=2]="NO"}(gi||(gi={}));var wi=function(e){function t(t){return e.call(this,t,Oe.ref.enhancer)||this}return tn(t,e),Object.defineProperty(t.prototype,"get",{enumerable:!1,configurable:!0,writable:!0,value:function(t){return e.prototype.get.call(this,""+t)}}),Object.defineProperty(t.prototype,"has",{enumerable:!1,configurable:!0,writable:!0,value:function(t){return e.prototype.has.call(this,""+t)}}),Object.defineProperty(t.prototype,"delete",{enumerable:!1,configurable:!0,writable:!0,value:function(t){return e.prototype.delete.call(this,""+t)}}),Object.defineProperty(t.prototype,"set",{enumerable:!1,configurable:!0,writable:!0,value:function(t,r){return e.prototype.set.call(this,""+t,r)}}),Object.defineProperty(t.prototype,"put",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(!e)throw Jn("Map.put cannot be used to set empty values");if(Hn(e)){var t=Bn(e);if(null===t.identifier)throw Jn(mi);return this.set(t.identifier,e),e}if(oi(e)){var r=Bn(this),n=r.type;if(n.identifierMode!==gi.YES)throw Jn(mi);var i=e[n.mapIdentifierAttribute];if(!bo(i)){var o=this.put(n.getChildType().create(e,r.environment));return this.put(cn(o))}var a=ho(i);return this.set(a,e),this.get(a)}throw Jn("Map.put can only be used to store complex values")}}),t}(jr),Oi=function(e){function t(t,r,n){void 0===n&&(n=[]);var i=e.call(this,t)||this;return Object.defineProperty(i,"_subType",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(i,"identifierMode",{enumerable:!0,configurable:!0,writable:!0,value:gi.UNKNOWN}),Object.defineProperty(i,"mapIdentifierAttribute",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(i,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Map}),Object.defineProperty(i,"hookInitializers",{enumerable:!0,configurable:!0,writable:!0,value:[]}),i._determineIdentifierMode(),i.hookInitializers=n,i}return tn(t,e),Object.defineProperty(t.prototype,"hooks",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var r=this.hookInitializers.length>0?this.hookInitializers.concat(e):[e];return new t(this.name,this._subType,r)}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return this._determineIdentifierMode(),Ln(this,e,t,r,n)}}),Object.defineProperty(t.prototype,"_determineIdentifierMode",{enumerable:!1,configurable:!0,writable:!0,value:function(){if(this.identifierMode===gi.UNKNOWN){var e=[];if(_i(this._subType,e)){var t=void 0;e.forEach((function(e){if(e.identifierAttribute){if(t&&t!==e.identifierAttribute)throw Jn("The objects in a map should all have the same identifier attribute, expected '"+t+"', but child of type '"+e.name+"' declared attribute '"+e.identifierAttribute+"' as identifier");t=e.identifierAttribute}})),t?(this.identifierMode=gi.YES,this.mapIdentifierAttribute=t):this.identifierMode=gi.NO}}}}),Object.defineProperty(t.prototype,"initializeChildNodes",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){void 0===t&&(t={});var r=e.type._subType,n={};return Object.keys(t).forEach((function(i){n[i]=r.instantiate(e,i,void 0,t[i])})),n}}),Object.defineProperty(t.prototype,"createNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return new wi(e)}}),Object.defineProperty(t.prototype,"finalizeNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){zt(t,e.unbox),e.type.hookInitializers.forEach((function(e){var r=e(t);Object.keys(r).forEach((function(e){var n=r[e],i=Tn(t,e,n);si(t,e,i)}))})),Ft(t,this.willChange),Kt(t,this.didChange)}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return"Map<string, "+this._subType.describe()+">"}}),Object.defineProperty(t.prototype,"getChildren",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return Ht(e.storedValue)}}),Object.defineProperty(t.prototype,"getChildNode",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=e.storedValue.get(""+t);if(!r)throw Jn("Not a child "+t);return r}}),Object.defineProperty(t.prototype,"willChange",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=Bn(e.object),r=e.name;t.assertWritable({subpath:r});var n=t.type,i=n._subType;switch(e.type){case"update":var o=e.newValue;if(o===e.object.get(r))return null;Un(i,o),e.newValue=i.reconcile(t.getChildNode(r),e.newValue,t,r),n.processIdentifier(r,e.newValue);break;case"add":Un(i,e.newValue),e.newValue=i.instantiate(t,r,void 0,e.newValue),n.processIdentifier(r,e.newValue)}return e}}),Object.defineProperty(t.prototype,"processIdentifier",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(this.identifierMode===gi.YES&&t instanceof yn){var r=t.identifier;if(r!==e)throw Jn("A map of objects containing an identifier should always store the object under their own identifier. Trying to store key '"+r+"', but expected: '"+e+"'")}}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t={};return e.getChildren().forEach((function(e){t[e.subpath]=e.snapshot})),t}}),Object.defineProperty(t.prototype,"processInitialSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t={};return Object.keys(e).forEach((function(r){t[r]=e[r].getSnapshot()})),t}}),Object.defineProperty(t.prototype,"didChange",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=Bn(e.object);switch(e.type){case"update":return void t.emitPatch({op:"replace",path:bi(e.name),value:e.newValue.snapshot,oldValue:e.oldValue?e.oldValue.snapshot:void 0},t);case"add":return void t.emitPatch({op:"add",path:bi(e.name),value:e.newValue.snapshot,oldValue:void 0},t);case"delete":var r=e.oldValue.snapshot;return e.oldValue.die(),void t.emitPatch({op:"remove",path:bi(e.name),oldValue:r},t)}}}),Object.defineProperty(t.prototype,"applyPatchLocally",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r){var n=e.storedValue;switch(r.op){case"add":case"replace":n.set(t,r.value);break;case"remove":n.delete(t)}}}),Object.defineProperty(t.prototype,"applySnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){Un(this,t);var r=e.storedValue,n={};if(Array.from(r.keys()).forEach((function(e){n[e]=!1})),t)for(var i in t)r.set(i,t[i]),n[""+i]=!0;Object.keys(n).forEach((function(e){!1===n[e]&&r.delete(e)}))}}),Object.defineProperty(t.prototype,"getChildType",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subType}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this;return ii(e)?Rn(Object.keys(e).map((function(n){return r._subType.validate(e[n],Dn(t,n,r._subType))}))):Vn(t,e,"Value is not a plain object")}}),Object.defineProperty(t.prototype,"getDefaultSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){return Xn}}),Object.defineProperty(t.prototype,"removeChild",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){e.storedValue.delete(t)}}),t}(wn);Oi.prototype.applySnapshot=Ot(Oi.prototype.applySnapshot);var ji=function(e){function t(t,r,n){void 0===n&&(n=[]);var i=e.call(this,t)||this;return Object.defineProperty(i,"_subType",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(i,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Array}),Object.defineProperty(i,"hookInitializers",{enumerable:!0,configurable:!0,writable:!0,value:[]}),i.hookInitializers=n,i}return tn(t,e),Object.defineProperty(t.prototype,"hooks",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var r=this.hookInitializers.length>0?this.hookInitializers.concat(e):[e];return new t(this.name,this._subType,r)}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return Ln(this,e,t,r,n)}}),Object.defineProperty(t.prototype,"initializeChildNodes",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){void 0===t&&(t=[]);var r=e.type._subType,n={};return t.forEach((function(t,i){var o=""+i;n[o]=r.instantiate(e,o,void 0,t)})),n}}),Object.defineProperty(t.prototype,"createNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return Oe.array(Wn(e),Zn)}}),Object.defineProperty(t.prototype,"finalizeNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){Gr(t).dehancer=e.unbox,e.type.hookInitializers.forEach((function(e){var r=e(t);Object.keys(r).forEach((function(e){var n=r[e],i=Tn(t,e,n);si(t,e,i)}))})),Ft(t,this.willChange),Kt(t,this.didChange)}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subType.describe()+"[]"}}),Object.defineProperty(t.prototype,"getChildren",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.storedValue.slice()}}),Object.defineProperty(t.prototype,"getChildNode",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=Number(t);if(r<e.storedValue.length)return e.storedValue[r];throw Jn("Not a child: "+t)}}),Object.defineProperty(t.prototype,"willChange",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=Bn(e.object);t.assertWritable({subpath:""+e.index});var r=t.type._subType,n=t.getChildren();switch(e.type){case"update":if(e.newValue===e.object[e.index])return null;var i=Pi(t,r,[n[e.index]],[e.newValue],[e.index]);if(!i)return null;e.newValue=i[0];break;case"splice":var o=e.index,a=e.removedCount,s=e.added,u=Pi(t,r,n.slice(o,o+a),s,s.map((function(e,t){return o+t})));if(!u)return null;e.added=u;for(var c=o+a;c<n.length;c++)n[c].setParent(t,""+(c+s.length-a))}return e}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.getChildren().map((function(e){return e.snapshot}))}}),Object.defineProperty(t.prototype,"processInitialSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=[];return Object.keys(e).forEach((function(r){t.push(e[r].getSnapshot())})),t}}),Object.defineProperty(t.prototype,"didChange",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=Bn(e.object);switch(e.type){case"update":return void t.emitPatch({op:"replace",path:""+e.index,value:e.newValue.snapshot,oldValue:e.oldValue?e.oldValue.snapshot:void 0},t);case"splice":for(var r=e.removedCount-1;r>=0;r--)t.emitPatch({op:"remove",path:""+(e.index+r),oldValue:e.removed[r].snapshot},t);for(r=0;r<e.addedCount;r++)t.emitPatch({op:"add",path:""+(e.index+r),value:t.getChildNode(""+(e.index+r)).snapshot,oldValue:void 0},t);return}}}),Object.defineProperty(t.prototype,"applyPatchLocally",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r){var n=e.storedValue,i="-"===t?n.length:Number(t);switch(r.op){case"replace":n[i]=r.value;break;case"add":n.splice(i,0,r.value);break;case"remove":n.splice(i,1)}}}),Object.defineProperty(t.prototype,"applySnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){Un(this,t),e.storedValue.replace(t)}}),Object.defineProperty(t.prototype,"getChildType",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subType}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this;return ri(e)?Rn(e.map((function(e,n){return r._subType.validate(e,Dn(t,""+n,r._subType))}))):Vn(t,e,"Value is not an array")}}),Object.defineProperty(t.prototype,"getDefaultSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){return Qn}}),Object.defineProperty(t.prototype,"removeChild",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){e.storedValue.splice(Number(t),1)}}),t}(wn);function Pi(e,t,r,n,i){for(var o,a=!0,s=0;;s++){var u=s<=n.length-1,c=r[s],l=u?n[s]:void 0,f=""+i[s];if(((o=l)instanceof pn||o instanceof yn)&&(l=l.storedValue),!c&&!u)break;if(u)if(c)if(Ai(c,l))r[s]=Si(t,e,f,l,c);else{for(var p=void 0,h=s;h<r.length;h++)if(Ai(r[h],l)){p=r.splice(h,1)[0];break}a=!1;b=Si(t,e,f,l,p);r.splice(s,0,b)}else{if(Hn(l)&&Bn(l).parent===e)throw Jn("Cannot add an object to a state tree if it is already part of the same or another state tree. Tried to assign an object to '"+e.path+"/"+f+"', but it lives already at '"+Bn(l).path+"'");a=!1;var b=Si(t,e,f,l);r.splice(s,0,b)}else a=!1,r.splice(s,1),c instanceof yn&&c.createObservableInstanceIfNeeded(),c.die(),s--}return a?null:r}function Si(e,t,r,n,i){Un(e,n);var o=function(){if(Hn(n)){var o=Bn(n);if(o.assertAlive(Xn),null!==o.parent&&o.parent===t)return o.setParent(t,r),o}return i?e.reconcile(i,n,t,r):e.instantiate(t,r,void 0,n)}();return i&&i!==o&&(i instanceof yn&&i.createObservableInstanceIfNeeded(),i.die()),o}function Ai(e,t){if(!e.isAlive)return!1;if(Hn(t)){var r=Bn(t);return r.isAlive&&r===e}return e.snapshot===t||e instanceof yn&&null!==e.identifier&&e.identifierAttribute&&ii(t)&&e.identifier===ho(t[e.identifierAttribute])&&e.type.is(t)}ji.prototype.applySnapshot=Ot(ji.prototype.applySnapshot);var xi="preProcessSnapshot",Ei="postProcessSnapshot";function Ti(){return Bn(this).toString()}var Ii={name:"AnonymousModel",properties:{},initializers:Qn};function Ci(e){return Object.keys(e).reduce((function(e,t){var r,n,i;if(t in Jr)throw Jn("Hook '"+t+"' was defined as property. Hooks should be defined as part of the actions");var o=Object.getOwnPropertyDescriptor(e,t);if("get"in o)throw Jn("Getters are not supported as properties. Please use views instead");var a=o.value;if(null==a)throw Jn("The default value of an attribute cannot be null or undefined as the type cannot be inferred. Did you mean `types.maybe(someType)`?");if(ai(a))return Object.assign({},e,((r={})[t]=Wi(function(e){switch(typeof e){case"string":return Ni;case"number":return Vi;case"boolean":return Ui;case"object":if(e instanceof Date)return Li}throw Jn("Cannot determine primitive type from value "+e)}(a),a),r));if(a instanceof Oi)return Object.assign({},e,((n={})[t]=Wi(a,{}),n));if(a instanceof ji)return Object.assign({},e,((i={})[t]=Wi(a,[]),i));if(jn(a))return e;throw Jn("Invalid type definition for property '"+t+"', cannot infer a type from a value like '"+a+"' ("+typeof a+")")}),e)}var ki=function(e){function t(t){var r=e.call(this,t.name||Ii.name)||this;return Object.defineProperty(r,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Object}),Object.defineProperty(r,"initializers",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(r,"properties",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(r,"preProcessor",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(r,"postProcessor",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(r,"propertyNames",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(r,"named",{enumerable:!0,configurable:!0,writable:!0,value:function(e){return r.cloneAndEnhance({name:e})}}),Object.defineProperty(r,"props",{enumerable:!0,configurable:!0,writable:!0,value:function(e){return r.cloneAndEnhance({properties:e})}}),Object.defineProperty(r,"preProcessSnapshot",{enumerable:!0,configurable:!0,writable:!0,value:function(e){var t=r.preProcessor;return t?r.cloneAndEnhance({preProcessor:function(r){return t(e(r))}}):r.cloneAndEnhance({preProcessor:e})}}),Object.defineProperty(r,"postProcessSnapshot",{enumerable:!0,configurable:!0,writable:!0,value:function(e){var t=r.postProcessor;return t?r.cloneAndEnhance({postProcessor:function(r){return e(t(r))}}):r.cloneAndEnhance({postProcessor:e})}}),Object.assign(r,Ii,t),r.properties=Ci(r.properties),r.properties,r.propertyNames=Object.keys(r.properties),r.identifierAttribute=r._getIdentifierAttribute(),r}return tn(t,e),Object.defineProperty(t.prototype,"_getIdentifierAttribute",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=void 0;return this.forAllProps((function(t,r){if(r.flags&bn.Identifier){if(e)throw Jn("Cannot define property '"+t+"' as object identifier, property '"+e+"' is already defined as identifier property");e=t}})),e}}),Object.defineProperty(t.prototype,"cloneAndEnhance",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return new t({name:e.name||this.name,properties:Object.assign({},this.properties,e.properties),initializers:this.initializers.concat(e.initializers||[]),preProcessor:e.preProcessor||this.preProcessor,postProcessor:e.postProcessor||this.postProcessor})}}),Object.defineProperty(t.prototype,"actions",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;return this.cloneAndEnhance({initializers:[function(r){return t.instantiateActions(r,e(r)),r}]})}}),Object.defineProperty(t.prototype,"instantiateActions",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(!ii(t))throw Jn("actions initializer should return a plain object containing actions");Object.keys(t).forEach((function(r){if(r===xi)throw Jn("Cannot define action 'preProcessSnapshot', it should be defined using 'type.preProcessSnapshot(fn)' instead");if(r===Ei)throw Jn("Cannot define action 'postProcessSnapshot', it should be defined using 'type.postProcessSnapshot(fn)' instead");var n=t[r],i=e[r];if(r in Jr&&i){var o=n;n=function(){i.apply(null,arguments),o.apply(null,arguments)}}var a=n.$mst_middleware,s=n.bind(t);s.$mst_middleware=a;var u=Tn(e,r,s);t[r]=u,si(e,r,u)}))}}),Object.defineProperty(t.prototype,"volatile",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;if("function"!=typeof e)throw Jn("You passed an "+typeof e+" to volatile state as an argument, when function is expected");return this.cloneAndEnhance({initializers:[function(r){return t.instantiateVolatileState(r,e(r)),r}]})}}),Object.defineProperty(t.prototype,"instantiateVolatileState",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(!ii(t))throw Jn("volatile state initializer should return a plain object containing state");Bt(e,t)}}),Object.defineProperty(t.prototype,"extend",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;return this.cloneAndEnhance({initializers:[function(r){var n=e(r),i=n.actions,o=n.views,a=n.state,s=function(e,t){var r={};for(var n in e)Object.prototype.hasOwnProperty.call(e,n)&&t.indexOf(n)<0&&(r[n]=e[n]);if(null!=e&&"function"==typeof Object.getOwnPropertySymbols){var i=0;for(n=Object.getOwnPropertySymbols(e);i<n.length;i++)t.indexOf(n[i])<0&&Object.prototype.propertyIsEnumerable.call(e,n[i])&&(r[n[i]]=e[n[i]])}return r}(n,["actions","views","state"]);for(var u in s)throw Jn("The `extend` function should return an object with a subset of the fields 'actions', 'views' and 'state'. Found invalid key '"+u+"'");return a&&t.instantiateVolatileState(r,a),o&&t.instantiateViews(r,o),i&&t.instantiateActions(r,i),r}]})}}),Object.defineProperty(t.prototype,"views",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;return this.cloneAndEnhance({initializers:[function(r){return t.instantiateViews(r,e(r)),r}]})}}),Object.defineProperty(t.prototype,"instantiateViews",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(!ii(t))throw Jn("views initializer should return a plain object containing views");Object.keys(t).forEach((function(r){var n,i=Object.getOwnPropertyDescriptor(t,r);if("get"in i)Gt(e,r,i),nr(e,((n={})[r]=Ae,n));else{if("function"!=typeof i.value)throw Jn("A view member should either be a function or getter based property");si(e,r,i.value)}}))}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return Ln(this,e,t,r,Hn(n)?n:this.applySnapshotPreProcessor(n))}}),Object.defineProperty(t.prototype,"initializeChildNodes",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){void 0===t&&(t={});var r=e.type,n={};return r.forAllProps((function(r,i){n[r]=i.instantiate(e,r,void 0,t[r])})),n}}),Object.defineProperty(t.prototype,"createNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return Oe.object(e,Xn,Zn)}}),Object.defineProperty(t.prototype,"finalizeNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){si(t,"toString",Ti),this.forAllProps((function(r){zt(t,r,e.unbox)})),this.initializers.reduce((function(e,t){return t(e)}),t),Ft(t,this.willChange),Kt(t,this.didChange)}}),Object.defineProperty(t.prototype,"willChange",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=e,r=Bn(t.object),n=t.name;r.assertWritable({subpath:n});var i=r.type.properties[n];return i&&(Un(i,t.newValue),t.newValue=i.reconcile(r.getChildNode(n),t.newValue,r,n)),t}}),Object.defineProperty(t.prototype,"didChange",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=e,r=Bn(t.object);if(r.type.properties[t.name]){var n=t.oldValue?t.oldValue.snapshot:void 0;r.emitPatch({op:"replace",path:bi(t.name),value:t.newValue.snapshot,oldValue:n},r)}}}),Object.defineProperty(t.prototype,"getChildren",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this,r=[];return this.forAllProps((function(n){r.push(t.getChildNode(e,n))})),r}}),Object.defineProperty(t.prototype,"getChildNode",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(!(t in this.properties))throw Jn("Not a value property: "+t);var r=Gr(e.storedValue,t).raw();if(!r)throw Jn("Node not available for property "+t);return r}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this;void 0===t&&(t=!0);var n={};return this.forAllProps((function(t,i){Br(e.storedValue,t).reportObserved(),n[t]=r.getChildNode(e,t).snapshot})),t?this.applySnapshotPostProcessor(n):n}}),Object.defineProperty(t.prototype,"processInitialSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t={};return Object.keys(e).forEach((function(r){t[r]=e[r].getSnapshot()})),this.applySnapshotPostProcessor(t)}}),Object.defineProperty(t.prototype,"applyPatchLocally",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r){if("replace"!==r.op&&"add"!==r.op)throw Jn("object does not support operation "+r.op);e.storedValue[t]=r.value}}),Object.defineProperty(t.prototype,"applySnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.applySnapshotPreProcessor(t);Un(this,r),this.forAllProps((function(t){e.storedValue[t]=r[t]}))}}),Object.defineProperty(t.prototype,"applySnapshotPreProcessor",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.preProcessor;return t?t.call(null,e):e}}),Object.defineProperty(t.prototype,"applySnapshotPostProcessor",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.postProcessor;return t?t.call(null,e):e}}),Object.defineProperty(t.prototype,"getChildType",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this.properties[e]}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this,n=this.applySnapshotPreProcessor(e);return ii(n)?Rn(this.propertyNames.map((function(e){return r.properties[e].validate(n[e],Dn(t,e,r.properties[e]))}))):Vn(t,n,"Value is not a plain object")}}),Object.defineProperty(t.prototype,"forAllProps",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this;this.propertyNames.forEach((function(r){return e(r,t.properties[r])}))}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this;return"{ "+this.propertyNames.map((function(t){return t+": "+e.properties[t].describe()})).join("; ")+" }"}}),Object.defineProperty(t.prototype,"getDefaultSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){return Xn}}),Object.defineProperty(t.prototype,"removeChild",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){e.storedValue[t]=void 0}}),t}(wn);ki.prototype.applySnapshot=Ot(ki.prototype.applySnapshot);var Di=function(e){function t(t,r,n,i){void 0===i&&(i=ei);var o=e.call(this,t)||this;return Object.defineProperty(o,"flags",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(o,"checker",{enumerable:!0,configurable:!0,writable:!0,value:n}),Object.defineProperty(o,"initializer",{enumerable:!0,configurable:!0,writable:!0,value:i}),o.flags=r,o}return tn(t,e),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.name}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return $n(this,e,t,r,n)}}),Object.defineProperty(t.prototype,"createNewInstance",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this.initializer(e)}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return ai(e)&&this.checker(e)?Nn():Vn(t,e,"Value is not a "+("Date"===this.name?"Date or a unix milliseconds timestamp":this.name))}}),t}(On),Ni=new Di("string",bn.String,(function(e){return"string"==typeof e})),Vi=new Di("number",bn.Number,(function(e){return"number"==typeof e})),Ri=new Di("integer",bn.Integer,(function(e){return ti(e)})),Ui=new Di("boolean",bn.Boolean,(function(e){return"boolean"==typeof e})),Mi=new Di("null",bn.Null,(function(e){return null===e})),zi=new Di("undefined",bn.Undefined,(function(e){return void 0===e})),Fi=new Di("Date",bn.Date,(function(e){return"number"==typeof e||e instanceof Date}),(function(e){return e instanceof Date?e:new Date(e)}));Fi.getSnapshot=function(e){return e.storedValue.getTime()};var Li=Fi;var $i=function(e){function t(t){var r=e.call(this,JSON.stringify(t))||this;return Object.defineProperty(r,"value",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(r,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Literal}),r.value=t,r}return tn(t,e),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return $n(this,e,t,r,n)}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return JSON.stringify(this.value)}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return ai(e)&&e===this.value?Nn():Vn(t,e,"Value is not a literal "+JSON.stringify(this.value))}}),t}(On);function Hi(e){return new $i(e)}var Bi=function(e){function t(t,r,n,i){var o=e.call(this,t)||this;return Object.defineProperty(o,"_subtype",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(o,"_predicate",{enumerable:!0,configurable:!0,writable:!0,value:n}),Object.defineProperty(o,"_message",{enumerable:!0,configurable:!0,writable:!0,value:i}),o}return tn(t,e),Object.defineProperty(t.prototype,"flags",{get:function(){return this._subtype.flags|bn.Refinement},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.name}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return this._subtype.instantiate(e,t,r,n)}}),Object.defineProperty(t.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._subtype.isAssignableFrom(e)}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this._subtype.validate(e,t);if(r.length>0)return r;var n=Hn(e)?Bn(e).snapshot:e;return this._predicate(n)?Nn():Vn(t,e,this._message(e))}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return this._subtype.reconcile(e,t,r,n)}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subtype}}),t}(_n);var Gi=function(e){function t(t,r,n){var i=e.call(this,t)||this;return Object.defineProperty(i,"_types",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(i,"_dispatcher",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(i,"_eager",{enumerable:!0,configurable:!0,writable:!0,value:!0}),n=rn({eager:!0,dispatcher:void 0},n),i._dispatcher=n.dispatcher,n.eager||(i._eager=!1),i}return tn(t,e),Object.defineProperty(t.prototype,"flags",{get:function(){var e=bn.Union;return this._types.forEach((function(t){e|=t.flags})),e},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._types.some((function(t){return t.isAssignableFrom(e)}))}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return"("+this._types.map((function(e){return e.describe()})).join(" | ")+")"}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=this.determineType(n,void 0);if(!i)throw Jn("No matching type for union "+this.describe());return i.instantiate(e,t,r,n)}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=this.determineType(t,e.type);if(!i)throw Jn("No matching type for union "+this.describe());return i.reconcile(e,t,r,n)}}),Object.defineProperty(t.prototype,"determineType",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return this._dispatcher?this._dispatcher(e):t?t.is(e)?t:this._types.filter((function(e){return e!==t})).find((function(t){return t.is(e)})):this._types.find((function(t){return t.is(e)}))}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(this._dispatcher)return this._dispatcher(e).validate(e,t);for(var r=[],n=0,i=0;i<this._types.length;i++){var o=this._types[i].validate(e,t);if(0===o.length){if(this._eager)return Nn();n++}else r.push(o)}return 1===n?Nn():Vn(t,e,"No type is applicable for the union").concat(Rn(r))}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._types}}),t}(_n);function Ki(e){for(var t=[],r=1;r<arguments.length;r++)t[r-1]=arguments[r];var n=jn(e)?void 0:e,i=jn(e)?an([e],t):t,o="("+i.map((function(e){return e.name})).join(" | ")+")";return new Gi(o,i,n)}var qi=function(e){function t(t,r,n){var i=e.call(this,t.name)||this;return Object.defineProperty(i,"_subtype",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(i,"_defaultValue",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(i,"optionalValues",{enumerable:!0,configurable:!0,writable:!0,value:n}),i}return tn(t,e),Object.defineProperty(t.prototype,"flags",{get:function(){return this._subtype.flags|bn.Optional},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subtype.describe()+"?"}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){if(this.optionalValues.indexOf(n)>=0){var i=this.getDefaultInstanceOrSnapshot();return this._subtype.instantiate(e,t,r,i)}return this._subtype.instantiate(e,t,r,n)}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return this._subtype.reconcile(e,this.optionalValues.indexOf(t)<0&&this._subtype.is(t)?t:this.getDefaultInstanceOrSnapshot(),r,n)}}),Object.defineProperty(t.prototype,"getDefaultInstanceOrSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e="function"==typeof this._defaultValue?this._defaultValue():this._defaultValue;return"function"==typeof this._defaultValue&&Un(this,e),e}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return this.optionalValues.indexOf(e)>=0?Nn():this._subtype.validate(e,t)}}),Object.defineProperty(t.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this._subtype.isAssignableFrom(e)}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this._subtype}}),t}(_n);function Wi(e,t,r){return function(e,t){if("function"!=typeof t&&Hn(t))throw Jn("default value cannot be an instance, pass a snapshot or a function that creates an instance/snapshot instead")}(0,t),new qi(e,t,r||Yi)}var Yi=[void 0],Qi=Wi(zi,void 0),Xi=Wi(Mi,null);function Zi(e){return Ki(e,Qi)}var Ji=function(e){function t(t,r){var n=e.call(this,t)||this;return Object.defineProperty(n,"_definition",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(n,"_subType",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),n}return tn(t,e),Object.defineProperty(t.prototype,"flags",{get:function(){return(this._subType?this._subType.flags:0)|bn.Late},enumerable:!1,configurable:!0}),Object.defineProperty(t.prototype,"getSubType",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(!this._subType){var t=void 0;try{t=this._definition()}catch(e){if(!(e instanceof ReferenceError))throw e;t=void 0}if(e&&void 0===t)throw Jn("Late type seems to be used too early, the definition (still) returns undefined");t&&(this._subType=t)}return this._subType}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return this.getSubType(!0).instantiate(e,t,r,n)}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return this.getSubType(!0).reconcile(e,t,r,n)}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this.getSubType(!1);return e?e.name:"<uknown late type>"}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this.getSubType(!1);return r?r.validate(e,t):Nn()}}),Object.defineProperty(t.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=this.getSubType(!1);return!!t&&t.isAssignableFrom(e)}}),Object.defineProperty(t.prototype,"getSubTypes",{enumerable:!1,configurable:!0,writable:!0,value:function(){var e=this.getSubType(!1);return e||gn}}),t}(_n);var eo=function(e){function t(t){var r=e.call(this,t?"frozen("+t.name+")":"frozen")||this;return Object.defineProperty(r,"subType",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(r,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Frozen}),r}return tn(t,e),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return"<any immutable value>"}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return $n(this,e,t,r,n)}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return function(e){return"function"!=typeof e}(e)?this.subType?this.subType.validate(e,t):Nn():Vn(t,e,"Value is not serializable and cannot be frozen")}}),t}(On),to=new eo;var ro=function(){function e(e,t){if(Object.defineProperty(this,"targetType",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(this,"identifier",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"node",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),Object.defineProperty(this,"resolvedReference",{enumerable:!0,configurable:!0,writable:!0,value:void 0}),bo(e))this.identifier=e;else{if(!Hn(e))throw Jn("Can only store references to tree nodes or identifiers, got: '"+e+"'");var r=Bn(e);if(!r.identifierAttribute)throw Jn("Can only store references with a defined identifier attribute.");var n=r.unnormalizedIdentifier;if(null==n)throw Jn("Can only store references to tree nodes with a defined identifier.");this.identifier=n}}return Object.defineProperty(e.prototype,"updateResolvedReference",{enumerable:!1,configurable:!0,writable:!0,value:function(e){var t=ho(this.identifier),r=e.root,n=r.identifierCache.getLastCacheModificationPerId(t);if(!this.resolvedReference||this.resolvedReference.lastCacheModification!==n){var i=this.targetType,o=r.identifierCache.resolve(i,t);if(!o)throw new no("[mobx-state-tree] Failed to resolve reference '"+this.identifier+"' to type '"+this.targetType.name+"' (from node: "+e.path+")");this.resolvedReference={node:o,lastCacheModification:n}}}}),Object.defineProperty(e.prototype,"resolvedValue",{get:function(){return this.updateResolvedReference(this.node),this.resolvedReference.node.value},enumerable:!1,configurable:!0}),e}(),no=function(e){function t(r){var n=e.call(this,r)||this;return Object.setPrototypeOf(n,t.prototype),n}return tn(t,e),t}(Error),io=function(e){function t(t,r){var n=e.call(this,"reference("+t.name+")")||this;return Object.defineProperty(n,"targetType",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(n,"onInvalidated",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(n,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Reference}),n}return tn(t,e),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.name}}),Object.defineProperty(t.prototype,"isAssignableFrom",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this.targetType.isAssignableFrom(e)}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return bo(e)?Nn():Vn(t,e,"Value is not a valid identifier, which is a string or a number")}}),Object.defineProperty(t.prototype,"fireInvalidated",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=t.parent;if(i&&i.isAlive){var o=i.storedValue;o&&this.onInvalidated({cause:e,parent:o,invalidTarget:n?n.storedValue:void 0,invalidId:r,replaceRef:function(e){un(t.root.storedValue,{op:"replace",value:e,path:t.path})},removeRef:function(){!function(e){return jn(e)&&(e.flags&bn.Object)>0}(i.type)?un(t.root.storedValue,{op:"remove",path:t.path}):this.replaceRef(void 0)}})}}}),Object.defineProperty(t.prototype,"addTargetNodeWatcher",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){var r=this,n=this.getValue(e);if(n){var i=Bn(n),o=function(n,o){var a=function(e){switch(e){case Jr.beforeDestroy:return"destroy";case Jr.beforeDetach:return"detach";default:return}}(o);a&&r.fireInvalidated(a,e,t,i)},a=i.registerHook(Jr.beforeDetach,o),s=i.registerHook(Jr.beforeDestroy,o);return function(){a(),s()}}}}),Object.defineProperty(t.prototype,"watchTargetNodeForInvalidations",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r){var n=this;if(this.onInvalidated){var i;e.registerHook(Jr.beforeDestroy,(function(){i&&i()}));var o=function(o){i&&i();var a=e.parent,s=a&&a.storedValue;if(a&&a.isAlive&&s){(r?!!r.get(t,s):e.root.identifierCache.has(n.targetType,ho(t)))?i=n.addTargetNodeWatcher(e,t):o||n.fireInvalidated("invalidSnapshotReference",e,t,null)}};e.state===Mn.FINALIZED?o(!0):(e.isRoot||e.root.registerHook(Jr.afterCreationFinalization,(function(){e.parent&&e.parent.createObservableInstanceIfNeeded()})),e.registerHook(Jr.afterAttach,(function(){o(!1)})))}}}),t}(On),oo=function(e){function t(t,r){return e.call(this,t,r)||this}return tn(t,e),Object.defineProperty(t.prototype,"getValue",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(e.isAlive)return e.storedValue.resolvedValue}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.storedValue.identifier}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=Hn(n)?Bn(n).identifier:n,o=new ro(n,this.targetType),a=$n(this,e,t,r,o);return o.node=a,this.watchTargetNodeForInvalidations(a,i,void 0),a}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){if(!e.isDetaching&&e.type===this){var i=Hn(t),o=e.storedValue;if(!i&&o.identifier===t||i&&o.resolvedValue===t)return e.setParent(r,n),e}var a=this.instantiate(r,n,void 0,t);return e.die(),a}}),t}(io),ao=function(e){function t(t,r,n){var i=e.call(this,t,n)||this;return Object.defineProperty(i,"options",{enumerable:!0,configurable:!0,writable:!0,value:r}),i}return tn(t,e),Object.defineProperty(t.prototype,"getValue",{enumerable:!1,configurable:!0,writable:!0,value:function(e){if(e.isAlive)return this.options.get(e.storedValue,e.parent?e.parent.storedValue:null)}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.storedValue}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=Hn(n)?this.options.set(n,e?e.storedValue:null):n,o=$n(this,e,t,r,i);return this.watchTargetNodeForInvalidations(o,i,this.options),o}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=Hn(t)?this.options.set(t,e?e.storedValue:null):t;if(!e.isDetaching&&e.type===this&&e.storedValue===i)return e.setParent(r,n),e;var o=this.instantiate(r,n,void 0,i);return e.die(),o}}),t}(io);function so(e,t){var r=t||void 0,n=t?t.onInvalidated:void 0;return r&&(r.get||r.set)?new ao(e,{get:r.get,set:r.set},n):new oo(e,n)}var uo=function(e){function t(t,r){var n=e.call(this,t)||this;return Object.defineProperty(n,"validType",{enumerable:!0,configurable:!0,writable:!0,value:r}),Object.defineProperty(n,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Identifier}),n}return tn(t,e),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){if(!(e&&e.type instanceof ki))throw Jn("Identifier types can only be instantiated as direct child of a model type");return $n(this,e,t,r,n)}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){if(e.storedValue!==t)throw Jn("Tried to change identifier from '"+e.storedValue+"' to '"+t+"'. Changing identifiers is not allowed.");return e.setParent(r,n),e}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){return typeof e!==this.validType?Vn(t,e,"Value is not a valid "+this.describe()+", expected a "+this.validType):Nn()}}),t}(On),co=function(e){function t(){var t=e.call(this,"identifier","string")||this;return Object.defineProperty(t,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Identifier}),t}return tn(t,e),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return"identifier"}}),t}(uo),lo=function(e){function t(){return e.call(this,"identifierNumber","number")||this}return tn(t,e),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return e.storedValue}}),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return"identifierNumber"}}),t}(uo),fo=new co,po=new lo;function ho(e){return""+e}function bo(e){return"string"==typeof e||"number"==typeof e}var vo,yo=function(e){function t(t){var r=e.call(this,t.name)||this;return Object.defineProperty(r,"options",{enumerable:!0,configurable:!0,writable:!0,value:t}),Object.defineProperty(r,"flags",{enumerable:!0,configurable:!0,writable:!0,value:bn.Custom}),r}return tn(t,e),Object.defineProperty(t.prototype,"describe",{enumerable:!1,configurable:!0,writable:!0,value:function(){return this.name}}),Object.defineProperty(t.prototype,"isValidSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t){if(this.options.isTargetType(e))return Nn();var r=this.options.getValidationMessage(e);return r?Vn(t,e,"Invalid value for type '"+this.name+"': "+r):Nn()}}),Object.defineProperty(t.prototype,"getSnapshot",{enumerable:!1,configurable:!0,writable:!0,value:function(e){return this.options.toSnapshot(e.storedValue)}}),Object.defineProperty(t.prototype,"instantiate",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){return $n(this,e,t,r,this.options.isTargetType(n)?n:this.options.fromSnapshot(n,e&&e.root.environment))}}),Object.defineProperty(t.prototype,"reconcile",{enumerable:!1,configurable:!0,writable:!0,value:function(e,t,r,n){var i=!this.options.isTargetType(t);if(!e.isDetaching&&(e.type===this&&(i?t===e.snapshot:t===e.storedValue)))return e.setParent(r,n),e;var o=i?this.options.fromSnapshot(t,r.root.environment):t,a=this.instantiate(r,n,void 0,o);return e.die(),a}}),t}(On),go={enumeration:function(e,t){var r="string"==typeof e?t:e,n=Ki.apply(void 0,an(r.map((function(e){return Hi(""+e)}))));return"string"==typeof e&&(n.name=e),n},model:function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var r="string"==typeof e[0]?e.shift():"AnonymousModel",n=e.shift()||{};return new ki({name:r,properties:n})},compose:function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var r="string"==typeof e[0],n=r?e[0]:"AnonymousModel";return r&&e.shift(),e.reduce((function(e,t){return e.cloneAndEnhance({name:e.name+"_"+t.name,properties:t.properties,initializers:t.initializers,preProcessor:function(r){return t.applySnapshotPreProcessor(e.applySnapshotPreProcessor(r))},postProcessor:function(r){return t.applySnapshotPostProcessor(e.applySnapshotPostProcessor(r))}})})).named(n)},custom:function(e){return new yo(e)},reference:so,safeReference:function(e,t){var r=so(e,rn(rn({},t),{onInvalidated:function(e){t&&t.onInvalidated&&t.onInvalidated(e),e.removeRef()}}));return t&&!1===t.acceptsUndefined?r:Zi(r)},union:Ki,optional:Wi,literal:Hi,maybe:Zi,maybeNull:function(e){return Ki(e,Xi)},refinement:function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var r="string"==typeof e[0]?e.shift():jn(e[0])?e[0].name:null,n=e[0],i=e[1],o=e[2]?e[2]:function(e){return"Value does not respect the refinement predicate"};return new Bi(r,n,i,o)},string:Ni,boolean:Ui,number:Vi,integer:Ri,Date:Li,map:function(e){return new Oi("map<string, "+e.name+">",e)},array:function(e){return new ji(e.name+"[]",e)},frozen:function(e){return 0===arguments.length?to:jn(e)?new eo(e):Wi(to,e)},identifier:fo,identifierNumber:po,late:function(e,t){var r="string"==typeof e?e:"late("+e.toString()+")";return new Ji(r,"string"==typeof e?t:e)},undefined:zi,null:Mi,snapshotProcessor:function(e,t,r){return new yi(e,t,r)}},mo=new Uint8Array(16);function _o(){if(!vo&&!(vo="undefined"!=typeof crypto&&crypto.getRandomValues&&crypto.getRandomValues.bind(crypto)||"undefined"!=typeof msCrypto&&"function"==typeof msCrypto.getRandomValues&&msCrypto.getRandomValues.bind(msCrypto)))throw new Error("crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported");return vo(mo)}var wo=/^(?:[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}|00000000-0000-0000-0000-000000000000)$/i;function Oo(e){return"string"==typeof e&&wo.test(e)}for(var jo=[],Po=0;Po<256;++Po)jo.push((Po+256).toString(16).substr(1));function So(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,r=(jo[e[t+0]]+jo[e[t+1]]+jo[e[t+2]]+jo[e[t+3]]+"-"+jo[e[t+4]]+jo[e[t+5]]+"-"+jo[e[t+6]]+jo[e[t+7]]+"-"+jo[e[t+8]]+jo[e[t+9]]+"-"+jo[e[t+10]]+jo[e[t+11]]+jo[e[t+12]]+jo[e[t+13]]+jo[e[t+14]]+jo[e[t+15]]).toLowerCase();if(!Oo(r))throw TypeError("Stringified UUID is invalid");return r}function Ao(e,t,r){var n=(e=e||{}).random||(e.rng||_o)();if(n[6]=15&n[6]|64,n[8]=63&n[8]|128,t){r=r||0;for(var i=0;i<16;++i)t[r+i]=n[i];return t}return So(n)}function xo(e,t,r,n){switch(e){case 0:return t&r^~t&n;case 1:return t^r^n;case 2:return t&r^t&n^r&n;case 3:return t^r^n}}function Eo(e,t){return e<<t|e>>>32-t}var To=function(e,t,r){function n(e,n,i,o){if("string"==typeof e&&(e=function(e){e=unescape(encodeURIComponent(e));for(var t=[],r=0;r<e.length;++r)t.push(e.charCodeAt(r));return t}(e)),"string"==typeof n&&(n=function(e){if(!Oo(e))throw TypeError("Invalid UUID");var t,r=new Uint8Array(16);return r[0]=(t=parseInt(e.slice(0,8),16))>>>24,r[1]=t>>>16&255,r[2]=t>>>8&255,r[3]=255&t,r[4]=(t=parseInt(e.slice(9,13),16))>>>8,r[5]=255&t,r[6]=(t=parseInt(e.slice(14,18),16))>>>8,r[7]=255&t,r[8]=(t=parseInt(e.slice(19,23),16))>>>8,r[9]=255&t,r[10]=(t=parseInt(e.slice(24,36),16))/1099511627776&255,r[11]=t/4294967296&255,r[12]=t>>>24&255,r[13]=t>>>16&255,r[14]=t>>>8&255,r[15]=255&t,r}(n)),16!==n.length)throw TypeError("Namespace must be array-like (16 iterable integer values, 0-255)");var a=new Uint8Array(16+e.length);if(a.set(n),a.set(e,n.length),(a=r(a))[6]=15&a[6]|t,a[8]=63&a[8]|128,i){o=o||0;for(var s=0;s<16;++s)i[o+s]=a[s];return i}return So(a)}try{n.name=e}catch(e){}return n.DNS="6ba7b810-9dad-11d1-80b4-00c04fd430c8",n.URL="6ba7b811-9dad-11d1-80b4-00c04fd430c8",n}("v5",80,(function(e){var t=[1518500249,1859775393,2400959708,3395469782],r=[1732584193,4023233417,2562383102,271733878,3285377520];if("string"==typeof e){var n=unescape(encodeURIComponent(e));e=[];for(var i=0;i<n.length;++i)e.push(n.charCodeAt(i))}else Array.isArray(e)||(e=Array.prototype.slice.call(e));e.push(128);for(var o=e.length/4+2,a=Math.ceil(o/16),s=new Array(a),u=0;u<a;++u){for(var c=new Uint32Array(16),l=0;l<16;++l)c[l]=e[64*u+4*l]<<24|e[64*u+4*l+1]<<16|e[64*u+4*l+2]<<8|e[64*u+4*l+3];s[u]=c}s[a-1][14]=8*(e.length-1)/Math.pow(2,32),s[a-1][14]=Math.floor(s[a-1][14]),s[a-1][15]=8*(e.length-1)&4294967295;for(var f=0;f<a;++f){for(var p=new Uint32Array(80),h=0;h<16;++h)p[h]=s[f][h];for(var b=16;b<80;++b)p[b]=Eo(p[b-3]^p[b-8]^p[b-14]^p[b-16],1);for(var d=r[0],v=r[1],y=r[2],g=r[3],m=r[4],_=0;_<80;++_){var w=Math.floor(_/20),O=Eo(d,5)+xo(w,v,y,g)+m+t[w]+p[_]>>>0;m=g,g=y,y=Eo(v,30)>>>0,v=d,d=O}r[0]=r[0]+d>>>0,r[1]=r[1]+v>>>0,r[2]=r[2]+y>>>0,r[3]=r[3]+g>>>0,r[4]=r[4]+m>>>0}return[r[0]>>24&255,r[0]>>16&255,r[0]>>8&255,255&r[0],r[1]>>24&255,r[1]>>16&255,r[1]>>8&255,255&r[1],r[2]>>24&255,r[2]>>16&255,r[2]>>8&255,255&r[2],r[3]>>24&255,r[3]>>16&255,r[3]>>8&255,255&r[3],r[4]>>24&255,r[4]>>16&255,r[4]>>8&255,255&r[4]]}));
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
const Io=go.enumeration("TaskState",["free","pending","complete","error","destroyed"]),Co={anyType:go.custom({name:"any",fromSnapshot:e=>e,toSnapshot:e=>e,isTargetType:()=>!0,getValidationMessage:()=>""})},ko=go.model({uuid:go.string,spiderUUID:go.string,dataSlide:go.array(go.string),dataSlideUUID:go.string,createdAt:go.Date,errorMessage:go.optional(go.string,""),status:Io,updatedAt:go.Date,originData:go.maybe(Co.anyType),output:go.maybe(Co.anyType)}).actions((e=>({$backup:()=>cn(e),$import(e){return Bn(e).applySnapshot(t);var t},start(){if("pending"===e.status)throw new Error("Task 处在 pending 状态");return e.dataSlide=[],e.dataSlideUUID="",e.status="pending",e.output||e.originData},success(t,r){e.status="free",e.output=t,this._addUUIDToDataSlide(r)},complete(t){e.status="complete",this._addUUIDToDataSlide(t)},error(t="",r){e.status="error",e.errorMessage=t,this._addUUIDToDataSlide(r)},destroy(){var t;(t=Bn(e)).isRoot?t.die():t.parent.removeChild(t.subpath)},_addUUIDToDataSlide(t){t&&(e.dataSlide.push(t),e.dataSlideUUID=t)}})));function Do(e){return ko.create(function({spiderUUID:e,originData:t}){return{uuid:Ao(),spiderUUID:e,createdAt:new Date,errorMessage:"",status:"free",updatedAt:new Date,dataSlideUUID:e,dataSlide:[],originData:t,output:null}}(e))}
/*! *****************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */var No=function(e,t){return(No=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(e,t){e.__proto__=t}||function(e,t){for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r])})(e,t)};function Vo(e,t){function r(){this.constructor=e}No(e,t),e.prototype=null===t?Object.create(t):(r.prototype=t.prototype,new r)}function Ro(e){return"function"==typeof e}var Uo=!1,Mo={Promise:void 0,set useDeprecatedSynchronousErrorHandling(e){e&&(new Error).stack;Uo=e},get useDeprecatedSynchronousErrorHandling(){return Uo}};function zo(e){setTimeout((function(){throw e}),0)}var Fo={closed:!0,next:function(e){},error:function(e){if(Mo.useDeprecatedSynchronousErrorHandling)throw e;zo(e)},complete:function(){}},Lo=function(){return Array.isArray||function(e){return e&&"number"==typeof e.length}}();function $o(e){return null!==e&&"object"==typeof e}var Ho=function(){function e(e){return Error.call(this),this.message=e?e.length+" errors occurred during unsubscription:\n"+e.map((function(e,t){return t+1+") "+e.toString()})).join("\n  "):"",this.name="UnsubscriptionError",this.errors=e,this}return e.prototype=Object.create(Error.prototype),e}(),Bo=function(){function e(e){this.closed=!1,this._parentOrParents=null,this._subscriptions=null,e&&(this._ctorUnsubscribe=!0,this._unsubscribe=e)}return e.prototype.unsubscribe=function(){var t;if(!this.closed){var r=this,n=r._parentOrParents,i=r._ctorUnsubscribe,o=r._unsubscribe,a=r._subscriptions;if(this.closed=!0,this._parentOrParents=null,this._subscriptions=null,n instanceof e)n.remove(this);else if(null!==n)for(var s=0;s<n.length;++s){n[s].remove(this)}if(Ro(o)){i&&(this._unsubscribe=void 0);try{o.call(this)}catch(e){t=e instanceof Ho?Go(e.errors):[e]}}if(Lo(a)){s=-1;for(var u=a.length;++s<u;){var c=a[s];if($o(c))try{c.unsubscribe()}catch(e){t=t||[],e instanceof Ho?t=t.concat(Go(e.errors)):t.push(e)}}}if(t)throw new Ho(t)}},e.prototype.add=function(t){var r=t;if(!t)return e.EMPTY;switch(typeof t){case"function":r=new e(t);case"object":if(r===this||r.closed||"function"!=typeof r.unsubscribe)return r;if(this.closed)return r.unsubscribe(),r;if(!(r instanceof e)){var n=r;(r=new e)._subscriptions=[n]}break;default:throw new Error("unrecognized teardown "+t+" added to Subscription.")}var i=r._parentOrParents;if(null===i)r._parentOrParents=this;else if(i instanceof e){if(i===this)return r;r._parentOrParents=[i,this]}else{if(-1!==i.indexOf(this))return r;i.push(this)}var o=this._subscriptions;return null===o?this._subscriptions=[r]:o.push(r),r},e.prototype.remove=function(e){var t=this._subscriptions;if(t){var r=t.indexOf(e);-1!==r&&t.splice(r,1)}},e.EMPTY=function(e){return e.closed=!0,e}(new e),e}();function Go(e){return e.reduce((function(e,t){return e.concat(t instanceof Ho?t.errors:t)}),[])}var Ko=function(){return"function"==typeof Symbol?Symbol("rxSubscriber"):"@@rxSubscriber_"+Math.random()}(),qo=function(e){function t(r,n,i){var o=e.call(this)||this;switch(o.syncErrorValue=null,o.syncErrorThrown=!1,o.syncErrorThrowable=!1,o.isStopped=!1,arguments.length){case 0:o.destination=Fo;break;case 1:if(!r){o.destination=Fo;break}if("object"==typeof r){r instanceof t?(o.syncErrorThrowable=r.syncErrorThrowable,o.destination=r,r.add(o)):(o.syncErrorThrowable=!0,o.destination=new Wo(o,r));break}default:o.syncErrorThrowable=!0,o.destination=new Wo(o,r,n,i)}return o}return Vo(t,e),t.prototype[Ko]=function(){return this},t.create=function(e,r,n){var i=new t(e,r,n);return i.syncErrorThrowable=!1,i},t.prototype.next=function(e){this.isStopped||this._next(e)},t.prototype.error=function(e){this.isStopped||(this.isStopped=!0,this._error(e))},t.prototype.complete=function(){this.isStopped||(this.isStopped=!0,this._complete())},t.prototype.unsubscribe=function(){this.closed||(this.isStopped=!0,e.prototype.unsubscribe.call(this))},t.prototype._next=function(e){this.destination.next(e)},t.prototype._error=function(e){this.destination.error(e),this.unsubscribe()},t.prototype._complete=function(){this.destination.complete(),this.unsubscribe()},t.prototype._unsubscribeAndRecycle=function(){var e=this._parentOrParents;return this._parentOrParents=null,this.unsubscribe(),this.closed=!1,this.isStopped=!1,this._parentOrParents=e,this},t}(Bo),Wo=function(e){function t(t,r,n,i){var o,a=e.call(this)||this;a._parentSubscriber=t;var s=a;return Ro(r)?o=r:r&&(o=r.next,n=r.error,i=r.complete,r!==Fo&&(Ro((s=Object.create(r)).unsubscribe)&&a.add(s.unsubscribe.bind(s)),s.unsubscribe=a.unsubscribe.bind(a))),a._context=s,a._next=o,a._error=n,a._complete=i,a}return Vo(t,e),t.prototype.next=function(e){if(!this.isStopped&&this._next){var t=this._parentSubscriber;Mo.useDeprecatedSynchronousErrorHandling&&t.syncErrorThrowable?this.__tryOrSetError(t,this._next,e)&&this.unsubscribe():this.__tryOrUnsub(this._next,e)}},t.prototype.error=function(e){if(!this.isStopped){var t=this._parentSubscriber,r=Mo.useDeprecatedSynchronousErrorHandling;if(this._error)r&&t.syncErrorThrowable?(this.__tryOrSetError(t,this._error,e),this.unsubscribe()):(this.__tryOrUnsub(this._error,e),this.unsubscribe());else if(t.syncErrorThrowable)r?(t.syncErrorValue=e,t.syncErrorThrown=!0):zo(e),this.unsubscribe();else{if(this.unsubscribe(),r)throw e;zo(e)}}},t.prototype.complete=function(){var e=this;if(!this.isStopped){var t=this._parentSubscriber;if(this._complete){var r=function(){return e._complete.call(e._context)};Mo.useDeprecatedSynchronousErrorHandling&&t.syncErrorThrowable?(this.__tryOrSetError(t,r),this.unsubscribe()):(this.__tryOrUnsub(r),this.unsubscribe())}else this.unsubscribe()}},t.prototype.__tryOrUnsub=function(e,t){try{e.call(this._context,t)}catch(e){if(this.unsubscribe(),Mo.useDeprecatedSynchronousErrorHandling)throw e;zo(e)}},t.prototype.__tryOrSetError=function(e,t,r){if(!Mo.useDeprecatedSynchronousErrorHandling)throw new Error("bad call");try{t.call(this._context,r)}catch(t){return Mo.useDeprecatedSynchronousErrorHandling?(e.syncErrorValue=t,e.syncErrorThrown=!0,!0):(zo(t),!0)}return!1},t.prototype._unsubscribe=function(){var e=this._parentSubscriber;this._context=null,this._parentSubscriber=null,e.unsubscribe()},t}(qo);var Yo=function(){return"function"==typeof Symbol&&Symbol.observable||"@@observable"}();function Qo(e){return e}function Xo(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return Zo(e)}function Zo(e){return 0===e.length?Qo:1===e.length?e[0]:function(t){return e.reduce((function(e,t){return t(e)}),t)}}var Jo=function(){function e(e){this._isScalar=!1,e&&(this._subscribe=e)}return e.prototype.lift=function(t){var r=new e;return r.source=this,r.operator=t,r},e.prototype.subscribe=function(e,t,r){var n=this.operator,i=function(e,t,r){if(e){if(e instanceof qo)return e;if(e[Ko])return e[Ko]()}return e||t||r?new qo(e,t,r):new qo(Fo)}(e,t,r);if(n?i.add(n.call(i,this.source)):i.add(this.source||Mo.useDeprecatedSynchronousErrorHandling&&!i.syncErrorThrowable?this._subscribe(i):this._trySubscribe(i)),Mo.useDeprecatedSynchronousErrorHandling&&i.syncErrorThrowable&&(i.syncErrorThrowable=!1,i.syncErrorThrown))throw i.syncErrorValue;return i},e.prototype._trySubscribe=function(e){try{return this._subscribe(e)}catch(t){Mo.useDeprecatedSynchronousErrorHandling&&(e.syncErrorThrown=!0,e.syncErrorValue=t),!function(e){for(;e;){var t=e,r=t.closed,n=t.destination,i=t.isStopped;if(r||i)return!1;e=n&&n instanceof qo?n:null}return!0}(e)?console.warn(t):e.error(t)}},e.prototype.forEach=function(e,t){var r=this;return new(t=ea(t))((function(t,n){var i;i=r.subscribe((function(t){try{e(t)}catch(e){n(e),i&&i.unsubscribe()}}),n,t)}))},e.prototype._subscribe=function(e){var t=this.source;return t&&t.subscribe(e)},e.prototype[Yo]=function(){return this},e.prototype.pipe=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return 0===e.length?this:Zo(e)(this)},e.prototype.toPromise=function(e){var t=this;return new(e=ea(e))((function(e,r){var n;t.subscribe((function(e){return n=e}),(function(e){return r(e)}),(function(){return e(n)}))}))},e.create=function(t){return new e(t)},e}();function ea(e){if(e||(e=Promise),!e)throw new Error("no Promise impl found");return e}var ta=function(){function e(){return Error.call(this),this.message="object unsubscribed",this.name="ObjectUnsubscribedError",this}return e.prototype=Object.create(Error.prototype),e}(),ra=function(e){function t(t,r){var n=e.call(this)||this;return n.subject=t,n.subscriber=r,n.closed=!1,n}return Vo(t,e),t.prototype.unsubscribe=function(){if(!this.closed){this.closed=!0;var e=this.subject,t=e.observers;if(this.subject=null,t&&0!==t.length&&!e.isStopped&&!e.closed){var r=t.indexOf(this.subscriber);-1!==r&&t.splice(r,1)}}},t}(Bo),na=function(e){function t(t){var r=e.call(this,t)||this;return r.destination=t,r}return Vo(t,e),t}(qo),ia=function(e){function t(){var t=e.call(this)||this;return t.observers=[],t.closed=!1,t.isStopped=!1,t.hasError=!1,t.thrownError=null,t}return Vo(t,e),t.prototype[Ko]=function(){return new na(this)},t.prototype.lift=function(e){var t=new oa(this,this);return t.operator=e,t},t.prototype.next=function(e){if(this.closed)throw new ta;if(!this.isStopped)for(var t=this.observers,r=t.length,n=t.slice(),i=0;i<r;i++)n[i].next(e)},t.prototype.error=function(e){if(this.closed)throw new ta;this.hasError=!0,this.thrownError=e,this.isStopped=!0;for(var t=this.observers,r=t.length,n=t.slice(),i=0;i<r;i++)n[i].error(e);this.observers.length=0},t.prototype.complete=function(){if(this.closed)throw new ta;this.isStopped=!0;for(var e=this.observers,t=e.length,r=e.slice(),n=0;n<t;n++)r[n].complete();this.observers.length=0},t.prototype.unsubscribe=function(){this.isStopped=!0,this.closed=!0,this.observers=null},t.prototype._trySubscribe=function(t){if(this.closed)throw new ta;return e.prototype._trySubscribe.call(this,t)},t.prototype._subscribe=function(e){if(this.closed)throw new ta;return this.hasError?(e.error(this.thrownError),Bo.EMPTY):this.isStopped?(e.complete(),Bo.EMPTY):(this.observers.push(e),new ra(this,e))},t.prototype.asObservable=function(){var e=new Jo;return e.source=this,e},t.create=function(e,t){return new oa(e,t)},t}(Jo),oa=function(e){function t(t,r){var n=e.call(this)||this;return n.destination=t,n.source=r,n}return Vo(t,e),t.prototype.next=function(e){var t=this.destination;t&&t.next&&t.next(e)},t.prototype.error=function(e){var t=this.destination;t&&t.error&&this.destination.error(e)},t.prototype.complete=function(){var e=this.destination;e&&e.complete&&this.destination.complete()},t.prototype._subscribe=function(e){return this.source?this.source.subscribe(e):Bo.EMPTY},t}(ia);function aa(){return function(e){return e.lift(new sa(e))}}var sa=function(){function e(e){this.connectable=e}return e.prototype.call=function(e,t){var r=this.connectable;r._refCount++;var n=new ua(e,r),i=t.subscribe(n);return n.closed||(n.connection=r.connect()),i},e}(),ua=function(e){function t(t,r){var n=e.call(this,t)||this;return n.connectable=r,n}return Vo(t,e),t.prototype._unsubscribe=function(){var e=this.connectable;if(e){this.connectable=null;var t=e._refCount;if(t<=0)this.connection=null;else if(e._refCount=t-1,t>1)this.connection=null;else{var r=this.connection,n=e._connection;this.connection=null,!n||r&&n!==r||n.unsubscribe()}}else this.connection=null},t}(qo),ca=function(e){function t(t,r){var n=e.call(this)||this;return n.source=t,n.subjectFactory=r,n._refCount=0,n._isComplete=!1,n}return Vo(t,e),t.prototype._subscribe=function(e){return this.getSubject().subscribe(e)},t.prototype.getSubject=function(){var e=this._subject;return e&&!e.isStopped||(this._subject=this.subjectFactory()),this._subject},t.prototype.connect=function(){var e=this._connection;return e||(this._isComplete=!1,(e=this._connection=new Bo).add(this.source.subscribe(new fa(this.getSubject(),this))),e.closed&&(this._connection=null,e=Bo.EMPTY)),e},t.prototype.refCount=function(){return aa()(this)},t}(Jo),la=function(){var e=ca.prototype;return{operator:{value:null},_refCount:{value:0,writable:!0},_subject:{value:null,writable:!0},_connection:{value:null,writable:!0},_subscribe:{value:e._subscribe},_isComplete:{value:e._isComplete,writable:!0},getSubject:{value:e.getSubject},connect:{value:e.connect},refCount:{value:e.refCount}}}(),fa=function(e){function t(t,r){var n=e.call(this,t)||this;return n.connectable=r,n}return Vo(t,e),t.prototype._error=function(t){this._unsubscribe(),e.prototype._error.call(this,t)},t.prototype._complete=function(){this.connectable._isComplete=!0,this._unsubscribe(),e.prototype._complete.call(this)},t.prototype._unsubscribe=function(){var e=this.connectable;if(e){this.connectable=null;var t=e._connection;e._refCount=0,e._subject=null,e._connection=null,t&&t.unsubscribe()}},t}(na),pa=function(e){function t(t,r){var n=e.call(this,t,r)||this;return n.scheduler=t,n.work=r,n.pending=!1,n}return Vo(t,e),t.prototype.schedule=function(e,t){if(void 0===t&&(t=0),this.closed)return this;this.state=e;var r=this.id,n=this.scheduler;return null!=r&&(this.id=this.recycleAsyncId(n,r,t)),this.pending=!0,this.delay=t,this.id=this.id||this.requestAsyncId(n,this.id,t),this},t.prototype.requestAsyncId=function(e,t,r){return void 0===r&&(r=0),setInterval(e.flush.bind(e,this),r)},t.prototype.recycleAsyncId=function(e,t,r){if(void 0===r&&(r=0),null!==r&&this.delay===r&&!1===this.pending)return t;clearInterval(t)},t.prototype.execute=function(e,t){if(this.closed)return new Error("executing a cancelled action");this.pending=!1;var r=this._execute(e,t);if(r)return r;!1===this.pending&&null!=this.id&&(this.id=this.recycleAsyncId(this.scheduler,this.id,null))},t.prototype._execute=function(e,t){var r=!1,n=void 0;try{this.work(e)}catch(e){r=!0,n=!!e&&e||new Error(e)}if(r)return this.unsubscribe(),n},t.prototype._unsubscribe=function(){var e=this.id,t=this.scheduler,r=t.actions,n=r.indexOf(this);this.work=null,this.state=null,this.pending=!1,this.scheduler=null,-1!==n&&r.splice(n,1),null!=e&&(this.id=this.recycleAsyncId(t,e,null)),this.delay=null},t}(function(e){function t(t,r){return e.call(this)||this}return Vo(t,e),t.prototype.schedule=function(e,t){return this},t}(Bo)),ha=function(){function e(t,r){void 0===r&&(r=e.now),this.SchedulerAction=t,this.now=r}return e.prototype.schedule=function(e,t,r){return void 0===t&&(t=0),new this.SchedulerAction(this,e).schedule(r,t)},e.now=function(){return Date.now()},e}(),ba=function(e){function t(r,n){void 0===n&&(n=ha.now);var i=e.call(this,r,(function(){return t.delegate&&t.delegate!==i?t.delegate.now():n()}))||this;return i.actions=[],i.active=!1,i.scheduled=void 0,i}return Vo(t,e),t.prototype.schedule=function(r,n,i){return void 0===n&&(n=0),t.delegate&&t.delegate!==this?t.delegate.schedule(r,n,i):e.prototype.schedule.call(this,r,n,i)},t.prototype.flush=function(e){var t=this.actions;if(this.active)t.push(e);else{var r;this.active=!0;do{if(r=e.execute(e.state,e.delay))break}while(e=t.shift());if(this.active=!1,r){for(;e=t.shift();)e.unsubscribe();throw r}}},t}(ha),da=new Jo((function(e){return e.complete()}));function va(e){return e&&"function"==typeof e.schedule}var ya=function(e){return function(t){for(var r=0,n=e.length;r<n&&!t.closed;r++)t.next(e[r]);t.complete()}};function ga(e,t){return new Jo((function(r){var n=new Bo,i=0;return n.add(t.schedule((function(){i!==e.length?(r.next(e[i++]),r.closed||n.add(this.schedule())):r.complete()}))),n}))}function ma(e,t){return t?ga(e,t):new Jo(ya(e))}function _a(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];var r=e[e.length-1];return va(r)?(e.pop(),ga(e,r)):ma(e)}var wa=new ba(pa);function Oa(){}function ja(e,t){return function(r){if("function"!=typeof e)throw new TypeError("argument is not a function. Are you looking for `mapTo()`?");return r.lift(new Pa(e,t))}}var Pa=function(){function e(e,t){this.project=e,this.thisArg=t}return e.prototype.call=function(e,t){return t.subscribe(new Sa(e,this.project,this.thisArg))},e}(),Sa=function(e){function t(t,r,n){var i=e.call(this,t)||this;return i.project=r,i.count=0,i.thisArg=n||i,i}return Vo(t,e),t.prototype._next=function(e){var t;try{t=this.project.call(this.thisArg,e,this.count++)}catch(e){return void this.destination.error(e)}this.destination.next(t)},t}(qo),Aa=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Vo(t,e),t.prototype.notifyNext=function(e,t,r,n,i){this.destination.next(t)},t.prototype.notifyError=function(e,t){this.destination.error(e)},t.prototype.notifyComplete=function(e){this.destination.complete()},t}(qo),xa=function(e){function t(t,r,n){var i=e.call(this)||this;return i.parent=t,i.outerValue=r,i.outerIndex=n,i.index=0,i}return Vo(t,e),t.prototype._next=function(e){this.parent.notifyNext(this.outerValue,e,this.outerIndex,this.index++,this)},t.prototype._error=function(e){this.parent.notifyError(e,this),this.unsubscribe()},t.prototype._complete=function(){this.parent.notifyComplete(this),this.unsubscribe()},t}(qo);function Ea(){return"function"==typeof Symbol&&Symbol.iterator?Symbol.iterator:"@@iterator"}var Ta=Ea(),Ia=function(e){return e&&"number"==typeof e.length&&"function"!=typeof e};function Ca(e){return!!e&&"function"!=typeof e.subscribe&&"function"==typeof e.then}var ka=function(e){if(e&&"function"==typeof e[Yo])return n=e,function(e){var t=n[Yo]();if("function"!=typeof t.subscribe)throw new TypeError("Provided object does not correctly implement Symbol.observable");return t.subscribe(e)};if(Ia(e))return ya(e);if(Ca(e))return r=e,function(e){return r.then((function(t){e.closed||(e.next(t),e.complete())}),(function(t){return e.error(t)})).then(null,zo),e};if(e&&"function"==typeof e[Ta])return t=e,function(e){for(var r=t[Ta]();;){var n=void 0;try{n=r.next()}catch(t){return e.error(t),e}if(n.done){e.complete();break}if(e.next(n.value),e.closed)break}return"function"==typeof r.return&&e.add((function(){r.return&&r.return()})),e};var t,r,n,i=$o(e)?"an invalid object":"'"+e+"'";throw new TypeError("You provided "+i+" where a stream was expected. You can provide an Observable, Promise, Array, or Iterable.")};function Da(e,t){if(null!=e){if(function(e){return e&&"function"==typeof e[Yo]}(e))return function(e,t){return new Jo((function(r){var n=new Bo;return n.add(t.schedule((function(){var i=e[Yo]();n.add(i.subscribe({next:function(e){n.add(t.schedule((function(){return r.next(e)})))},error:function(e){n.add(t.schedule((function(){return r.error(e)})))},complete:function(){n.add(t.schedule((function(){return r.complete()})))}}))}))),n}))}(e,t);if(Ca(e))return function(e,t){return new Jo((function(r){var n=new Bo;return n.add(t.schedule((function(){return e.then((function(e){n.add(t.schedule((function(){r.next(e),n.add(t.schedule((function(){return r.complete()})))})))}),(function(e){n.add(t.schedule((function(){return r.error(e)})))}))}))),n}))}(e,t);if(Ia(e))return ga(e,t);if(function(e){return e&&"function"==typeof e[Ta]}(e)||"string"==typeof e)return function(e,t){if(!e)throw new Error("Iterable cannot be null");return new Jo((function(r){var n,i=new Bo;return i.add((function(){n&&"function"==typeof n.return&&n.return()})),i.add(t.schedule((function(){n=e[Ta](),i.add(t.schedule((function(){if(!r.closed){var e,t;try{var i=n.next();e=i.value,t=i.done}catch(e){return void r.error(e)}t?r.complete():(r.next(e),this.schedule())}})))}))),i}))}(e,t)}throw new TypeError((null!==e&&typeof e||e)+" is not observable")}function Na(e,t){return t?Da(e,t):e instanceof Jo?e:new Jo(ka(e))}var Va=function(e){function t(t){var r=e.call(this)||this;return r.parent=t,r}return Vo(t,e),t.prototype._next=function(e){this.parent.notifyNext(e)},t.prototype._error=function(e){this.parent.notifyError(e),this.unsubscribe()},t.prototype._complete=function(){this.parent.notifyComplete(),this.unsubscribe()},t}(qo),Ra=function(e){function t(){return null!==e&&e.apply(this,arguments)||this}return Vo(t,e),t.prototype.notifyNext=function(e){this.destination.next(e)},t.prototype.notifyError=function(e){this.destination.error(e)},t.prototype.notifyComplete=function(){this.destination.complete()},t}(qo);function Ua(e,t){if(!t.closed)return e instanceof Jo?e.subscribe(t):ka(e)(t)}function Ma(e,t,r){return void 0===r&&(r=Number.POSITIVE_INFINITY),"function"==typeof t?function(n){return n.pipe(Ma((function(r,n){return Na(e(r,n)).pipe(ja((function(e,i){return t(r,e,n,i)})))}),r))}:("number"==typeof t&&(r=t),function(t){return t.lift(new za(e,r))})}var za=function(){function e(e,t){void 0===t&&(t=Number.POSITIVE_INFINITY),this.project=e,this.concurrent=t}return e.prototype.call=function(e,t){return t.subscribe(new Fa(e,this.project,this.concurrent))},e}(),Fa=function(e){function t(t,r,n){void 0===n&&(n=Number.POSITIVE_INFINITY);var i=e.call(this,t)||this;return i.project=r,i.concurrent=n,i.hasCompleted=!1,i.buffer=[],i.active=0,i.index=0,i}return Vo(t,e),t.prototype._next=function(e){this.active<this.concurrent?this._tryNext(e):this.buffer.push(e)},t.prototype._tryNext=function(e){var t,r=this.index++;try{t=this.project(e,r)}catch(e){return void this.destination.error(e)}this.active++,this._innerSub(t)},t.prototype._innerSub=function(e){var t=new Va(this),r=this.destination;r.add(t);var n=Ua(e,t);n!==t&&r.add(n)},t.prototype._complete=function(){this.hasCompleted=!0,0===this.active&&0===this.buffer.length&&this.destination.complete(),this.unsubscribe()},t.prototype.notifyNext=function(e){this.destination.next(e)},t.prototype.notifyComplete=function(){var e=this.buffer;this.active--,e.length>0?this._next(e.shift()):0===this.active&&this.hasCompleted&&this.destination.complete()},t}(Ra);function La(e,t,r){return r?La(e,t).pipe(ja((function(e){return Lo(e)?r.apply(void 0,e):r(e)}))):new Jo((function(r){var n,i=function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t];return r.next(1===e.length?e[0]:e)};try{n=e(i)}catch(e){return void r.error(e)}if(Ro(t))return function(){return t(i,n)}}))}function $a(e){return!Lo(e)&&e-parseFloat(e)+1>=0}function Ha(e,t){return function(r){return r.lift(new Ba(e,t))}}var Ba=function(){function e(e,t){this.predicate=e,this.thisArg=t}return e.prototype.call=function(e,t){return t.subscribe(new Ga(e,this.predicate,this.thisArg))},e}(),Ga=function(e){function t(t,r,n){var i=e.call(this,t)||this;return i.predicate=r,i.thisArg=n,i.count=0,i}return Vo(t,e),t.prototype._next=function(e){var t;try{t=this.predicate.call(this.thisArg,e,this.count++)}catch(e){return void this.destination.error(e)}t&&this.destination.next(e)},t}(qo);function Ka(e,t,r){void 0===e&&(e=0);var n=-1;return $a(t)?n=Number(t)<1?1:Number(t):va(t)&&(r=t),va(r)||(r=wa),new Jo((function(t){var i=$a(e)?e:+e-r.now();return r.schedule(qa,i,{index:0,period:n,subscriber:t})}))}function qa(e){var t=e.index,r=e.period,n=e.subscriber;if(n.next(t),!n.closed){if(-1===r)return n.complete();e.index=t+1,this.schedule(e,r)}}var Wa="object"==typeof global&&global&&global.Object===Object&&global,Ya="object"==typeof self&&self&&self.Object===Object&&self,Qa=Wa||Ya||Function("return this")(),Xa=Qa.Symbol,Za=Object.prototype,Ja=Za.hasOwnProperty,es=Za.toString,ts=Xa?Xa.toStringTag:void 0;var rs=Object.prototype.toString;var ns=Xa?Xa.toStringTag:void 0;function is(e){return null==e?void 0===e?"[object Undefined]":"[object Null]":ns&&ns in Object(e)?function(e){var t=Ja.call(e,ts),r=e[ts];try{e[ts]=void 0;var n=!0}catch(e){}var i=es.call(e);return n&&(t?e[ts]=r:delete e[ts]),i}(e):function(e){return rs.call(e)}(e)}function os(e){return null!=e&&"object"==typeof e}function as(e){return"symbol"==typeof e||os(e)&&"[object Symbol]"==is(e)}function ss(e,t){for(var r=-1,n=null==e?0:e.length,i=Array(n);++r<n;)i[r]=t(e[r],r,e);return i}var us=Array.isArray,cs=Xa?Xa.prototype:void 0,ls=cs?cs.toString:void 0;function fs(e){if("string"==typeof e)return e;if(us(e))return ss(e,fs)+"";if(as(e))return ls?ls.call(e):"";var t=e+"";return"0"==t&&1/e==-Infinity?"-0":t}function ps(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}function hs(e){return e}function bs(e){if(!ps(e))return!1;var t=is(e);return"[object Function]"==t||"[object GeneratorFunction]"==t||"[object AsyncFunction]"==t||"[object Proxy]"==t}var ds,vs=Qa["__core-js_shared__"],ys=(ds=/[^.]+$/.exec(vs&&vs.keys&&vs.keys.IE_PROTO||""))?"Symbol(src)_1."+ds:"";var gs=Function.prototype.toString;var ms=/^\[object .+?Constructor\]$/,_s=Function.prototype,ws=Object.prototype,Os=_s.toString,js=ws.hasOwnProperty,Ps=RegExp("^"+Os.call(js).replace(/[\\^$.*+?()[\]{}|]/g,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function Ss(e){return!(!ps(e)||(t=e,ys&&ys in t))&&(bs(e)?Ps:ms).test(function(e){if(null!=e){try{return gs.call(e)}catch(e){}try{return e+""}catch(e){}}return""}(e));var t}function As(e,t){var r=function(e,t){return null==e?void 0:e[t]}(e,t);return Ss(r)?r:void 0}function xs(e,t,r){switch(r.length){case 0:return e.call(t);case 1:return e.call(t,r[0]);case 2:return e.call(t,r[0],r[1]);case 3:return e.call(t,r[0],r[1],r[2])}return e.apply(t,r)}var Es=Date.now;var Ts,Is,Cs,ks=function(){try{var e=As(Object,"defineProperty");return e({},"",{}),e}catch(e){}}(),Ds=(Ts=ks?function(e,t){return ks(e,"toString",{configurable:!0,enumerable:!1,value:(r=t,function(){return r}),writable:!0});var r}:hs,Is=0,Cs=0,function(){var e=Es(),t=16-(e-Cs);if(Cs=e,t>0){if(++Is>=800)return arguments[0]}else Is=0;return Ts.apply(void 0,arguments)});function Ns(e){return e!=e}function Vs(e,t){return!!(null==e?0:e.length)&&function(e,t,r){return t==t?function(e,t,r){for(var n=r-1,i=e.length;++n<i;)if(e[n]===t)return n;return-1}(e,t,r):function(e,t,r,n){for(var i=e.length,o=r+(n?1:-1);n?o--:++o<i;)if(t(e[o],o,e))return o;return-1}(e,Ns,r)}(e,t,0)>-1}var Rs=/^(?:0|[1-9]\d*)$/;function Us(e,t){var r=typeof e;return!!(t=null==t?9007199254740991:t)&&("number"==r||"symbol"!=r&&Rs.test(e))&&e>-1&&e%1==0&&e<t}function Ms(e,t){return e===t||e!=e&&t!=t}var zs=Object.prototype.hasOwnProperty;function Fs(e,t,r){var n=e[t];zs.call(e,t)&&Ms(n,r)&&(void 0!==r||t in e)||function(e,t,r){"__proto__"==t&&ks?ks(e,t,{configurable:!0,enumerable:!0,value:r,writable:!0}):e[t]=r}(e,t,r)}var Ls=Math.max;function $s(e,t,r){return t=Ls(void 0===t?e.length-1:t,0),function(){for(var n=arguments,i=-1,o=Ls(n.length-t,0),a=Array(o);++i<o;)a[i]=n[t+i];i=-1;for(var s=Array(t+1);++i<t;)s[i]=n[i];return s[t]=r(a),xs(e,this,s)}}function Hs(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=9007199254740991}function Bs(e){return os(e)&&"[object Arguments]"==is(e)}var Gs=Object.prototype,Ks=Gs.hasOwnProperty,qs=Gs.propertyIsEnumerable,Ws=Bs(function(){return arguments}())?Bs:function(e){return os(e)&&Ks.call(e,"callee")&&!qs.call(e,"callee")};function Ys(e){return function(t){return e(t)}}var Qs="object"==typeof exports&&exports&&!exports.nodeType&&exports,Xs=Qs&&"object"==typeof module&&module&&!module.nodeType&&module,Zs=Xs&&Xs.exports===Qs&&Wa.process,Js=function(){try{var e=Xs&&Xs.require&&Xs.require("util").types;return e||Zs&&Zs.binding&&Zs.binding("util")}catch(e){}}(),eu=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,tu=/^\w*$/;var ru=As(Object,"create");var nu=Object.prototype.hasOwnProperty;var iu=Object.prototype.hasOwnProperty;function ou(e){var t=-1,r=null==e?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1])}}function au(e,t){for(var r=e.length;r--;)if(Ms(e[r][0],t))return r;return-1}ou.prototype.clear=function(){this.__data__=ru?ru(null):{},this.size=0},ou.prototype.delete=function(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t},ou.prototype.get=function(e){var t=this.__data__;if(ru){var r=t[e];return"__lodash_hash_undefined__"===r?void 0:r}return nu.call(t,e)?t[e]:void 0},ou.prototype.has=function(e){var t=this.__data__;return ru?void 0!==t[e]:iu.call(t,e)},ou.prototype.set=function(e,t){var r=this.__data__;return this.size+=this.has(e)?0:1,r[e]=ru&&void 0===t?"__lodash_hash_undefined__":t,this};var su=Array.prototype.splice;function uu(e){var t=-1,r=null==e?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1])}}uu.prototype.clear=function(){this.__data__=[],this.size=0},uu.prototype.delete=function(e){var t=this.__data__,r=au(t,e);return!(r<0)&&(r==t.length-1?t.pop():su.call(t,r,1),--this.size,!0)},uu.prototype.get=function(e){var t=this.__data__,r=au(t,e);return r<0?void 0:t[r][1]},uu.prototype.has=function(e){return au(this.__data__,e)>-1},uu.prototype.set=function(e,t){var r=this.__data__,n=au(r,e);return n<0?(++this.size,r.push([e,t])):r[n][1]=t,this};var cu=As(Qa,"Map");function lu(e,t){var r=e.__data__;return function(e){var t=typeof e;return"string"==t||"number"==t||"symbol"==t||"boolean"==t?"__proto__"!==e:null===e}(t)?r["string"==typeof t?"string":"hash"]:r.map}function fu(e){var t=-1,r=null==e?0:e.length;for(this.clear();++t<r;){var n=e[t];this.set(n[0],n[1])}}fu.prototype.clear=function(){this.size=0,this.__data__={hash:new ou,map:new(cu||uu),string:new ou}},fu.prototype.delete=function(e){var t=lu(this,e).delete(e);return this.size-=t?1:0,t},fu.prototype.get=function(e){return lu(this,e).get(e)},fu.prototype.has=function(e){return lu(this,e).has(e)},fu.prototype.set=function(e,t){var r=lu(this,e),n=r.size;return r.set(e,t),this.size+=r.size==n?0:1,this};function pu(e,t){if("function"!=typeof e||null!=t&&"function"!=typeof t)throw new TypeError("Expected a function");var r=function(){var n=arguments,i=t?t.apply(this,n):n[0],o=r.cache;if(o.has(i))return o.get(i);var a=e.apply(this,n);return r.cache=o.set(i,a)||o,a};return r.cache=new(pu.Cache||fu),r}pu.Cache=fu;var hu=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,bu=/\\(\\)?/g,du=function(e){var t=pu(e,(function(e){return 500===r.size&&r.clear(),e})),r=t.cache;return t}((function(e){var t=[];return 46===e.charCodeAt(0)&&t.push(""),e.replace(hu,(function(e,r,n,i){t.push(n?i.replace(bu,"$1"):r||e)})),t}));function vu(e,t){return us(e)?e:function(e,t){if(us(e))return!1;var r=typeof e;return!("number"!=r&&"symbol"!=r&&"boolean"!=r&&null!=e&&!as(e))||tu.test(e)||!eu.test(e)||null!=t&&e in Object(t)}(e,t)?[e]:du(function(e){return null==e?"":fs(e)}(e))}function yu(e){if("string"==typeof e||as(e))return e;var t=e+"";return"0"==t&&1/e==-Infinity?"-0":t}function gu(e,t){for(var r=0,n=(t=vu(t,e)).length;null!=e&&r<n;)e=e[yu(t[r++])];return r&&r==n?e:void 0}function mu(e,t){for(var r=-1,n=t.length,i=e.length;++r<n;)e[i+r]=t[r];return e}var _u=Xa?Xa.isConcatSpreadable:void 0;function wu(e){return us(e)||Ws(e)||!!(_u&&e&&e[_u])}function Ou(e,t,r,n,i){var o=-1,a=e.length;for(r||(r=wu),i||(i=[]);++o<a;){var s=e[o];t>0&&r(s)?t>1?Ou(s,t-1,r,n,i):mu(i,s):n||(i[i.length]=s)}return i}function ju(e){return(null==e?0:e.length)?Ou(e,1):[]}function Pu(e){var t=-1,r=null==e?0:e.length;for(this.__data__=new fu;++t<r;)this.add(e[t])}function Su(e,t){return e.has(t)}function Au(e,t){return null!=e&&t in Object(e)}function xu(e,t){return null!=e&&function(e,t,r){for(var n=-1,i=(t=vu(t,e)).length,o=!1;++n<i;){var a=yu(t[n]);if(!(o=null!=e&&r(e,a)))break;e=e[a]}return o||++n!=i?o:!!(i=null==e?0:e.length)&&Hs(i)&&Us(a,i)&&(us(e)||Ws(e))}(e,t,Au)}function Eu(e){return os(e)&&function(e){return null!=e&&Hs(e.length)&&!bs(e)}(e)}function Tu(e,t,r){for(var n=-1,i=null==e?0:e.length;++n<i;)if(r(t,e[n]))return!0;return!1}Pu.prototype.add=Pu.prototype.push=function(e){return this.__data__.set(e,"__lodash_hash_undefined__"),this},Pu.prototype.has=function(e){return this.__data__.has(e)};var Iu=function(e,t){return Ds($s(e,t,hs),e+"")}((function(e,t){return Eu(e)?function(e,t,r,n){var i=-1,o=Vs,a=!0,s=e.length,u=[],c=t.length;if(!s)return u;r&&(t=ss(t,Ys(r))),n?(o=Tu,a=!1):t.length>=200&&(o=Su,a=!1,t=new Pu(t));e:for(;++i<s;){var l=e[i],f=null==r?l:r(l);if(l=n||0!==l?l:0,a&&f==f){for(var p=c;p--;)if(t[p]===f)continue e;u.push(l)}else o(t,f,n)||u.push(l)}return u}(e,Ou(t,1,Eu,!0)):[]}));var Cu=Js&&Js.isRegExp,ku=Cu?Ys(Cu):function(e){return os(e)&&"[object RegExp]"==is(e)};function Du(e,t,r,n){if(!ps(e))return e;for(var i=-1,o=(t=vu(t,e)).length,a=o-1,s=e;null!=s&&++i<o;){var u=yu(t[i]),c=r;if("__proto__"===u||"constructor"===u||"prototype"===u)return e;if(i!=a){var l=s[u];void 0===(c=n?n(l,u,s):void 0)&&(c=ps(l)?l:Us(t[i+1])?[]:{})}Fs(s,u,c),s=s[u]}return e}function Nu(e,t){return function(e,t,r){for(var n=-1,i=t.length,o={};++n<i;){var a=t[n],s=gu(e,a);r(s,a)&&Du(o,vu(a,e),s)}return o}(e,t,(function(t,r){return xu(e,r)}))}var Vu=function(e){return Ds($s(e,void 0,ju),e+"")}((function(e,t){return null==e?{}:Nu(e,t)}));var Ru=function(){function e(e,t,r,n){this.bufferTimeSpan=e,this.bufferCreationInterval=t,this.maxBufferSize=r,this.scheduler=n}return e.prototype.call=function(e,t){return t.subscribe(new Mu(e,this.bufferTimeSpan,this.bufferCreationInterval,this.maxBufferSize,this.scheduler))},e}(),Uu=function(){return function(){this.buffer=[]}}(),Mu=function(e){function t(t,r,n,i,o){var a=e.call(this,t)||this;a.bufferTimeSpan=r,a.bufferCreationInterval=n,a.maxBufferSize=i,a.scheduler=o,a.contexts=[];var s=a.openContext();if(a.timespanOnly=null==n||n<0,a.timespanOnly){var u={subscriber:a,context:s,bufferTimeSpan:r};a.add(s.closeAction=o.schedule(zu,r,u))}else{var c={subscriber:a,context:s},l={bufferTimeSpan:r,bufferCreationInterval:n,subscriber:a,scheduler:o};a.add(s.closeAction=o.schedule(Lu,r,c)),a.add(o.schedule(Fu,n,l))}return a}return Vo(t,e),t.prototype._next=function(e){for(var t,r=this.contexts,n=r.length,i=0;i<n;i++){var o=r[i],a=o.buffer;a.push(e),a.length==this.maxBufferSize&&(t=o)}t&&this.onBufferFull(t)},t.prototype._error=function(t){this.contexts.length=0,e.prototype._error.call(this,t)},t.prototype._complete=function(){for(var t=this.contexts,r=this.destination;t.length>0;){var n=t.shift();r.next(n.buffer)}e.prototype._complete.call(this)},t.prototype._unsubscribe=function(){this.contexts=null},t.prototype.onBufferFull=function(e){this.closeContext(e);var t=e.closeAction;if(t.unsubscribe(),this.remove(t),!this.closed&&this.timespanOnly){e=this.openContext();var r=this.bufferTimeSpan,n={subscriber:this,context:e,bufferTimeSpan:r};this.add(e.closeAction=this.scheduler.schedule(zu,r,n))}},t.prototype.openContext=function(){var e=new Uu;return this.contexts.push(e),e},t.prototype.closeContext=function(e){this.destination.next(e.buffer);var t=this.contexts;(t?t.indexOf(e):-1)>=0&&t.splice(t.indexOf(e),1)},t}(qo);function zu(e){var t=e.subscriber,r=e.context;r&&t.closeContext(r),t.closed||(e.context=t.openContext(),e.context.closeAction=this.schedule(e,e.bufferTimeSpan))}function Fu(e){var t=e.bufferCreationInterval,r=e.bufferTimeSpan,n=e.subscriber,i=e.scheduler,o=n.openContext();n.closed||(n.add(o.closeAction=i.schedule(Lu,r,{subscriber:n,context:o})),this.schedule(e,t))}function Lu(e){var t=e.subscriber,r=e.context;t.closeContext(r)}function $u(e){return function(t){var r=new Hu(e),n=t.lift(r);return r.caught=n}}var Hu=function(){function e(e){this.selector=e}return e.prototype.call=function(e,t){return t.subscribe(new Bu(e,this.selector,this.caught))},e}(),Bu=function(e){function t(t,r,n){var i=e.call(this,t)||this;return i.selector=r,i.caught=n,i}return Vo(t,e),t.prototype.error=function(t){if(!this.isStopped){var r=void 0;try{r=this.selector(t,this.caught)}catch(t){return void e.prototype.error.call(this,t)}this._unsubscribeAndRecycle();var n=new Va(this);this.add(n);var i=Ua(r,n);i!==n&&this.add(i)}},t}(Ra);function Gu(e,t){return Ma(e,t,1)}function Ku(e,t){return t?function(r){return new Yu(r,t).lift(new qu(e))}:function(t){return t.lift(new qu(e))}}var qu=function(){function e(e){this.delayDurationSelector=e}return e.prototype.call=function(e,t){return t.subscribe(new Wu(e,this.delayDurationSelector))},e}(),Wu=function(e){function t(t,r){var n=e.call(this,t)||this;return n.delayDurationSelector=r,n.completed=!1,n.delayNotifierSubscriptions=[],n.index=0,n}return Vo(t,e),t.prototype.notifyNext=function(e,t,r,n,i){this.destination.next(e),this.removeSubscription(i),this.tryComplete()},t.prototype.notifyError=function(e,t){this._error(e)},t.prototype.notifyComplete=function(e){var t=this.removeSubscription(e);t&&this.destination.next(t),this.tryComplete()},t.prototype._next=function(e){var t=this.index++;try{var r=this.delayDurationSelector(e,t);r&&this.tryDelay(r,e)}catch(e){this.destination.error(e)}},t.prototype._complete=function(){this.completed=!0,this.tryComplete(),this.unsubscribe()},t.prototype.removeSubscription=function(e){e.unsubscribe();var t=this.delayNotifierSubscriptions.indexOf(e);return-1!==t&&this.delayNotifierSubscriptions.splice(t,1),e.outerValue},t.prototype.tryDelay=function(e,t){var r=function(e,t,r,n,i){if(void 0===i&&(i=new xa(e,r,n)),!i.closed)return t instanceof Jo?t.subscribe(i):ka(t)(i)}(this,e,t);r&&!r.closed&&(this.destination.add(r),this.delayNotifierSubscriptions.push(r))},t.prototype.tryComplete=function(){this.completed&&0===this.delayNotifierSubscriptions.length&&this.destination.complete()},t}(Aa),Yu=function(e){function t(t,r){var n=e.call(this)||this;return n.source=t,n.subscriptionDelay=r,n}return Vo(t,e),t.prototype._subscribe=function(e){this.subscriptionDelay.subscribe(new Qu(e,this.source))},t}(Jo),Qu=function(e){function t(t,r){var n=e.call(this)||this;return n.parent=t,n.source=r,n.sourceSubscribed=!1,n}return Vo(t,e),t.prototype._next=function(e){this.subscribeToSource()},t.prototype._error=function(e){this.unsubscribe(),this.parent.error(e)},t.prototype._complete=function(){this.unsubscribe(),this.subscribeToSource()},t.prototype.subscribeToSource=function(){this.sourceSubscribed||(this.sourceSubscribed=!0,this.unsubscribe(),this.source.subscribe(this.parent))},t}(qo);var Xu=function(){function e(e,t,r){void 0===r&&(r=!1),this.accumulator=e,this.seed=t,this.hasSeed=r}return e.prototype.call=function(e,t){return t.subscribe(new Zu(e,this.accumulator,this.seed,this.hasSeed))},e}(),Zu=function(e){function t(t,r,n,i){var o=e.call(this,t)||this;return o.accumulator=r,o._seed=n,o.hasSeed=i,o.index=0,o}return Vo(t,e),Object.defineProperty(t.prototype,"seed",{get:function(){return this._seed},set:function(e){this.hasSeed=!0,this._seed=e},enumerable:!0,configurable:!0}),t.prototype._next=function(e){if(this.hasSeed)return this._tryNext(e);this.seed=e,this.destination.next(e)},t.prototype._tryNext=function(e){var t,r=this.index++;try{t=this.accumulator(this.seed,e,r)}catch(e){this.destination.error(e)}this.seed=t,this.destination.next(t)},t}(qo);var Ju=function(){function e(e,t){this.subjectFactory=e,this.selector=t}return e.prototype.call=function(e,t){var r=this.selector,n=this.subjectFactory(),i=r(n).subscribe(e);return i.add(t.subscribe(n)),i},e}();var ec=function(){function e(e,t){this.notifier=e,this.source=t}return e.prototype.call=function(e,t){return t.subscribe(new tc(e,this.notifier,this.source))},e}(),tc=function(e){function t(t,r,n){var i=e.call(this,t)||this;return i.notifier=r,i.source=n,i}return Vo(t,e),t.prototype.error=function(t){if(!this.isStopped){var r=this.errors,n=this.retries,i=this.retriesSubscription;if(n)this.errors=void 0,this.retriesSubscription=void 0;else{r=new ia;try{n=(0,this.notifier)(r)}catch(t){return e.prototype.error.call(this,t)}i=Ua(n,new Va(this))}this._unsubscribeAndRecycle(),this.errors=r,this.retries=n,this.retriesSubscription=i,r.next(t)}},t.prototype._unsubscribe=function(){var e=this.errors,t=this.retriesSubscription;e&&(e.unsubscribe(),this.errors=void 0),t&&(t.unsubscribe(),this.retriesSubscription=void 0),this.retries=void 0},t.prototype.notifyNext=function(){var e=this._unsubscribe;this._unsubscribe=null,this._unsubscribeAndRecycle(),this._unsubscribe=e,this.source.subscribe(this)},t}(Ra);function rc(){return new ia}function nc(){return function(e){return aa()((t=rc,function(e){var n;if(n="function"==typeof t?t:function(){return t},"function"==typeof r)return e.lift(new Ju(n,r));var i=Object.create(e,la);return i.source=e,i.subjectFactory=n,i})(e));var t,r}}function ic(e,t){return"function"==typeof t?function(r){return r.pipe(ic((function(r,n){return Na(e(r,n)).pipe(ja((function(e,i){return t(r,e,n,i)})))})))}:function(t){return t.lift(new oc(e))}}var oc=function(){function e(e){this.project=e}return e.prototype.call=function(e,t){return t.subscribe(new ac(e,this.project))},e}(),ac=function(e){function t(t,r){var n=e.call(this,t)||this;return n.project=r,n.index=0,n}return Vo(t,e),t.prototype._next=function(e){var t,r=this.index++;try{t=this.project(e,r)}catch(e){return void this.destination.error(e)}this._innerSub(t)},t.prototype._innerSub=function(e){var t=this.innerSubscription;t&&t.unsubscribe();var r=new Va(this),n=this.destination;n.add(r),this.innerSubscription=Ua(e,r),this.innerSubscription!==r&&n.add(this.innerSubscription)},t.prototype._complete=function(){var t=this.innerSubscription;t&&!t.closed||e.prototype._complete.call(this),this.unsubscribe()},t.prototype._unsubscribe=function(){this.innerSubscription=void 0},t.prototype.notifyComplete=function(){this.innerSubscription=void 0,this.isStopped&&e.prototype._complete.call(this)},t.prototype.notifyNext=function(e){this.destination.next(e)},t}(Ra);function sc(e,t,r){return function(n){return n.lift(new uc(e,t,r))}}var uc=function(){function e(e,t,r){this.nextOrObserver=e,this.error=t,this.complete=r}return e.prototype.call=function(e,t){return t.subscribe(new cc(e,this.nextOrObserver,this.error,this.complete))},e}(),cc=function(e){function t(t,r,n,i){var o=e.call(this,t)||this;return o._tapNext=Oa,o._tapError=Oa,o._tapComplete=Oa,o._tapError=n||Oa,o._tapComplete=i||Oa,Ro(r)?(o._context=o,o._tapNext=r):r&&(o._context=r,o._tapNext=r.next||Oa,o._tapError=r.error||Oa,o._tapComplete=r.complete||Oa),o}return Vo(t,e),t.prototype._next=function(e){try{this._tapNext.call(this._context,e)}catch(e){return void this.destination.error(e)}this.destination.next(e)},t.prototype._error=function(e){try{this._tapError.call(this._context,e)}catch(e){return void this.destination.error(e)}this.destination.error(e)},t.prototype._complete=function(){try{this._tapComplete.call(this._context)}catch(e){return void this.destination.error(e)}return this.destination.complete()},t}(qo);
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
class lc{all=new Map;constructor(e={},t=null){this.bindThis=t||globalThis,this.on(e),this.createSource$=pu((e=>La((t=>this.on(e,t)),(t=>this.off(e,t))).pipe(nc())))}#on(e,t){const r=this.all.get(e);r?r.unshift(t):this.all.set(e,[t])}on(e,t){"string"==typeof e?this.#on(e,t):e instanceof Object&&Object.entries(e).forEach((([e,t])=>{t instanceof Array?t.forEach((t=>this.#on(e,t))):t instanceof Function&&this.#on(e,t)}))}off(e,t){if("*"===e)return this.all.clear();{const r=this.all.get(e);return!!r&&(t?r.splice(r.indexOf(t)>>>0,1):this.all.set(e,[]))}}emit(e,...t){const r=this.all.get(e);return r?r.map((e=>e.apply(this.bindThis,t))):[]}}class fc{_belongTo=null;constructor(e,t){this.$EventHub=new lc({},this),this.$store=Do({spiderUUID:t,originData:e})}$commit(e,...t){const r=this.$store[e](...t);return this.$EventHub.emit(e,...t),r}$on(...e){return this.$EventHub.on(...e)}$off(...e){return this.$EventHub.off(...e)}$isSameTask(e){return e.$store.spiderUUID===this.$store.spiderUUID&&e.$store.uuid===this.$store.uuid}$destroy(){this._belongTo=null,this.$commit("destroy"),this.$EventHub.off("*")}get[Symbol.toStringTag](){return"Task"}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */class pc extends fc{constructor(e,t="00000"){super({},e?.[0]?.spiderUUID||t),this.member=new Set(e),this.#linkTask()}#linkTask(){this.member.forEach((e=>{e._belongTo=this,e.$on("destroy",(()=>this.$removeLink(e)))}))}consume=!1;$commit(e,...t){const r=[];return this.member.forEach((n=>r.push(n.$commit(e,...t)))),this.$EventHub.emit(e,...t),this.consume?r[0]:("start"===e&&(this.consume=!0),r)}$destroy(){this.$EventHub.emit("destroy"),this.$store.destroy();const e=this.member;return this.member=null,e}$removeLink(e){this.member.delete(e)}get[Symbol.toStringTag](){return"TaskGroup"}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */const hc=new lc,bc=hc.createSource$("TaskUpdate"),dc=hc.createSource$("ControlUpdate");
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
var vc={stateChange(e){this.status=e,hc.emit("ControlUpdate",{name:"stateChange",value:e})},"Flow:stop"(){this._stop=!0,hc.emit("ControlUpdate",{name:"flowStop"})},"Flow:start"(){console.log("jspider 开始运行"),this.status="pending",this._stop=!1,hc.emit("ControlUpdate",{name:"flowStart"})},"Task:success"(e){e.$commit("complete"),e instanceof pc&&e.$destroy(),hc.emit("ControlUpdate",{name:"TaskSuccess"})},"Task:error"(e){console.log(e)}};
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */function yc(){const e=this.$store.$backup();hc.emit("TaskUpdate",e)}class gc{#Tasks=new Map;viewModel=[];createTask(e,t){const r=new fc(e,t);this.#Tasks.set(r.uuid,r);const n=this;return r.$on({start:yc,success:yc,complete:yc,error:yc,destroy(){n.#Tasks.delete(this.uuid)}}),this.viewModel.push(r.$store),r}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */class mc{QueuePromise=Promise.resolve();enQueue(...e){return this.QueuePromise=e.reduce(((e,t)=>e.then(t)),this.QueuePromise),this}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
function _c(e){return To(e,To.URL)}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */class wc{constructor(e){this.Plugins=e}UUID=null;operator=null;pluginUUIDMap=new Map;#PluginQueue=new mc;preparePipeline(){let e="";const t=this.Plugins.map(((t,r)=>(e+=(t?.main||t.operator).toString(),t.init instanceof Function&&this.#PluginQueue.enQueue(t.init),t.initUUID(r),this.pluginUUIDMap.set(t.uuid,t?.name||t.uuid),t.operator(this))));return this.operator=Xo(...t),this.UUID=_c(e),this.#PluginQueue.QueuePromise}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
var Oc=new class{status="free";#runningQueue=new mc;_stop=!1;spiderSource$=null;_pipeline=null;TaskManager=new gc;constructor(){this.$EventHub=new lc(vc,this)}_createLogicLine(){var e,t;this.spiderSource$=this.$EventHub.createSource$("Flow:input").pipe((e=this.$EventHub.createSource$("Flow:stop"),t=this.$EventHub.createSource$("Flow:start"),r=>new Jo((n=>{const i=new Set;let o=!1;const a=r.subscribe((e=>{o?i.add(e):n.next(e)}),Oa,(()=>{i.forEach((e=>n.next(e))),i.clear(),n.complete()})),s=e.subscribe((()=>{o=t.subscribe((()=>{i.forEach((e=>n.next(e))),i.clear(),o.unsubscribe(),o=!1}))}));return()=>{i.clear(),a.unsubscribe(),s.unsubscribe(),o&&o.unsubscribe()}}))),this._pipeline.operator),this.spiderSource$.subscribe((e=>this.$EventHub.emit("Task:success",e)),(e=>this.$EventHub.emit("Task:error",e)),(()=>this.$EventHub.emit("Flow:complete"))),this.$EventHub.emit("Flow:stop")}pipeline(...e){const t=new wc(e);if("free"!==this.status)throw new Error("在运行阶段是不能够进行操作的哦");this.$EventHub.emit("stateChange","preparing"),this._pipeline=t,this.#runningQueue.enQueue((()=>this._pipeline.preparePipeline()),(()=>{this.$EventHub.emit("Spider:clearFlow"),this._createLogicLine(),this.$EventHub.emit("stateChange","free")}))}createFlow(e,{fromBackup:t=!1}={}){return this.#runningQueue.enQueue((()=>{e.forEach((e=>{if(!this._pipeline)throw new Error("没有创建pipeline");const r=this.TaskManager.createTask(e,this._pipeline.UUID);t&&r.$store.$import(e),this.$EventHub.emit("Flow:input",r)}))}))}startFlow(){this.#runningQueue.enQueue((()=>{this.$EventHub.emit("Flow:start")}))}stopFlow(){this.$EventHub.emit("Flow:stop")}};class jc extends class{constructor({tasks:e=!1,controlPanel:t=!1}={}){this.config={tasks:e,controlPanel:t},e&&this.#subscriber.push(bc.subscribe((e=>this._update(e)))),t&&this.#subscriber.push(dc.subscribe((e=>this._change(e))))}config={};#subscriber=[];$destroy(){this.#subscriber.forEach((e=>{e.unsubscribe()}))}}{constructor(e){super(Object.assign(e,{tasks:!0,controlPanel:!0}))}tasks=[];#uuidArray=[];_update(e){const t=this.#uuidArray.indexOf(e.uuid);-1===t?(this.#uuidArray.push(e.uuid),this.tasks.push(e)):this.tasks.splice(t,1,e)}_change({name:e,value:t=""}){}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
class Pc extends Error{constructor(e){super(e),this.name="PluginError"}}class Sc extends Error{constructor(e){super(e),this.name="FlowError"}}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */class Ac{constructor({forceRetry:e=!0,saveResult:t=!1,name:r=null,main:n,init:i=null,error:o=null,complete:a=null,options:s={},operator:u}){u&&(this.operator=u),Object.assign(this,{name:r,main:n,init:i,error:o,complete:a,options:s,saveResult:t,forceRetry:e})}initUUID(e){this.uuid=_c((this?.main||this.operator).toString()+e)}TaskStarter(e){return _a(e).pipe(ic((e=>_a(e).pipe(ja((e=>[e.$commit("start",this.uuid),e._originData])),ic((([e,t])=>{const r=this.main(e,t);return r instanceof Promise||r instanceof Jo?Na(r):_a(r)})),ja((t=>(e.$commit("success",t,this.uuid,this.saveResult),e)))))),$u(((...t)=>{if(this.error instanceof Function){const r=this.error(e,...t);if(r)throw new Pc(r);return da}throw new Pc(t[0])})),sc((e=>this.complete&&this.complete(e))))}operator(e){return Xo(ic((e=>this.TaskStarter(e))))}}function xc(e,t={}){if(e instanceof Function)return new Ac(Object.assign(t,{main:e}));if(e instanceof Object)return new Ac(e);throw new Pc("Plugin 必须是一个函数或者是 Plugin 描述对象")}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */const Ec=(e,t)=>{return Xo((r=r=>r.pipe(function(e,t){var r=!1;return arguments.length>=2&&(r=!0),function(n){return n.lift(new Xu(e,t,r))}}(((t,r)=>{if(console.log("尝试次数",t,r),t>=e)throw new Sc("超出尝试次数",r);return t+1}),0),Ku(((...e)=>{switch(typeof t){case"string":case"number":return Ka(parseInt(t,10));case"function":return Ka(t(...e));default:throw new Sc("您输入的 delay 错误")}}))),function(e){return e.lift(new ec(r,e))}));
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */var r};function Tc(e,{retry:t=3,buffer:r=3,delay:n=1e3,retryDelay:i=300,handleError:o=function(e,t){throw new Error(e,t)}}={}){return Xo(function(e){var t=arguments.length,r=wa;va(arguments[arguments.length-1])&&(r=arguments[arguments.length-1],t--);var n=null;t>=2&&(n=arguments[1]);var i=Number.POSITIVE_INFINITY;return t>=3&&(i=arguments[2]),function(t){return t.lift(new Ru(e,n,i,r))}}(1e3,void 0,r),Ha((e=>e.length)),Ku(((e,t)=>Ka(t*n))),Ma((e=>Na(e))),Gu((r=>_a(r).pipe(Ma((t=>e(t))),Ec(t,i),$u(((...e)=>(o instanceof Function?o(...e):o)||da))))))}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */function Ic({url:e,options:t={}}){const{returnType:r="json"}=this.options;return console.log("- 爬取 ",e),fetch(e,Object.assign({cache:"force-cache"},t)).then((e=>{if(!e.ok)throw Error(e.statusText);return function(e,t){const r=e.headers.get("content-type")||"";return t&&"auto"!==t?t?e[t]():e.json():/text|html|rtf|xml/.test(r)?e.text():/json/.test(r)?e.json():/arrayBuffer/.test(r)?e.arrayBuffer():e.buffer?e.buffer():e.blob()}(e,r)})).then((t=>(console.log(`${e} 爬取成功`),t))).catch((e=>{throw e}))}function Cc(e){throw e}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
function kc(e,t){return e instanceof File?e:e instanceof Blob?(e.name=t,e):new File([JSON.stringify(e)],t)}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */const Dc={main:Promise.resolve(!0),add(e){this.main.then((()=>Nc(e)))}},Nc=function(e){const t=document.createElement("a");t.href=URL.createObjectURL(e),t.download=e.name,t.click(),URL.revokeObjectURL(t.href),t.remove(),console.log("%c 下载完成","color:green")},Vc=(e,{DownloadFileName:t}={},r)=>{const n=kc(e,t||("string"==typeof url?r.url.replace(/[^\/]*?\//g,""):""));return Dc.add(n),null},Rc={script:e=>new Promise(((t,r)=>{const n=document.createElement("script");n.src=e,n.onload=()=>{console.log(`${e} 加载完成`),n.remove(),t()},n.onerror=e=>r(e),document.body.append(n)})),css:e=>new Promise(((t,r)=>{const n=document.createElement("style");n.rel="stylesheet",n.src=e,n.onload=()=>{console.log(`${e} 加载完成`),t()},n.onerror=e=>r(e),document.body.append(n)}))};
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
var Uc={zangodb:"https://cdn.jsdelivr.net/gh/erikolson186/zangodb/dist/zangodb.min.js",dexie:"https://cdn.jsdelivr.net/npm/dexie@3.0.3/dist/dexie.min.js",mockjs:"https://cdn.jsdelivr.net/npm/mockjs-esm/dist/mock.min.js",xlsx:"https://cdn.jsdelivr.net/npm/xlsx@0.17.0/dist/xlsx.full.min.js",lodash:"https://cdn.jsdelivr.net/npm/lodash@4.17.21/lodash.min.js",gsap:["https://cdn.jsdelivr.net/npm/gsap@3.6.1/dist/gsap.min.js","https://cdn.jsdelivr.net/npm/gsap@3.6.1/dist/ScrollTrigger.min.js"],animejs:"https://cdn.jsdelivr.net/npm/animejs@3.2.1/lib/anime.min.js",rxjs:"https://cdn.jsdelivr.net/npm/rxjs@7.1.0/dist/bundles/rxjs.umd.min.js",jszip:"https://cdn.jsdelivr.net/npm/jszip@3.6.0/dist/jszip.min.js","ajax-hook":"https://unpkg.com/ajax-hook@2.0.3/dist/ajaxhook.min.js",axios:"https://cdn.jsdelivr.net/npm/axios@0.21.1/dist/axios.min.js",react:["https://unpkg.com/react@16/umd/react.production.min.js","https://unpkg.com/react-dom@16/umd/react-dom.production.min.js"],vue:["https://cdn.jsdelivr.net/npm/vue@2.6.14/dist/vue.min.js","https://cdn.jsdelivr.net/npm/vuex@3.6.2/dist/vuex.min.js"]};
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */const Mc=Object.entries({npm:/^npm?/i,gh:/gh?|github/i,wp:/wordpress|wp/i});function zc(e,{version:t="",store:r="npm",path:n=""}={}){return`https://cdn.jsdelivr.net/${Mc.reduce(((e,[t,n])=>n.test(r)?t:e),"npm")}/${e}${t?`@${t}`:""}${n?`/${n}`:""}`}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */function Fc(e){return Object.prototype.toString.call(e).match(/(?<=\[object\s+)\S+?(?=\])/)[0]}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */const Lc=/(https?|ftp|file):\/\/[-A-Za-z0-9+&@#/%?=~_|!:,.;]+[-A-Za-z0-9+&@#/%=~_|]/;const $c={Object({url:e="",name:t,way:r="npm",path:n="",version:i="",type:o="script"}){if(!e){const o=function({name:e,way:t,path:r,version:n}){return Uc.hasOwnProperty(e)?Uc[e]:zc(e,{version:n,store:t,path:r})}({name:t,way:r,path:n,version:i});if("string"!=typeof o)return Hc(o);e=o}return Rc[o](e)},String(e){return this.Object({[Lc.test(e)?"url":"name"]:e})},Array:e=>Promise.all(e.map((e=>Hc(e))))};async function Hc(e){return $c[Fc(e)](e)}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */let Bc;function Gc(){return Hc("xlsx").then((()=>{Bc=window.XLSX}))}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */function Kc(e){const t=Bc.utils.book_new();return Object.entries(e).forEach((([e,r])=>{const n=function(e){return e.forEach((e=>{Object.entries(e).forEach((([t,r])=>{r instanceof Object&&(e[t]=JSON.stringify(r))}))})),Bc.utils.json_to_sheet(e)}(r);Bc.utils.book_append_sheet(t,n,e)})),t}function qc(e,t,r){const{bookType:n="xlsx",bookSST:i=!0,type:o="array"}=r||{};return function(e,t,r){const n=Bc.write(e,r);return new File([n],`${t}.${r.bookType}`)}(Kc(e),t,{bookType:n,bookSST:i,type:o})}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
let Wc;function Yc(){return Hc("jszip").then((()=>{Wc=window.JSZip}))}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */let Qc=0;
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
function Xc(e=3,t=1e3){return r=>new Jo((n=>{const i=new Set;let o=0;const a=()=>{i.size&&(n.next([...i]),i.clear())},s=r.subscribe((r=>{switch(console.log(o),o&&(clearTimeout(o),o=0),i.size){case 0:return i.add(r);case e-1:return i.add(r),a();default:return o=setTimeout(a,t),i.add(r)}}),Oa,(()=>{a(),n.complete()}));return()=>{i.clear(),s.unsubscribe()}}))}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
var Zc=Object.freeze({__proto__:null,Request:function(e={}){return xc({init(){},name:"Request",main:Ic,options:e,operator(){const{delay:e=200,buffer:t=1,retry:r=3,handleError:n=null}=this.options;return i=>i.pipe(Tc((e=>this.TaskStarter(e)),{delay:e,buffer:t,retry:r,handleError:n||Cc}))}})},Download:function(e={}){return xc({name:"Download",main:Vc,options:e})},ExcelHelper:function(e,t={}){return xc({name:"ExcelHelper",init:Gc,options:t,main(t){const{fileName:r="result",XLSXOptions:n={}}=this.options;return e instanceof Function&&(t=e(t)),qc(t,`${r}-${(new Date).getTime()}`,n)}})},ZipFile:function(e={}){return e.zipFileName||(e.zipFileName=(new Date).getTime()),xc({init:Yc,name:"zipFile",main(e){const{zipFileName:t}=this.options;return async function(e,t){const r=new Wc;e.forEach((e=>r.file(e.name,e)));const n=await r.generateAsync({type:"blob"});return n.name=`${t}-${Qc++}.zip`,n}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */(kc(e,t),t)},options:e})},Combine:function(e,t=1e3,r){return xc({name:"Combine",main:r,operator(){return n=>n.pipe(Xc(e,t),Ha((e=>e.length)),Gu((e=>{const t=_a(new pc(e));return r instanceof Function?t.pipe(Gu((e=>this.TaskStarter(e)))):t})))}})},Break:function(){return xc({name:"Break",options:options,operator:()=>e=>e.pipe(Gu((e=>e instanceof pc?from(e.$destroy()):_a(e))))})}});
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */var Jc=["webkitStorageInfo","0","1","cookieStore","crossOriginIsolated","onbeforexrselect","ontransitioncancel","ontransitionrun","ontransitionstart","originAgentCluster","showDirectoryPicker","showOpenFilePicker","showSaveFilePicker","trustedTypes","parent","opener","top","length","frames","closed","location","self","window","document","name","customElements","history","locationbar","menubar","personalbar","scrollbars","statusbar","toolbar","status","frameElement","navigator","origin","external","screen","innerWidth","innerHeight","scrollX","pageXOffset","scrollY","pageYOffset","visualViewport","screenX","screenY","outerWidth","outerHeight","devicePixelRatio","clientInformation","screenLeft","screenTop","defaultStatus","defaultstatus","styleMedia","onsearch","isSecureContext","onabort","onblur","oncancel","oncanplay","oncanplaythrough","onchange","onclick","onclose","oncontextmenu","oncuechange","ondblclick","ondrag","ondragend","ondragenter","ondragleave","ondragover","ondragstart","ondrop","ondurationchange","onemptied","onended","onerror","onfocus","onformdata","oninput","oninvalid","onkeydown","onkeypress","onkeyup","onload","onloadeddata","onloadedmetadata","onloadstart","onmousedown","onmouseenter","onmouseleave","onmousemove","onmouseout","onmouseover","onmouseup","onmousewheel","onpause","onplay","onplaying","onprogress","onratechange","onreset","onresize","onscroll","onseeked","onseeking","onselect","onstalled","onsubmit","onsuspend","ontimeupdate","ontoggle","onvolumechange","onwaiting","onwebkitanimationend","onwebkitanimationiteration","onwebkitanimationstart","onwebkittransitionend","onwheel","onauxclick","ongotpointercapture","onlostpointercapture","onpointerdown","onpointermove","onpointerup","onpointercancel","onpointerover","onpointerout","onpointerenter","onpointerleave","onselectstart","onselectionchange","onanimationend","onanimationiteration","onanimationstart","ontransitionend","onafterprint","onbeforeprint","onbeforeunload","onhashchange","onlanguagechange","onmessage","onmessageerror","onoffline","ononline","onpagehide","onpageshow","onpopstate","onrejectionhandled","onstorage","onunhandledrejection","onunload","performance","stop","open","alert","confirm","prompt","print","queueMicrotask","requestAnimationFrame","cancelAnimationFrame","captureEvents","releaseEvents","requestIdleCallback","cancelIdleCallback","getComputedStyle","matchMedia","moveTo","moveBy","resizeTo","resizeBy","scroll","scrollTo","scrollBy","getSelection","find","webkitRequestAnimationFrame","webkitCancelAnimationFrame","fetch","btoa","atob","setTimeout","clearTimeout","setInterval","clearInterval","createImageBitmap","close","focus","blur","postMessage","onappinstalled","onbeforeinstallprompt","crypto","indexedDB","sessionStorage","localStorage","chrome","onpointerrawupdate","speechSynthesis","webkitRequestFileSystem","webkitResolveLocalFileSystemURL","openDatabase","applicationCache","caches","ondevicemotion","ondeviceorientation","ondeviceorientationabsolute"];
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
const el={RE:"",StringFunction(e,t,r,n){return this.RE.test(`${r}`)&&e.push(n),e},Array(e,t,r){const n=tl(Object.entries(r),this.RE),i=n.reduce(((e,[t,r])=>(e[parseInt(t)]=r,e)),[]);return n.length&&e.push([t,i]),e},Function(e,t,r){const n=tl(Object.entries(r),this.RE);return n.length&&e.push([t,Object.assign(r,Object.fromEntries(n))]),e},Object(e,t,r){const n=tl(Object.entries(r),this.RE);return n.length&&e.push([t,Object.fromEntries(n)]),e}};function tl(e,t,r=!1){return e.reduce(((e,n)=>{const[i,o]=n;if(t.test(i))return[...e,n];el.RE=t;const a=(s=o,Object.prototype.toString.call(s).match(/(?<=\[object\s+)\S+?(?=\])/)[0]);
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
var s;return el.hasOwnProperty(a)?el[a](e,i,o,n):r?[e,n]:e}),[])}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */el.Number=el.StringFunction,el.String=el.StringFunction;
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
const rl=/(https?|ftp|file):\/\/[-A-Za-z0-9+&@#/%?=~_|!:,.;]+[-A-Za-z0-9+&@#/%=~_|]/;const nl={Object({url:e="",name:t,way:r="npm",path:n="",version:i="",type:o="script"}){if(!e){const o=function({name:e,way:t,path:r,version:n}){return Uc.hasOwnProperty(e)?Uc[e]:zc(e,{version:n,store:t,path:r})}({name:t,way:r,path:n,version:i});if("string"!=typeof o)return il(o);e=o}return Rc[o](e)},String(e){return this.Object({[rl.test(e)?"url":"name"]:e})},Array:e=>Promise.all(e.map((e=>il(e))))};async function il(e){return nl[Fc(e)](e)}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
const ol={excel:{url:"/fake/excel",type:"get",template:{"data|100":[{ID:"@increment()",name:"@cname()",description:"@csentence()",avatar:'@dataImage("64x64")',address:"@region()",province:"@province()"}]}}};
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */const al=pu((async function(e){window.Mock||(await il("mockjs"),console.warn("Mockjs 载入并代理 Ajax 中")),console.warn("mock 启动后台 ",e);const{url:t,type:r,template:n}=ol[e];return window.Mock.mock(t,r,n),!0})),sl=["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","hash"],ul={strict:/^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,loose:/^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/};
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */function cl(e,t=!1){const r=ul[t?"strict":"loose"].exec(decodeURI(e)),n=sl.reduce(((e,t,n)=>(e[t]=r[n]||"",e)),{});var i;return n.queryParams=(i=n.query,String(i).split(/&|;/).reduce((function(e,t){try{t=decodeURIComponent(t.replace(/\+/g," "))}catch(e){}const[r,n]=t.split(/=/g);return e[r]=n,e}),{})),n.base=n.host?(n.protocol?n.protocol+"://"+n.host:n.host)+(n.port?":"+n.port:""):"",n}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */
var ll=Object.freeze({__proto__:null,$antiDebugger:function(){Function.prototype.$constructor||(Function.prototype.$constructor=Function.prototype.constructor,Function.prototype.constructor=function(...e){if(!e||"string"!=typeof e[0]||"debugger"!==e[0])return Function.prototype.$constructor.apply(this,e)})}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */,$copy:function(e,t=!1){t&&([document,...document.querySelectorAll("*")].forEach((e=>{e.oncontextmenu="",e.oncopy="",e.oncut="",e.onselectstart=!0;const t=window.getEventListeners(e);["keydown","copy","cut"].forEach((r=>{t.hasOwnProperty(r)&&t[r].forEach((t=>e.removeEventListener(r,t.listener)))}))})),console.log("copy 方式清理完成")),window.copy&&"function copy(value) { [Command Line API] }"===window.copy.toString()||delete window.copy,window.copy(e),console.log("copy 完成，请您查看您的剪贴版")},$GlobalVars:function(){const e=Iu(Object.keys(window),Jc);return Vu(window,e)},$search:function(e,t){var r;if(!ku(t)&&("string"==typeof(r=t)||!us(r)&&os(r)&&"[object String]"==is(r))&&(t=new RegExp(t)),e instanceof Array)return tl(Object.entries({i:e}),t)[0][1];if(e instanceof Object)return Object.fromEntries(tl(Object.entries(e),t));throw new Error("不是对象，不能搜索")},$load:il,$Mock:al,parseURL:function(e,t=!1){return cl(e,t)}});
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */class fl{constructor({forceRetry:e=!0,saveResult:t=!1,name:r=null,main:n,init:i=null,error:o=null,complete:a=null,options:s={},operator:u}){u&&(this.operator=u),Object.assign(this,{name:r,main:n,init:i,error:o,complete:a,options:s,saveResult:t,forceRetry:e})}initUUID(e){this.uuid=_c((this?.main||this.operator).toString()+e)}TaskStarter(e){return _a(e).pipe(ic((e=>_a(e).pipe(ja((e=>[e.$commit("start",this.uuid),e._originData])),ic((([e,t])=>{const r=this.main(e,t);return r instanceof Promise||r instanceof Jo?Na(r):_a(r)})),ja((t=>(e.$commit("success",t,this.uuid,this.saveResult),e)))))),$u(((...t)=>{if(this.error instanceof Function){const r=this.error(e,...t);if(r)throw new Pc(r);return da}throw new Pc(t[0])})),sc((e=>this.complete&&this.complete(e))))}operator(e){return Xo(ic((e=>this.TaskStarter(e))))}}return Object.assign(class{constructor({logEvery:e=!1}={}){this.config={logEvery:e},this.views=new jc(this.config)}crawl(...e){return Oc.createFlow(e.flat()),this}pipeline(...e){return Oc.pipeline(...e),this}start(){return Oc.startFlow(),this}stop(){return Oc.stopFlow(),this}},ll,{plugins:Zc,Plugin:function(e,t={}){if(e instanceof Function)return new fl(Object.assign(t,{main:e}));if(e instanceof Object)return new fl(e);throw new Pc("Plugin 必须是一个函数或者是 Plugin 描述对象")}
/**
   * @license
   * Copyright 2021 KonghaYao 江夏尧 <dongzhongzhidong@qq.com>
   * SPDX-License-Identifier: Apache-2.0
   */,Task:fc,TaskGroup:pc,version:"3.2.2",buildDate:new Date(1628382223946)})}));
